@@ -24,6 +24,7 @@
 #include "platform.h"
 #include "taler_merchantdb_lib.h"
 #include <microhttpd.h>
+#include <taler/taler_mhd_lib.h>
 
 /**
  * Shorthand for exit jumps.
@@ -41,11 +42,6 @@
 struct IterateInstancesCls
 {
 
-  /**
-   * Handle for the configuration beig parsed
-   */
-  const struct GNUNET_CONFIGURATION_Handle *config;
-
   /**
    * Current index in the global array of #MerchantInstance
    * types. Used by the callback in order to know which index
@@ -58,16 +54,11 @@ struct IterateInstancesCls
    */
   unsigned int default_instance;
 
-  /**
-   * Wire plugin
-   */
-  struct TALER_WIRE_Plugin *plugin;
-
   /**
    * Tells if the parsing encountered any error. We need this
    * field since the iterator must return void
    */
-   unsigned int ret;
+  unsigned int ret;
 };
 
 
@@ -89,7 +80,7 @@ struct WireMethod
   /**
    * Which wire method is @e j_wire using?  Points into @e j_wire.
    */
-  const char *wire_method;
+  char *wire_method;
 
   /**
    * Wire details for this instance
@@ -101,6 +92,11 @@ struct WireMethod
    */
   struct GNUNET_HashCode h_wire;
 
+  /**
+   * Is this wire method active (should it be included in new contracts)?
+   */
+  int active;
+
 };
 
 
@@ -203,13 +199,15 @@ struct TMH_RequestHandler
    * @param[in,out] connection_cls the connection's closure (can be updated)
    * @param upload_data upload data
    * @param[in,out] upload_data_size number of bytes (left) in @a upload_data
+   * @param mi merchant backend instance, never NULL
    * @return MHD result code
    */
   int (*handler)(struct TMH_RequestHandler *rh,
                  struct MHD_Connection *connection,
                  void **connection_cls,
                  const char *upload_data,
-                 size_t *upload_data_size);
+                 size_t *upload_data_size,
+                 struct MerchantInstance *mi);
 
   /**
    * Default response code.
@@ -257,6 +255,39 @@ struct TM_HandlerContext
    */
   const struct TMH_RequestHandler *rh;
 
+  /**
+   * Asynchronous request context id.
+   */
+  struct GNUNET_AsyncScopeId async_scope_id;
+};
+
+
+/**
+ * Entry in a #resume_timeout_heap.
+ */
+struct TMH_SuspendedConnection
+{
+  /**
+   * Which connection was suspended.
+   */
+  struct MHD_Connection *con;
+
+  /**
+   * Associated heap node.
+   */
+  struct GNUNET_CONTAINER_HeapNode *hn;
+
+  /**
+   * Key of this entry in the #payment_trigger_map
+   */
+  struct GNUNET_HashCode key;
+
+  /**
+   * At what time does this request expire? If set in the future, we
+   * may wait this long for a payment to arrive before responding.
+   */
+  struct GNUNET_TIME_Absolute long_poll_timeout;
+
 };
 
 
@@ -284,6 +315,33 @@ extern struct TALER_Amount default_max_deposit_fee;
  */
 extern unsigned long long default_wire_fee_amortization;
 
+/**
+ * MIN-Heap of suspended connections to resume when the timeout expires,
+ * ordered by timeout. Values are of type `struct TMH_SuspendedConnection`
+ */
+extern struct GNUNET_CONTAINER_Heap *resume_timeout_heap;
+
+/**
+ * Task responsible for timeouts in the #resume_timeout_heap.
+ */
+extern struct GNUNET_SCHEDULER_Task *resume_timeout_task;
+
+/**
+ * Hash map from H(order_id,merchant_pub) to `struct TMH_SuspendedConnection`
+ * entries to resume when a payment is made for the given order.
+ */
+extern struct GNUNET_CONTAINER_MultiHashMap *payment_trigger_map;
+
+/**
+ * Which currency do we use?
+ */
+extern char *TMH_currency;
+
+/**
+ * Inform the auditor for all deposit confirmations (global option)
+ */
+extern int TMH_force_audit;
+
 /**
  * Hash of our wire format details as given in #j_wire.
  */
@@ -299,11 +357,6 @@ extern struct TALER_MerchantPrivateKeyP privkey;
  */
 extern struct TALER_MerchantPublicKeyP pubkey;
 
-/**
- * Should a "Connection: close" header be added to each HTTP response?
- */
-extern int TMH_merchant_connection_close;
-
 /**
  * Handle to the database backend.
  */
@@ -315,7 +368,7 @@ extern struct TALER_MERCHANTDB_Plugin *db;
  * executing the wire transfer?  This delay is added to the current
  * time when we generate the advisory execution time for the exchange.
  */
-extern struct GNUNET_TIME_Relative wire_transfer_delay;
+extern struct GNUNET_TIME_Relative default_wire_transfer_delay;
 
 /**
  * If the frontend does NOT specify a payment deadline, how long should
@@ -332,31 +385,45 @@ extern struct GNUNET_TIME_Relative default_pay_deadline;
 void
 TMH_trigger_daemon (void);
 
+
 /**
- * Lookup a merchant instance by its name.
+ * Compute @a key to use for @a order_id and @a mpub in our
+ * #payment_trigger_map.
  *
- * @param name name of the instance to resolve
- * @return NULL if that instance is unknown to us
+ * @param order_id an order ID
+ * @param mpub an instance public key
+ * @param key[out] set to the hash map key to use
  */
-struct MerchantInstance *
-TMH_lookup_instance (const char *name);
+void
+TMH_compute_pay_key (const char *order_id,
+                     const struct TALER_MerchantPublicKeyP *mpub,
+                     struct GNUNET_HashCode *key);
 
 
 /**
- * Extract merchant instance from the given JSON
+ * Suspend connection from @a sc until payment has been received.
  *
- * @param json the JSON to inspect; it is not required to
- * comply with any particular format. It will only be checked
- * if the field "instance" is there.
- * @return a pointer to a #struct MerchantInstance. This will be
- * the 'default' merchant if the frontend did not specif any
- * "instance" field. The user should not care to free the returned
- * value, as it is taken from a global array that will be freed
- * by the general shutdown routine. NULL if the frontend specified
- * a wrong instance
+ * @param sc connection to suspend
  */
-struct MerchantInstance *
-TMH_lookup_instance_json (struct json_t *json);
+void
+TMH_long_poll_suspend (struct TMH_SuspendedConnection *sc);
+
+
+/**
+ * Create a taler://pay/ URI for the given @a con and @a order_id
+ * and @a session_id and @a instance_id.
+ *
+ * @param con HTTP connection
+ * @param order_id the order id
+ * @param session_id session, may be NULL
+ * @param instance_id instance, may be "default"
+ * @return corresponding taler://pay/ URI, or NULL on missing "host"
+ */
+char *
+TMH_make_taler_pay_uri (struct MHD_Connection *con,
+                        const char *order_id,
+                        const char *session_id,
+                        const char *instance_id);
 
 
 #endif
