@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  (C) 2014-2017 INRIA
+  (C) 2014-2018 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU Affero General Public License as published by the Free Software
@@ -24,7 +24,6 @@
 #include "taler-merchant-httpd_exchanges.h"
 
 
-
 /**
  * Delay after which we'll re-fetch key information from the exchange.
  */
@@ -33,7 +32,8 @@
 /**
  * Threshold after which exponential backoff should not increase.
  */
-#define RETRY_BACKOFF_THRESHOLD GNUNET_TIME_relative_multiply (GNUNET_TIME_UNIT_SECONDS, 60)
+#define RETRY_BACKOFF_THRESHOLD GNUNET_TIME_relative_multiply ( \
+    GNUNET_TIME_UNIT_SECONDS, 60)
 
 
 /**
@@ -43,7 +43,10 @@
  * @param r current backoff time, initially zero
  */
 #define RETRY_BACKOFF(r) GNUNET_TIME_relative_min (RETRY_BACKOFF_THRESHOLD, \
-   GNUNET_TIME_relative_multiply (GNUNET_TIME_relative_max (GNUNET_TIME_UNIT_MILLISECONDS, (r)), 2));
+                                                   GNUNET_TIME_relative_multiply ( \
+                                                     GNUNET_TIME_relative_max ( \
+                                                       GNUNET_TIME_UNIT_MILLISECONDS, \
+                                                       (r)), 2));
 
 
 /**
@@ -260,7 +263,7 @@ json_t *trusted_exchanges;
 static void
 keys_mgmt_cb (void *cls,
               const struct TALER_EXCHANGE_Keys *keys,
-	      enum TALER_EXCHANGE_VersionCompatibility compat);
+              enum TALER_EXCHANGE_VersionCompatibility compat);
 
 
 /**
@@ -302,22 +305,18 @@ retry_exchange (void *cls)
  * @param cls closure
  * @param wire_method name of the wire method (i.e. "sepa")
  * @param fees fee structure for this method
+ * @return #GNUNET_OK on success
  */
-static void
-process_wire_fees (void *cls,
+static int
+process_wire_fees (struct Exchange *exchange,
+                   const struct TALER_MasterPublicKeyP *master_pub,
                    const char *wire_method,
                    const struct TALER_EXCHANGE_WireAggregateFees *fees)
 {
-  struct Exchange *exchange = cls;
   struct FeesByWireMethod *f;
   struct TALER_EXCHANGE_WireAggregateFees *endp;
   struct TALER_EXCHANGE_WireAggregateFees *af;
-  const struct TALER_EXCHANGE_Keys *keys;
-  const struct TALER_MasterPublicKeyP *master_pub;
 
-  keys = TALER_EXCHANGE_get_keys (exchange->conn);
-  GNUNET_assert (NULL != keys);
-  master_pub = &keys->master_pub;
   for (f = exchange->wire_fees_head; NULL != f; f = f->next)
     if (0 == strcasecmp (wire_method,
                          f->wire_method))
@@ -342,7 +341,7 @@ process_wire_fees (void *cls,
   {
     /* Hole in the fee structure, not allowed! */
     GNUNET_break_op (0);
-    return;
+    return GNUNET_SYSERR;
   }
   while (NULL != fees)
   {
@@ -352,36 +351,36 @@ process_wire_fees (void *cls,
     af = GNUNET_new (struct TALER_EXCHANGE_WireAggregateFees);
     *af = *fees;
     GNUNET_CRYPTO_hash (wire_method,
-			strlen (wire_method) + 1,
-			&h_wire_method);
+                        strlen (wire_method) + 1,
+                        &h_wire_method);
     GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
-		"Storing wire fee for `%s' and method `%s' at %s in DB; the fee is %s\n",
-		TALER_B2S (master_pub),
-		wire_method,
-		GNUNET_STRINGS_absolute_time_to_string (af->start_date),
-		TALER_amount2s (&af->wire_fee));
+                "Storing wire fee for `%s' and method `%s' at %s in DB; the fee is %s\n",
+                TALER_B2S (master_pub),
+                wire_method,
+                GNUNET_STRINGS_absolute_time_to_string (af->start_date),
+                TALER_amount2s (&af->wire_fee));
     db->preflight (db->cls);
     if (GNUNET_OK !=
         db->start (db->cls,
                    "store wire fee"))
     {
       GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-		  "Failed to start database transaction!\n");
+                  "Failed to start database transaction!\n");
       GNUNET_free (af);
-      break;
+      return GNUNET_SYSERR;
     }
     qs = db->store_wire_fee_by_exchange (db->cls,
-					 master_pub,
-					 &h_wire_method,
-					 &af->wire_fee,
-					 &af->closing_fee,
-					 af->start_date,
-					 af->end_date,
-					 &af->master_sig);
+                                         master_pub,
+                                         &h_wire_method,
+                                         &af->wire_fee,
+                                         &af->closing_fee,
+                                         af->start_date,
+                                         af->end_date,
+                                         &af->master_sig);
     if (0 > qs)
     {
       GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-		  "Failed to persist exchange wire fees in merchant DB!\n");
+                  "Failed to persist exchange wire fees in merchant DB!\n");
       GNUNET_free (af);
       fees = fees->next;
       db->rollback (db->cls);
@@ -391,7 +390,7 @@ process_wire_fees (void *cls,
     {
       /* Entry was already in DB, fine, continue as if we had succeeded */
       GNUNET_log (GNUNET_ERROR_TYPE_INFO,
-		  "Fees already in DB, rolling back transaction attempt!\n");
+                  "Fees already in DB, rolling back transaction attempt!\n");
       db->rollback (db->cls);
     }
     if (0 < qs)
@@ -416,6 +415,46 @@ process_wire_fees (void *cls,
     endp = af;
     fees = fees->next;
   }
+  return GNUNET_OK;
+}
+
+
+/**
+ * Function called with information about the wire accounts
+ * of the exchanage.  Stores the wire fees with the
+ * exchange for laster use.
+ *
+ * @param exchange the exchange
+ * @param master_pub public key of the exchange
+ * @param accounts_len length of the @a accounts array
+ * @param accounts list of wire accounts of the exchange
+ * @return #GNUNET_OK on success
+ */
+static int
+process_wire_accounts (struct Exchange *exchange,
+                       const struct TALER_MasterPublicKeyP *master_pub,
+                       unsigned int accounts_len,
+                       const struct TALER_EXCHANGE_WireAccount *accounts)
+{
+  for (unsigned int i = 0; i<accounts_len; i++)
+  {
+    char *method;
+
+    method = TALER_payto_get_method (accounts[i].payto_uri);
+    if (NULL == method)
+      return GNUNET_SYSERR;
+    if (GNUNET_OK !=
+        process_wire_fees (exchange,
+                           master_pub,
+                           method,
+                           accounts[i].fees))
+    {
+      GNUNET_free (method);
+      return GNUNET_SYSERR;
+    }
+    GNUNET_free (method);
+  }
+  return GNUNET_OK;
 }
 
 
@@ -495,8 +534,9 @@ process_find_operations (struct Exchange *exchange)
         /* Disagreement on the current time */
         GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                     "Exchange's earliest fee is %s adhead of our time. Clock skew issue?\n",
-                    GNUNET_STRINGS_relative_time_to_string (GNUNET_TIME_absolute_get_remaining (af->start_date),
-                                                            GNUNET_YES));
+                    GNUNET_STRINGS_relative_time_to_string (
+                      GNUNET_TIME_absolute_get_remaining (af->start_date),
+                      GNUNET_YES));
         continue;
       }
       /* found fee, great! */
@@ -549,15 +589,15 @@ wire_task_cb (void *cls);
  * @param http_status HTTP response code, #MHD_HTTP_OK (200) for successful request;
  *                    0 if the exchange's reply is bogus (fails to follow the protocol)
  * @param ec taler-specific error code, #TALER_EC_NONE on success
- * @param obj the received JSON reply, if successful this should be the wire
- *            format details as provided by /wire, or NULL if the
- *            reply was not in JSON format.
+ * @param accounts_len length of the @a accounts array
+ * @param accounts list of wire accounts of the exchange, NULL on error
  */
 static void
 handle_wire_data (void *cls,
                   unsigned int http_status,
                   enum TALER_ErrorCode ec,
-                  const json_t *obj)
+                  unsigned int accounts_len,
+                  const struct TALER_EXCHANGE_WireAccount *accounts)
 {
   struct Exchange *exchange = cls;
   const struct TALER_EXCHANGE_Keys *keys;
@@ -574,10 +614,10 @@ handle_wire_data (void *cls,
   keys = TALER_EXCHANGE_get_keys (exchange->conn);
   if ( (NULL == keys) ||
        (GNUNET_OK !=
-        TALER_EXCHANGE_wire_get_fees (&keys->master_pub,
-                                      obj,
-                                      &process_wire_fees,
-                                      exchange)) )
+        process_wire_accounts (exchange,
+                               &keys->master_pub,
+                               accounts_len,
+                               accounts)) )
   {
     /* Report hard failure to all callbacks! */
     struct TMH_EXCHANGES_FindOperation *fo;
@@ -614,9 +654,10 @@ handle_wire_data (void *cls,
                 "Do not have sufficient wire data. Will re-request /wire in 1 minute\n");
 
     GNUNET_assert (NULL == exchange->wire_task);
-    exchange->wire_task = GNUNET_SCHEDULER_add_delayed (GNUNET_TIME_UNIT_MINUTES,
-                                                        &wire_task_cb,
-                                                        exchange);
+    exchange->wire_task = GNUNET_SCHEDULER_add_delayed (
+      GNUNET_TIME_UNIT_MINUTES,
+      &wire_task_cb,
+      exchange);
   }
 }
 
@@ -663,7 +704,7 @@ wire_task_cb (void *cls)
 static void
 keys_mgmt_cb (void *cls,
               const struct TALER_EXCHANGE_Keys *keys,
-	      enum TALER_EXCHANGE_VersionCompatibility compat)
+              enum TALER_EXCHANGE_VersionCompatibility compat)
 {
   struct Exchange *exchange = cls;
   struct GNUNET_TIME_Absolute expire;
@@ -686,8 +727,8 @@ keys_mgmt_cb (void *cls,
     {
       /* Give up, log hard error. */
       GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-		  "Exchange `%s' runs an incompatible more recent version of the Taler protocol. Will not retry. This client may need to be updated.\n",
-		  exchange->url);
+                  "Exchange `%s' runs an incompatible more recent version of the Taler protocol. Will not retry. This client may need to be updated.\n",
+                  exchange->url);
       return;
     }
     exchange->retry_delay = RETRY_BACKOFF (exchange->retry_delay);
@@ -711,11 +752,12 @@ keys_mgmt_cb (void *cls,
     {
       once = 1;
       GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
-		  "Exchange `%s' runs a more recent version of the Taler protocol. You may want to update this client.\n",
-		  exchange->url);
+                  "Exchange `%s' runs a more recent version of the Taler protocol. You may want to update this client.\n",
+                  exchange->url);
     }
   }
   expire = TALER_EXCHANGE_check_keys_current (exchange->conn,
+                                              GNUNET_NO,
                                               GNUNET_NO);
   if (0 == expire.abs_value_us)
     delay = RELOAD_DELAY;
@@ -786,8 +828,8 @@ return_result (void *cls)
 struct TMH_EXCHANGES_FindOperation *
 TMH_EXCHANGES_find_exchange (const char *chosen_exchange,
                              const char *wire_method,
-			     TMH_EXCHANGES_FindContinuation fc,
-			     void *fc_cls)
+                             TMH_EXCHANGES_FindContinuation fc,
+                             void *fc_cls)
 {
   struct Exchange *exchange;
   struct TMH_EXCHANGES_FindOperation *fo;
@@ -805,8 +847,8 @@ TMH_EXCHANGES_find_exchange (const char *chosen_exchange,
 
   /* Check if the exchange is known */
   for (exchange = exchange_head; NULL != exchange; exchange = exchange->next)
-    /* test it by checking public key --- FIXME: hostname or public key!?
-       Should probably be URL, not hostname anyway! */
+  /* test it by checking public key --- FIXME: hostname or public key!?
+     Should probably be URL, not hostname anyway! */
   {
     if (0 == strcmp (exchange->url,
                      chosen_exchange))
@@ -896,7 +938,7 @@ TMH_EXCHANGES_find_exchange_cancel (struct TMH_EXCHANGES_FindOperation *fo)
   {
     GNUNET_SCHEDULER_cancel (fo->at);
     fo->at = NULL;
-   }
+  }
   GNUNET_CONTAINER_DLL_remove (exchange->fo_head,
                                exchange->fo_tail,
                                fo);
@@ -929,12 +971,12 @@ accept_exchanges (void *cls,
   if (GNUNET_OK !=
       GNUNET_CONFIGURATION_get_value_string (cfg,
                                              section,
-                                             "URL",
+                                             "BASE_URL",
                                              &url))
   {
     GNUNET_log_config_missing (GNUNET_ERROR_TYPE_ERROR,
                                section,
-                               "URL");
+                               "BASE_URL");
     return;
   }
   exchange = GNUNET_new (struct Exchange);
@@ -948,7 +990,8 @@ accept_exchanges (void *cls,
     if (GNUNET_OK ==
         GNUNET_CRYPTO_eddsa_public_key_from_string (mks,
                                                     strlen (mks),
-                                                    &exchange->master_pub.eddsa_pub))
+                                                    &exchange->master_pub.
+                                                    eddsa_pub))
     {
       exchange->trusted = GNUNET_YES;
     }
@@ -957,7 +1000,7 @@ accept_exchanges (void *cls,
       GNUNET_log_config_invalid (GNUNET_ERROR_TYPE_ERROR,
                                  section,
                                  "MASTER_KEY",
-                                 _("ill-formed key"));
+                                 _ ("ill-formed key"));
     }
     GNUNET_free (mks);
   }
@@ -996,6 +1039,8 @@ TMH_EXCHANGES_init (const struct GNUNET_CONFIGURATION_Handle *cfg)
     GNUNET_break (0);
     return GNUNET_SYSERR;
   }
+  GNUNET_CURL_enable_async_scope_header (merchant_curl_ctx,
+                                         "Taler-Correlation-Id");
   merchant_curl_rc = GNUNET_CURL_gnunet_rc_create (merchant_curl_ctx);
   /* get exchanges from the merchant configuration and try to connect to them */
   GNUNET_CONFIGURATION_iterate_sections (cfg,
@@ -1013,7 +1058,8 @@ TMH_EXCHANGES_init (const struct GNUNET_CONFIGURATION_Handle *cfg)
       continue;
     j_exchange = json_pack ("{s:s, s:o}",
                             "url", exchange->url,
-                            "master_pub", GNUNET_JSON_from_data_auto (&exchange->master_pub));
+                            "master_pub", GNUNET_JSON_from_data_auto (
+                              &exchange->master_pub));
     GNUNET_assert (0 ==
                    json_array_append_new (trusted_exchanges,
                                           j_exchange));
@@ -1083,4 +1129,5 @@ TMH_EXCHANGES_done ()
   trusted_exchanges = NULL;
 }
 
+
 /* end of taler-merchant-httpd_exchanges.c */
