@@ -23,7 +23,6 @@
 #include <taler/taler_signatures.h>
 #include <taler/taler_json_lib.h>
 #include "taler-merchant-httpd.h"
-#include "taler-merchant-httpd_responses.h"
 
 
 /**
@@ -88,17 +87,28 @@ pd_cb (void *cls,
   if (NULL == summary)
     summary = json_string (order_id);
 
-  GNUNET_break (NULL != (entry = json_pack ("{s:I, s:s, s:O, s:O, s:O, s:O}",
-                                            "row_id", row_id,
-                                            "order_id", order_id,
-                                            "amount", amount,
-                                            "timestamp", timestamp,
-                                            "instance", instance,
-                                            "summary", summary)));
-
-  GNUNET_break (0 ==
-                json_array_append_new (pcc->response,
-                                       entry));
+  if (NULL == (entry =
+                 json_pack ("{s:I, s:s, s:O, s:O, s:O, s:O}",
+                            "row_id", row_id,
+                            "order_id", order_id,
+                            "amount", amount,
+                            "timestamp", timestamp,
+                            "instance", instance,
+                            "summary", summary)))
+  {
+    GNUNET_break (0);
+    pcc->failure = GNUNET_SYSERR;
+    return;
+  }
+  if (0 !=
+      json_array_append_new (pcc->response,
+                             entry))
+  {
+    GNUNET_break (0);
+    pcc->failure = GNUNET_SYSERR;
+    json_decref (entry);
+    return;
+  }
 }
 
 
@@ -111,6 +121,7 @@ pd_cb (void *cls,
  * @param[in,out] connection_cls the connection's closure (can be updated)
  * @param upload_data upload data
  * @param[in,out] upload_data_size number of bytes (left) in @a upload_data
+ * @param mi merchant backend instance, never NULL
  * @return MHD result code
  */
 int
@@ -118,17 +129,18 @@ MH_handler_history (struct TMH_RequestHandler *rh,
                     struct MHD_Connection *connection,
                     void **connection_cls,
                     const char *upload_data,
-                    size_t *upload_data_size)
+                    size_t *upload_data_size,
+                    struct MerchantInstance *mi)
 {
   #define LOG_INFO(...) GNUNET_log (GNUNET_ERROR_TYPE_INFO, __VA_ARGS__)
   const char *str;
   struct GNUNET_TIME_Absolute date;
   json_t *response;
   int ret;
+  unsigned int ascending = GNUNET_NO;
   unsigned long long seconds;
-  struct MerchantInstance *mi;
-  int start = -1;
-  unsigned int delta;
+  unsigned long long start = INT64_MAX;
+  long long delta = -20;
   enum GNUNET_DB_QueryStatus qs;
   struct ProcessContractClosure pcc;
 
@@ -142,39 +154,27 @@ MH_handler_history (struct TMH_RequestHandler *rh,
   if (NULL != str)
   {
     if (1 != sscanf (str,
-		     "%llu",
-		     &seconds))
+                     "%llu",
+                     &seconds))
     {
       json_decref (response);
-      return TMH_RESPONSE_reply_arg_invalid (connection,
-                                             TALER_EC_PARAMETER_MALFORMED,
-                                             "date");
+      return TALER_MHD_reply_with_error (connection,
+                                         MHD_HTTP_BAD_REQUEST,
+                                         TALER_EC_PARAMETER_MALFORMED,
+                                         "date");
     }
-
     date.abs_value_us = seconds * 1000LL * 1000LL;
-
     if (date.abs_value_us / 1000LL / 1000LL != seconds)
     {
       json_decref (response);
-      return TMH_RESPONSE_reply_bad_request (connection,
-                                             TALER_EC_HISTORY_TIMESTAMP_OVERFLOW,
-                                             "Timestamp overflowed");
+      return TALER_MHD_reply_with_error (connection,
+                                         MHD_HTTP_BAD_REQUEST,
+                                         TALER_EC_HISTORY_TIMESTAMP_OVERFLOW,
+                                         "Timestamp overflowed");
     }
   }
 
-  str = MHD_lookup_connection_value (connection,
-                                     MHD_GET_ARGUMENT_KIND,
-                                     "instance");
-  mi = TMH_lookup_instance (NULL != str ? str : "default");
-
-  if (NULL == mi)
-  {
-    json_decref (response);
-    return TMH_RESPONSE_reply_not_found (connection,
-                                         TALER_EC_HISTORY_INSTANCE_UNKNOWN,
-                                         "instance");
-  }
-
+  /* Sanity check that we don't have some odd stale transaction running */
   db->preflight (db->cls);
 
   /* Here goes the cherry-picking logic */
@@ -186,10 +186,10 @@ MH_handler_history (struct TMH_RequestHandler *rh,
     pcc.response = response;
     pcc.failure = GNUNET_NO;
     qs = db->find_contract_terms_history (db->cls,
-					  str,
-					  &mi->pubkey,
-					  &pd_cb,
-					  &pcc);
+                                          str,
+                                          &mi->pubkey,
+                                          &pd_cb,
+                                          &pcc);
     /* single, read-only SQL statements should never cause
        serialization problems */
     GNUNET_break (GNUNET_DB_STATUS_SOFT_ERROR != qs);
@@ -199,32 +199,34 @@ MH_handler_history (struct TMH_RequestHandler *rh,
          (GNUNET_SYSERR == pcc.failure) )
     {
       json_decref (response);
-      return TMH_RESPONSE_reply_internal_error (connection,
-                                                TALER_EC_HISTORY_DB_FETCH_ERROR,
-					        "db error to get history");
+      return TALER_MHD_reply_with_error (connection,
+                                         MHD_HTTP_INTERNAL_SERVER_ERROR,
+                                         TALER_EC_HISTORY_DB_FETCH_ERROR,
+                                         "db error to get history");
     }
-    ret = TMH_RESPONSE_reply_json (connection,
-                                   response,
-                                   MHD_HTTP_OK);
+    ret = TALER_MHD_reply_json (connection,
+                                response,
+                                MHD_HTTP_OK);
     json_decref (response);
     return ret;
   }
 
-  delta = 20;
   str = MHD_lookup_connection_value (connection,
                                      MHD_GET_ARGUMENT_KIND,
                                      "start");
   if (NULL != str)
   {
-    if ( (1 != sscanf (str,
-		       "%d",
-		       &start)) ||
-         (0 > start) )
+    TALER_LOG_DEBUG ("'start' argument given ('%s')\n",
+                     str);
+    if (1 != sscanf (str,
+                     "%llu",
+                     &start))
     {
       json_decref (response);
-      return TMH_RESPONSE_reply_arg_invalid (connection,
-                                             TALER_EC_PARAMETER_MALFORMED,
-                                             "start");
+      return TALER_MHD_reply_with_error (connection,
+                                         MHD_HTTP_BAD_REQUEST,
+                                         TALER_EC_PARAMETER_MALFORMED,
+                                         "start");
     }
   }
 
@@ -235,36 +237,40 @@ MH_handler_history (struct TMH_RequestHandler *rh,
   if (NULL != str)
   {
     if (1 != sscanf (str,
-		     "%u",
-		     &delta))
-      return TMH_RESPONSE_reply_arg_invalid (connection,
-                                             TALER_EC_PARAMETER_MALFORMED,
-                                             "delta");
+                     "%lld",
+                     &delta))
+      return TALER_MHD_reply_with_error (connection,
+                                         MHD_HTTP_BAD_REQUEST,
+                                         TALER_EC_PARAMETER_MALFORMED,
+                                         "delta");
   }
   GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
-              "Querying history back to %s, start: %u, delta: %u\n",
+              "Querying history back to %s, start: %llu, delta: %lld\n",
               GNUNET_STRINGS_absolute_time_to_string (date),
               start,
               delta);
 
   pcc.response = response;
   pcc.failure = GNUNET_NO;
-  if (0 > start)
-    qs = db->find_contract_terms_by_date (db->cls,
-					  date,
-					  &mi->pubkey,
-					  delta,
-					  &pd_cb,
-					  &pcc);
-  else
-    qs = db->find_contract_terms_by_date_and_range (db->cls,
-						    date,
-						    &mi->pubkey,
-						    (unsigned int) start,
-						    delta,
-						    GNUNET_NO,
-						    &pd_cb,
-						    &pcc);
+
+  str = MHD_lookup_connection_value (connection,
+                                     MHD_GET_ARGUMENT_KIND,
+                                     "ordering");
+  if ( (NULL != str) &&
+       (0 == strcmp ("ascending",
+                     str)) )
+    ascending = GNUNET_YES;
+
+  qs = db->find_contract_terms_by_date_and_range (db->cls,
+                                                  date,
+                                                  &mi->pubkey,
+                                                  start,
+                                                  llabs (delta),
+                                                  (delta < 0) ? GNUNET_YES :
+                                                  GNUNET_NO,
+                                                  ascending,
+                                                  &pd_cb,
+                                                  &pcc);
   if ( (0 > qs) ||
        (GNUNET_SYSERR == pcc.failure) )
   {
@@ -274,16 +280,20 @@ MH_handler_history (struct TMH_RequestHandler *rh,
     /* Always report on hard error as well to enable diagnostics */
     GNUNET_break (GNUNET_DB_STATUS_HARD_ERROR == qs);
     json_decref (response);
-    return TMH_RESPONSE_reply_internal_error (connection,
-					      TALER_EC_HISTORY_DB_FETCH_ERROR,
-					      "db error to get history");
+    return TALER_MHD_reply_with_error (connection,
+                                       MHD_HTTP_INTERNAL_SERVER_ERROR,
+                                       TALER_EC_HISTORY_DB_FETCH_ERROR,
+                                       "db error to get history");
   }
-  ret = TMH_RESPONSE_reply_json (connection,
-                                 response,
-                                 MHD_HTTP_OK);
-  LOG_INFO ("/history, http code: %d\n", MHD_HTTP_OK);
-  json_decref (response);
+  ret = TALER_MHD_reply_json_pack (connection,
+                                   MHD_HTTP_OK,
+                                   "{ s:o }",
+                                   "history",
+                                   response /* consumes 'response' */);
+  LOG_INFO ("/history, http code: %d\n",
+            MHD_HTTP_OK);
   return ret;
 }
 
+
 /* end of taler-merchant-httpd_history.c */
