@@ -37,6 +37,7 @@
  * @param[in,out] connection_cls the connection's closure (can be updated)
  * @param upload_data upload data
  * @param[in,out] upload_data_size number of bytes (left) in @a upload_data
+ * @param mi merchant backend instance, NULL is allowed in this case!
  * @return MHD result code
  */
 int
@@ -44,7 +45,8 @@ TMH_MHD_handler_static_response (struct TMH_RequestHandler *rh,
                                  struct MHD_Connection *connection,
                                  void **connection_cls,
                                  const char *upload_data,
-                                 size_t *upload_data_size);
+                                 size_t *upload_data_size,
+                                 struct MerchantInstance *mi);
 
 
 /**
@@ -56,6 +58,7 @@ TMH_MHD_handler_static_response (struct TMH_RequestHandler *rh,
  * @param[in,out] connection_cls the connection's closure (can be updated)
  * @param upload_data upload data
  * @param[in,out] upload_data_size number of bytes (left) in @a upload_data
+ * @param mi merchant backend instance, never NULL
  * @return MHD result code
  */
 int
@@ -63,7 +66,8 @@ TMH_MHD_handler_agpl_redirect (struct TMH_RequestHandler *rh,
                                struct MHD_Connection *connection,
                                void **connection_cls,
                                const char *upload_data,
-                               size_t *upload_data_size);
+                               size_t *upload_data_size,
+                               struct MerchantInstance *mi);
 
 
 /**
@@ -98,6 +102,7 @@ TMH_MHD_helper_send_json_pack (struct TMH_RequestHandler *rh,
  * @param[in,out] connection_cls the connection's closure (can be updated)
  * @param upload_data upload data
  * @param[in,out] upload_data_size number of bytes (left) in @a upload_data
+ * @param mi merchant backend instance, never NULL
  * @return MHD result code
  */
 int
@@ -105,7 +110,8 @@ TMH_MHD_handler_send_json_pack_error (struct TMH_RequestHandler *rh,
                                       struct MHD_Connection *connection,
                                       void **connection_cls,
                                       const char *upload_data,
-                                      size_t *upload_data_size);
+                                      size_t *upload_data_size,
+                                      struct MerchantInstance *mi);
 
 
 #endif
