@@ -23,8 +23,6 @@
 #include <taler/taler_signatures.h>
 #include <taler/taler_json_lib.h>
 #include "taler-merchant-httpd.h"
-#include "taler-merchant-httpd_parsing.h"
-#include "taler-merchant-httpd_responses.h"
 #include "taler-merchant-httpd_refund.h"
 
 /**
@@ -49,7 +47,7 @@ struct ProcessRefundData
   /**
    * Both public and private key are needed by the callback
    */
-   const struct MerchantInstance *merchant;
+  const struct MerchantInstance *merchant;
 
   /**
    * Return code: #TALER_EC_NONE if successful.
@@ -71,12 +69,74 @@ struct TMH_JsonParseContext
   struct TM_HandlerContext hc;
 
   /**
-   * Placeholder for #TMH_PARSE_post_json() to keep its internal state.
+   * Placeholder for #TALER_MHD_parse_post_json() to keep its internal state.
    */
   void *json_parse_context;
 };
 
 
+/**
+ * Make a taler://refund URI
+ *
+ * @param connection MHD connection to take host and path from
+ * @param instance_id merchant's instance ID
+ * @param order_id order ID to show a refund for
+ * @returns the URI, must be freed with #GNUNET_free
+ */
+static char *
+make_taler_refund_uri (struct MHD_Connection *connection,
+                       const char *instance_id,
+                       const char *order_id)
+{
+  const char *host;
+  const char *forwarded_host;
+  const char *uri_path;
+  const char *uri_instance_id;
+  const char *query;
+  char *result;
+
+  host = MHD_lookup_connection_value (connection, MHD_HEADER_KIND, "Host");
+  forwarded_host = MHD_lookup_connection_value (connection, MHD_HEADER_KIND,
+                                                "X-Forwarded-Host");
+
+  uri_path = MHD_lookup_connection_value (connection, MHD_HEADER_KIND,
+                                          "X-Forwarded-Prefix");
+  if (NULL == uri_path)
+    uri_path = "-";
+
+  if (NULL != forwarded_host)
+    host = forwarded_host;
+
+  if (0 == strcmp (instance_id, "default"))
+    uri_instance_id = "-";
+  else
+    uri_instance_id = instance_id;
+
+  if (NULL == host)
+  {
+    /* Should never happen, at least the host header should be defined */
+    GNUNET_break (0);
+    return NULL;
+  }
+
+  if (GNUNET_YES == TALER_mhd_is_https (connection))
+    query = "";
+  else
+    query = "?insecure=1";
+
+  GNUNET_assert (NULL != order_id);
+
+  GNUNET_assert (0 < GNUNET_asprintf (&result,
+                                      "taler://refund/%s/%s/%s/%s%s",
+                                      host,
+                                      uri_path,
+                                      uri_instance_id,
+                                      order_id,
+                                      query));
+  return result;
+}
+
+
 /**
  * Custom cleanup routine for a `struct TMH_JsonParseContext`.
  *
@@ -87,7 +147,7 @@ json_parse_cleanup (struct TM_HandlerContext *hc)
 {
   struct TMH_JsonParseContext *jpc = (struct TMH_JsonParseContext *) hc;
 
-  TMH_PARSE_post_cleanup_callback (jpc->json_parse_context);
+  TALER_MHD_parse_post_cleanup_callback (jpc->json_parse_context);
   GNUNET_free (jpc);
 }
 
@@ -100,6 +160,7 @@ json_parse_cleanup (struct TM_HandlerContext *hc)
  * @param[in,out] connection_cls the connection's closure (can be updated)
  * @param upload_data upload data
  * @param[in,out] upload_data_size number of bytes (left) in @a upload_data
+ * @param mi merchant backend instance, never NULL
  * @return MHD result code
  */
 int
@@ -107,7 +168,8 @@ MH_handler_refund_increase (struct TMH_RequestHandler *rh,
                             struct MHD_Connection *connection,
                             void **connection_cls,
                             const char *upload_data,
-                            size_t *upload_data_size)
+                            size_t *upload_data_size,
+                            struct MerchantInstance *mi)
 {
   int res;
   struct TMH_JsonParseContext *ctx;
@@ -116,9 +178,6 @@ MH_handler_refund_increase (struct TMH_RequestHandler *rh,
   json_t *contract_terms;
   const char *order_id;
   const char *reason;
-  const char *merchant;
-  char *last_session_id;
-  struct MerchantInstance *mi;
   struct GNUNET_HashCode h_contract_terms;
   struct TALER_MerchantRefundConfirmationPS confirmation;
   struct GNUNET_CRYPTO_EddsaSignature sig;
@@ -126,10 +185,10 @@ MH_handler_refund_increase (struct TMH_RequestHandler *rh,
     TALER_JSON_spec_amount ("refund", &refund),
     GNUNET_JSON_spec_string ("order_id", &order_id),
     GNUNET_JSON_spec_string ("reason", &reason),
-    GNUNET_JSON_spec_string ("instance", &merchant),
     GNUNET_JSON_spec_end ()
   };
   enum GNUNET_DB_QueryStatus qs;
+  enum GNUNET_DB_QueryStatus qsx;
 
   if (NULL == *connection_cls)
   {
@@ -142,11 +201,11 @@ MH_handler_refund_increase (struct TMH_RequestHandler *rh,
     ctx = *connection_cls;
   }
 
-  res = TMH_PARSE_post_json (connection,
-                             &ctx->json_parse_context,
-                             upload_data,
-                             upload_data_size,
-                             &root);
+  res = TALER_MHD_parse_post_json (connection,
+                                   &ctx->json_parse_context,
+                                   upload_data,
+                                   upload_data_size,
+                                   &root);
   if (GNUNET_SYSERR == res)
     return MHD_NO;
   /* the POST's body has to be further fetched */
@@ -154,41 +213,29 @@ MH_handler_refund_increase (struct TMH_RequestHandler *rh,
        (NULL == root) )
     return MHD_YES;
 
-  res = TMH_PARSE_json_data (connection,
-                             root,
-                             spec);
+  res = TALER_MHD_parse_json_data (connection,
+                                   root,
+                                   spec);
   if (GNUNET_NO == res)
   {
     GNUNET_break_op (0);
+    json_decref (root);
     return MHD_YES;
   }
-
   if (GNUNET_SYSERR == res)
   {
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                 "Hard error from JSON parser\n");
+    json_decref (root);
     return MHD_NO;
   }
 
-  mi = TMH_lookup_instance (merchant);
-  if (NULL == mi)
-  {
-    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                "No instance found\n");
-    GNUNET_JSON_parse_free (spec);
-    return TMH_RESPONSE_reply_not_found (connection,
-                                         TALER_EC_REFUND_INSTANCE_UNKNOWN,
-					 "Unknown instance given");
-  }
-
   db->preflight (db->cls);
-
   /* Convert order id to h_contract_terms */
   qs = db->find_contract_terms (db->cls,
-				&contract_terms,
-                                &last_session_id,
-				order_id,
-				&mi->pubkey);
+                                &contract_terms,
+                                order_id,
+                                &mi->pubkey);
   if (0 > qs)
   {
     /* single, read-only SQL statements should never cause
@@ -196,42 +243,84 @@ MH_handler_refund_increase (struct TMH_RequestHandler *rh,
     GNUNET_break (GNUNET_DB_STATUS_SOFT_ERROR != qs);
     /* Always report on hard error as well to enable diagnostics */
     GNUNET_break (GNUNET_DB_STATUS_HARD_ERROR == qs);
-    return TMH_RESPONSE_reply_internal_error (connection,
-                                              TALER_EC_REFUND_LOOKUP_DB_ERROR,
-                                              "An error occurred while retrieving payment data from db");
+    json_decref (root);
+    return TALER_MHD_reply_with_error (connection,
+                                       MHD_HTTP_INTERNAL_SERVER_ERROR,
+                                       TALER_EC_REFUND_LOOKUP_DB_ERROR,
+                                       "An error occurred while retrieving payment data from db");
   }
   if (GNUNET_DB_STATUS_SUCCESS_NO_RESULTS == qs)
   {
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                 "Unknown order id given: %s\n",
                 order_id);
-    return TMH_RESPONSE_reply_not_found (connection,
-                                         TALER_EC_REFUND_ORDER_ID_UNKNOWN,
-                                         "Order id not found in database");
+    json_decref (root);
+    return TALER_MHD_reply_with_error (connection,
+                                       MHD_HTTP_NOT_FOUND,
+                                       TALER_EC_REFUND_ORDER_ID_UNKNOWN,
+                                       "Order id not found in database");
   }
 
-  GNUNET_free (last_session_id);
-
   if (GNUNET_OK !=
       TALER_JSON_hash (contract_terms,
                        &h_contract_terms))
   {
     GNUNET_break (0);
     GNUNET_JSON_parse_free (spec);
+    json_decref (contract_terms);
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                 "Could not hash contract terms\n");
-    return TMH_RESPONSE_reply_internal_error (connection,
-                                              TALER_EC_INTERNAL_LOGIC_ERROR,
-                                              "Could not hash contract terms");
+    json_decref (root);
+    return TALER_MHD_reply_with_error (connection,
+                                       MHD_HTTP_INTERNAL_SERVER_ERROR,
+                                       TALER_EC_INTERNAL_LOGIC_ERROR,
+                                       "Could not hash contract terms");
   }
-  for (unsigned int i=0;i<MAX_RETRIES;i++)
+  for (unsigned int i = 0; i<MAX_RETRIES; i++)
   {
-    qs = db->increase_refund_for_contract (db->cls,
-					   &h_contract_terms,
-					   &mi->pubkey,
-					   &refund,
-					   reason);
-    if (GNUNET_DB_STATUS_SOFT_ERROR != qs)
+    if (GNUNET_OK !=
+        db->start (db->cls,
+                   "increase refund"))
+    {
+      GNUNET_break (0);
+      json_decref (contract_terms);
+      json_decref (root);
+      return GNUNET_DB_STATUS_HARD_ERROR;
+    }
+    qs = db->increase_refund_for_contract_NT (db->cls,
+                                              &h_contract_terms,
+                                              &mi->pubkey,
+                                              &refund,
+                                              reason);
+    GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
+                "increase refund returned %d\n",
+                qs);
+
+    if (GNUNET_DB_STATUS_HARD_ERROR == qs)
+    {
+      GNUNET_break (0);
+      db->rollback (db->cls);
+      break;
+    }
+    if (GNUNET_DB_STATUS_SOFT_ERROR == qs)
+    {
+      db->rollback (db->cls);
+      continue;
+    }
+    /* Got one or more deposits */
+    if (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT != qs)
+    {
+      db->rollback (db->cls);
+      break;
+    }
+    qsx = db->commit (db->cls);
+    if (GNUNET_DB_STATUS_HARD_ERROR == qsx)
+    {
+      GNUNET_break (0);
+      qs = qsx;
+      break;
+    }
+    if (GNUNET_DB_STATUS_SOFT_ERROR != qsx)
       break;
   }
   if (0 > qs)
@@ -241,9 +330,12 @@ MH_handler_refund_increase (struct TMH_RequestHandler *rh,
     /* Always report on hard error as well to enable diagnostics */
     GNUNET_break (GNUNET_DB_STATUS_HARD_ERROR == qs);
     GNUNET_JSON_parse_free (spec);
-    return TMH_RESPONSE_reply_internal_error (connection,
-                                              TALER_EC_REFUND_MERCHANT_DB_COMMIT_ERROR,
-                                              "Internal database error or refund amount too big");
+    json_decref (contract_terms);
+    json_decref (root);
+    return TALER_MHD_reply_with_error (connection,
+                                       MHD_HTTP_INTERNAL_SERVER_ERROR,
+                                       TALER_EC_REFUND_MERCHANT_DB_COMMIT_ERROR,
+                                       "Internal database error or refund amount too big");
   }
   if (GNUNET_DB_STATUS_SUCCESS_NO_RESULTS == qs)
   {
@@ -251,9 +343,12 @@ MH_handler_refund_increase (struct TMH_RequestHandler *rh,
                 "Refunded amount lower or equal to previous refund: %s\n",
                 TALER_amount2s (&refund));
     GNUNET_JSON_parse_free (spec);
-    return TMH_RESPONSE_reply_external_error (connection,
-                                              TALER_EC_REFUND_INCONSISTENT_AMOUNT,
-                                              "Amount incorrect: not larger than the previous one");
+    json_decref (contract_terms);
+    json_decref (root);
+    return TALER_MHD_reply_with_error (connection,
+                                       MHD_HTTP_CONFLICT,
+                                       TALER_EC_REFUND_INCONSISTENT_AMOUNT,
+                                       "Amount incorrect: not larger than the previous one");
   }
 
   /**
@@ -264,62 +359,46 @@ MH_handler_refund_increase (struct TMH_RequestHandler *rh,
    *
    * Just a "200 OK" should be fine here, as the frontend has all
    * the information needed to generate the right response.
-   */
-
-  confirmation.purpose.purpose = htonl (TALER_SIGNATURE_MERCHANT_REFUND_OK);
-  confirmation.purpose.size = htonl (sizeof (struct TALER_MerchantRefundConfirmationPS));
+   */confirmation.purpose.purpose = htonl (TALER_SIGNATURE_MERCHANT_REFUND_OK);
+  confirmation.purpose.size = htonl (sizeof (struct
+                                             TALER_MerchantRefundConfirmationPS));
   GNUNET_CRYPTO_hash (order_id,
                       strlen (order_id),
                       &confirmation.h_order_id);
 
   if (GNUNET_OK !=
       GNUNET_CRYPTO_eddsa_sign (&mi->privkey.eddsa_priv,
-				&confirmation.purpose,
-				&sig))
+                                &confirmation.purpose,
+                                &sig))
   {
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                 "Failed to sign successful refund confirmation\n");
     json_decref (contract_terms);
-    json_decref (root);
     GNUNET_JSON_parse_free (spec);
-    return TMH_RESPONSE_reply_internal_error (connection,
-                                              TALER_EC_NONE, /* FIXME! */
-                                              "Refund done, but failed to sign confirmation");
+    json_decref (root);
+    return TALER_MHD_reply_with_error (connection,
+                                       MHD_HTTP_INTERNAL_SERVER_ERROR,
+                                       TALER_EC_REFUND_MERCHANT_SIGNING_FAILED,
+                                       "Refund done, but failed to sign confirmation");
 
   }
 
   {
     int ret;
-    char *refund_pickup_url;
-    char *refund_redirect_url;
+    char *taler_refund_uri;
 
-    refund_pickup_url = TALER_url_absolute_mhd (connection,
-                                                "/public/refund",
-                                                "instance",
-                                                mi->id,
-                                                "order_id",
-                                                order_id,
-                                                NULL);
-    GNUNET_assert (NULL != refund_pickup_url);
-    refund_redirect_url = TALER_url_absolute_mhd (connection,
-                                                  "public/trigger-pay",
-                                                  "refund_url",
-                                                  refund_pickup_url,
-                                                  NULL);
-    GNUNET_assert (NULL != refund_redirect_url);
-    ret = TMH_RESPONSE_reply_json_pack (connection,
-                                        MHD_HTTP_OK,
-                                        "{s:o, s:s, s:o}",
-                                        "sig",
-                                        GNUNET_JSON_from_data_auto (&sig),
-                                        "refund_redirect_url",
-                                        refund_redirect_url,
-                                        "contract_terms",
-                                        contract_terms);
-    GNUNET_free (refund_pickup_url);
-    GNUNET_free (refund_redirect_url);
-    json_decref (root);
+    taler_refund_uri = make_taler_refund_uri (connection, mi->id, order_id);
+    ret = TALER_MHD_reply_json_pack (connection,
+                                     MHD_HTTP_OK,
+                                     "{s:o, s:o, s:s}",
+                                     "sig",
+                                     GNUNET_JSON_from_data_auto (&sig),
+                                     "contract_terms",
+                                     contract_terms,
+                                     "taler_refund_uri", taler_refund_uri);
+    GNUNET_free (taler_refund_uri);
     GNUNET_JSON_parse_free (spec);
+    json_decref (root);
     return ret;
   }
 }
@@ -367,8 +446,8 @@ process_refunds_cb (void *cls,
 
   if (GNUNET_OK !=
       GNUNET_CRYPTO_eddsa_sign (&prd->merchant->privkey.eddsa_priv,
-				&rr.purpose,
-				&sig))
+                                &rr.purpose,
+                                &sig))
   {
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                 "Could not sign refund request\n");
@@ -409,6 +488,7 @@ process_refunds_cb (void *cls,
  * @param[in,out] connection_cls the connection's closure (can be updated)
  * @param upload_data upload data
  * @param[in,out] upload_data_size number of bytes (left) in @a upload_data
+ * @param mi merchant backend instance, never NULL
  * @return MHD result code
  */
 int
@@ -416,39 +496,13 @@ MH_handler_refund_lookup (struct TMH_RequestHandler *rh,
                           struct MHD_Connection *connection,
                           void **connection_cls,
                           const char *upload_data,
-                          size_t *upload_data_size)
+                          size_t *upload_data_size,
+                          struct MerchantInstance *mi)
 {
   const char *order_id;
-  const char *instance;
   struct GNUNET_HashCode h_contract_terms;
   json_t *contract_terms;
-  struct MerchantInstance *mi;
   enum GNUNET_DB_QueryStatus qs;
-  char *last_session_id;
-
-  instance = MHD_lookup_connection_value (connection,
-                                          MHD_GET_ARGUMENT_KIND,
-                                          "instance");
-  if (NULL == instance)
-  {
-    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                "Argument 'instance' not given\n");
-    return TMH_RESPONSE_reply_arg_missing (connection,
-					   TALER_EC_PARAMETER_MISSING,
-                                           "instance");
-  }
-
-  mi = TMH_lookup_instance (instance);
-
-  if (NULL == mi)
-  {
-    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                "Unknown instance given: %s\n",
-                instance);
-    return TMH_RESPONSE_reply_not_found (connection,
-                                         TALER_EC_REFUND_INSTANCE_UNKNOWN,
-                                         "Unknown instance given");
-  }
 
   order_id = MHD_lookup_connection_value (connection,
                                           MHD_GET_ARGUMENT_KIND,
@@ -457,18 +511,19 @@ MH_handler_refund_lookup (struct TMH_RequestHandler *rh,
   {
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                 "Argument 'order_id' not given\n");
-    return TMH_RESPONSE_reply_arg_missing (connection,
-					   TALER_EC_PARAMETER_MISSING,
-                                           "order_id");
+    return TALER_MHD_reply_with_error (connection,
+                                       MHD_HTTP_BAD_REQUEST,
+                                       TALER_EC_PARAMETER_MISSING,
+                                       "order_id");
   }
 
   /* Convert order id to h_contract_terms */
   contract_terms = NULL;
+  db->preflight (db->cls);
   qs = db->find_contract_terms (db->cls,
-				&contract_terms,
-                                &last_session_id,
-				order_id,
-				&mi->pubkey);
+                                &contract_terms,
+                                order_id,
+                                &mi->pubkey);
   if (0 > qs)
   {
     /* single, read-only SQL statements should never cause
@@ -476,9 +531,10 @@ MH_handler_refund_lookup (struct TMH_RequestHandler *rh,
     GNUNET_break (GNUNET_DB_STATUS_SOFT_ERROR != qs);
     /* Always report on hard error as well to enable diagnostics */
     GNUNET_break (GNUNET_DB_STATUS_HARD_ERROR == qs);
-    return TMH_RESPONSE_reply_internal_error (connection,
-                                              TALER_EC_REFUND_LOOKUP_DB_ERROR,
-                                              "database error looking up order_id from merchant_contract_terms table");
+    return TALER_MHD_reply_with_error (connection,
+                                       MHD_HTTP_INTERNAL_SERVER_ERROR,
+                                       TALER_EC_REFUND_LOOKUP_DB_ERROR,
+                                       "database error looking up order_id from merchant_contract_terms table");
   }
 
   if (GNUNET_DB_STATUS_SUCCESS_NO_RESULTS == qs)
@@ -486,13 +542,12 @@ MH_handler_refund_lookup (struct TMH_RequestHandler *rh,
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                 "Unknown order id given: %s\n",
                 order_id);
-    return TMH_RESPONSE_reply_not_found (connection,
-                                         TALER_EC_REFUND_ORDER_ID_UNKNOWN,
-                                         "Order id not found in database");
+    return TALER_MHD_reply_with_error (connection,
+                                       MHD_HTTP_NOT_FOUND,
+                                       TALER_EC_REFUND_ORDER_ID_UNKNOWN,
+                                       "Order id not found in database");
   }
 
-  GNUNET_free (last_session_id);
-
   if (GNUNET_OK !=
       TALER_JSON_hash (contract_terms,
                        &h_contract_terms))
@@ -501,9 +556,10 @@ MH_handler_refund_lookup (struct TMH_RequestHandler *rh,
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                 "Could not hash contract terms\n");
     json_decref (contract_terms);
-    return TMH_RESPONSE_reply_internal_error (connection,
-                                              TALER_EC_INTERNAL_LOGIC_ERROR,
-                                              "Could not hash contract terms");
+    return TALER_MHD_reply_with_error (connection,
+                                       MHD_HTTP_INTERNAL_SERVER_ERROR,
+                                       TALER_EC_INTERNAL_LOGIC_ERROR,
+                                       "Could not hash contract terms");
   }
   json_decref (contract_terms);
 
@@ -517,17 +573,20 @@ MH_handler_refund_lookup (struct TMH_RequestHandler *rh,
                                    &ec,
                                    &errmsg);
     if (NULL == response)
-      return TMH_RESPONSE_reply_internal_error (connection,
-                                                ec,
-                                                errmsg);
-    return TMH_RESPONSE_reply_json_pack (connection, MHD_HTTP_OK,
-                                         "{s:o, s:o, s:o}",
-                                         "refund_permissions",
-                                         response,
-                                         "merchant_pub",
-                                         GNUNET_JSON_from_data_auto (&mi->pubkey),
-                                         "h_contract_terms",
-                                         GNUNET_JSON_from_data_auto (&h_contract_terms));
+      return TALER_MHD_reply_with_error (connection,
+                                         MHD_HTTP_INTERNAL_SERVER_ERROR,
+                                         ec,
+                                         errmsg);
+    return TALER_MHD_reply_json_pack (connection, MHD_HTTP_OK,
+                                      "{s:o, s:o, s:o}",
+                                      "refund_permissions",
+                                      response,
+                                      "merchant_pub",
+                                      GNUNET_JSON_from_data_auto (
+                                        &mi->pubkey),
+                                      "h_contract_terms",
+                                      GNUNET_JSON_from_data_auto (
+                                        &h_contract_terms));
   }
 }
 
@@ -554,13 +613,14 @@ TM_get_refund_json (const struct MerchantInstance *mi,
   prd.h_contract_terms = h_contract_terms;
   prd.merchant = mi;
   prd.ec = TALER_EC_NONE;
-  for (unsigned int i=0;i<MAX_RETRIES;i++)
+  db->preflight (db->cls);
+  for (unsigned int i = 0; i<MAX_RETRIES; i++)
   {
     qs = db->get_refunds_from_contract_terms_hash (db->cls,
-						   &mi->pubkey,
-						   h_contract_terms,
-						   &process_refunds_cb,
-						   &prd);
+                                                   &mi->pubkey,
+                                                   h_contract_terms,
+                                                   &process_refunds_cb,
+                                                   &prd);
     if (GNUNET_DB_STATUS_SOFT_ERROR != qs)
       break;
   }
@@ -572,7 +632,7 @@ TM_get_refund_json (const struct MerchantInstance *mi,
     json_decref (prd.response);
     *ret_ec = TALER_EC_REFUND_LOOKUP_DB_ERROR;
     *ret_errmsg = ("database hard error: looking for "
-                  "h_contract_terms in merchant_refunds table");
+                   "h_contract_terms in merchant_refunds table");
   }
   if (TALER_EC_NONE != prd.ec)
   {
