@@ -25,9 +25,7 @@
 #include <taler/taler_signatures.h>
 #include "taler-merchant-httpd.h"
 #include "taler-merchant-httpd_mhd.h"
-#include "taler-merchant-httpd_parsing.h"
 #include "taler-merchant-httpd_exchanges.h"
-#include "taler-merchant-httpd_responses.h"
 #include "taler-merchant-httpd_tip-pickup.h"
 
 
@@ -73,7 +71,7 @@ struct PickupContext
   struct TM_HandlerContext hc;
 
   /**
-   * Placeholder for #TMH_PARSE_post_json() to keep its internal state.
+   * Placeholder for #TALER_MHD_parse_post_json() to keep its internal state.
    */
 
   void *json_parse_context;
@@ -151,7 +149,7 @@ pickup_cleanup (struct TM_HandlerContext *hc)
 
   if (NULL != pc->planchets)
   {
-    for (unsigned int i=0;i<pc->planchets_len;i++)
+    for (unsigned int i = 0; i<pc->planchets_len; i++)
       GNUNET_free_non_null (pc->planchets[i].coin_ev);
     GNUNET_free (pc->planchets);
     pc->planchets = NULL;
@@ -161,7 +159,7 @@ pickup_cleanup (struct TM_HandlerContext *hc)
     TMH_EXCHANGES_find_exchange_cancel (pc->fo);
     pc->fo = NULL;
   }
-  TMH_PARSE_post_cleanup_callback (pc->json_parse_context);
+  TALER_MHD_parse_post_cleanup_callback (pc->json_parse_context);
   GNUNET_free_non_null (pc->exchange_url);
   GNUNET_free (pc);
 }
@@ -181,7 +179,7 @@ pickup_cleanup (struct TM_HandlerContext *hc)
  */
 static int
 run_pickup (struct MHD_Connection *connection,
-	    struct PickupContext *pc)
+            struct PickupContext *pc)
 {
   struct TALER_ReservePrivateKeyP reserve_priv;
   struct TALER_ReservePublicKeyP reserve_pub;
@@ -190,16 +188,17 @@ run_pickup (struct MHD_Connection *connection,
 
   if (TALER_EC_NONE != pc->ec)
   {
-    return TMH_RESPONSE_reply_rc (connection,
-				  pc->response_code,
-				  pc->ec,
-				  pc->error_hint);
+    return TALER_MHD_reply_with_error (connection,
+                                       pc->response_code,
+                                       pc->ec,
+                                       pc->error_hint);
   }
-  ec = db->pickup_tip (db->cls,
-		       &pc->total,
-		       &pc->tip_id,
-		       &pc->pickup_id,
-		       &reserve_priv);
+  db->preflight (db->cls);
+  ec = db->pickup_tip_TR (db->cls,
+                          &pc->total,
+                          &pc->tip_id,
+                          &pc->pickup_id,
+                          &reserve_priv);
   if (TALER_EC_NONE != ec)
   {
     unsigned int response_code;
@@ -220,34 +219,54 @@ run_pickup (struct MHD_Connection *connection,
       human = "database failure";
       break;
     }
-    return TMH_RESPONSE_reply_rc (connection,
-				  response_code,
-				  ec,
-				  human);
+    return TALER_MHD_reply_with_error (connection,
+                                       response_code,
+                                       ec,
+                                       human);
   }
   sigs = json_array ();
+  if (NULL == sigs)
+  {
+    GNUNET_break (0);
+    return TALER_MHD_reply_with_error (connection,
+                                       MHD_HTTP_INTERNAL_SERVER_ERROR,
+                                       TALER_EC_JSON_ALLOCATION_FAILURE,
+                                       "could not create JSON array");
+  }
   GNUNET_CRYPTO_eddsa_key_get_public (&reserve_priv.eddsa_priv,
-				      &reserve_pub.eddsa_pub);
-  for (unsigned int i=0;i<pc->planchets_len;i++)
+                                      &reserve_pub.eddsa_pub);
+  for (unsigned int i = 0; i<pc->planchets_len; i++)
   {
     struct PlanchetDetail *pd = &pc->planchets[i];
     struct TALER_ReserveSignatureP reserve_sig;
 
     pd->wr.reserve_pub = reserve_pub;
     GNUNET_assert (GNUNET_OK ==
-		   GNUNET_CRYPTO_eddsa_sign (&reserve_priv.eddsa_priv,
-					     &pd->wr.purpose,
-					     &reserve_sig.eddsa_signature));
-    json_array_append_new (sigs,
-                           json_pack ("{s:o}",
-                                      "reserve_sig",
-                                      GNUNET_JSON_from_data_auto (&reserve_sig)));
+                   GNUNET_CRYPTO_eddsa_sign (&reserve_priv.eddsa_priv,
+                                             &pd->wr.purpose,
+                                             &reserve_sig.eddsa_signature));
+    if (0 !=
+        json_array_append_new (sigs,
+                               json_pack ("{s:o}",
+                                          "reserve_sig",
+                                          GNUNET_JSON_from_data_auto (
+                                            &reserve_sig))))
+    {
+      GNUNET_break (0);
+      json_decref (sigs);
+      return TALER_MHD_reply_with_error (connection,
+                                         MHD_HTTP_INTERNAL_SERVER_ERROR,
+                                         TALER_EC_JSON_ALLOCATION_FAILURE,
+                                         "could not add element to JSON array");
+    }
   }
-  return TMH_RESPONSE_reply_json_pack (connection,
-                                       MHD_HTTP_OK,
-                                       "{s:o, s:o}",
-                                       "reserve_pub", GNUNET_JSON_from_data_auto (&reserve_pub),
-				       "reserve_sigs", sigs);
+  return TALER_MHD_reply_json_pack (connection,
+                                    MHD_HTTP_OK,
+                                    "{s:o, s:o}",
+                                    "reserve_pub",
+                                    GNUNET_JSON_from_data_auto (
+                                      &reserve_pub),
+                                    "reserve_sigs", sigs);
 }
 
 
@@ -262,9 +281,9 @@ run_pickup (struct MHD_Connection *connection,
  */
 static void
 exchange_found_cb (void *cls,
-		   struct TALER_EXCHANGE_Handle *eh,
-		   const struct TALER_Amount *wire_fee,
-		   int exchange_trusted)
+                   struct TALER_EXCHANGE_Handle *eh,
+                   const struct TALER_Amount *wire_fee,
+                   int exchange_trusted)
 {
   struct PickupContext *pc = cls;
   const struct TALER_EXCHANGE_Keys *keys;
@@ -286,7 +305,8 @@ exchange_found_cb (void *cls,
   if (NULL == keys)
   {
     pc->ec = TALER_EC_TIP_PICKUP_EXCHANGE_LACKED_KEYS;
-    pc->error_hint = "could not obtain denomination keys from exchange, check URL";
+    pc->error_hint =
+      "could not obtain denomination keys from exchange, check URL";
     pc->response_code = MHD_HTTP_FAILED_DEPENDENCY;
     TMH_trigger_daemon ();
     return;
@@ -294,20 +314,20 @@ exchange_found_cb (void *cls,
   GNUNET_assert (0 != pc->planchets_len);
   ae = GNUNET_NO;
   memset (&total,
-	  0,
-	  sizeof (total));
+          0,
+          sizeof (total));
   GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
               "Calculating tip amount over %u planchets!\n",
               pc->planchets_len);
   hc = GNUNET_CRYPTO_hash_context_start ();
-  for (unsigned int i=0;i<pc->planchets_len;i++)
+  for (unsigned int i = 0; i<pc->planchets_len; i++)
   {
     struct PlanchetDetail *pd = &pc->planchets[i];
     const struct TALER_EXCHANGE_DenomPublicKey *dk;
     struct TALER_Amount amount_with_fee;
 
     dk = TALER_EXCHANGE_get_denomination_key_by_hash (keys,
-						      &pd->wr.h_denomination_pub);
+                                                      &pd->wr.h_denomination_pub);
     if (NULL == dk)
     {
       pc->ec = TALER_EC_TIP_PICKUP_EXCHANGE_LACKED_KEY;
@@ -318,15 +338,15 @@ exchange_found_cb (void *cls,
       return;
     }
     GNUNET_CRYPTO_hash_context_read (hc,
-				     &pd->wr.h_denomination_pub,
-				     sizeof (struct GNUNET_HashCode));
+                                     &pd->wr.h_denomination_pub,
+                                     sizeof (struct GNUNET_HashCode));
     GNUNET_CRYPTO_hash_context_read (hc,
-				     pd->coin_ev,
-				     pd->coin_ev_size);
+                                     pd->coin_ev,
+                                     pd->coin_ev_size);
     if (GNUNET_OK !=
-	TALER_amount_add (&amount_with_fee,
-			  &dk->value,
-			  &dk->fee_withdraw))
+        TALER_amount_add (&amount_with_fee,
+                          &dk->value,
+                          &dk->fee_withdraw))
     {
       ae = GNUNET_YES;
     }
@@ -337,20 +357,20 @@ exchange_found_cb (void *cls,
     else
     {
       if (GNUNET_OK !=
-	  TALER_amount_add (&total,
-			    &total,
-			    &amount_with_fee))
+          TALER_amount_add (&total,
+                            &total,
+                            &amount_with_fee))
       {
-	ae = GNUNET_YES;
+        ae = GNUNET_YES;
       }
     }
     TALER_amount_hton (&pd->wr.withdraw_fee,
-		       &dk->fee_withdraw);
+                       &dk->fee_withdraw);
     TALER_amount_hton (&pd->wr.amount_with_fee,
-		       &amount_with_fee);
+                       &amount_with_fee);
   }
   GNUNET_CRYPTO_hash_context_finish (hc,
-				     &pc->pickup_id);
+                                     &pc->pickup_id);
   if (GNUNET_YES == ae)
   {
     pc->ec = TALER_EC_TIP_PICKUP_EXCHANGE_AMOUNT_OVERFLOW;
@@ -377,10 +397,11 @@ prepare_pickup (struct PickupContext *pc)
 {
   enum GNUNET_DB_QueryStatus qs;
 
+  db->preflight (db->cls);
   qs = db->lookup_tip_by_id (db->cls,
                              &pc->tip_id,
                              &pc->exchange_url,
-                             NULL, NULL);
+                             NULL, NULL, NULL, NULL);
   if (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT != qs)
   {
     unsigned int response_code;
@@ -406,22 +427,22 @@ prepare_pickup (struct PickupContext *pc)
       response_code = MHD_HTTP_INTERNAL_SERVER_ERROR;
       break;
     }
-    return TMH_RESPONSE_reply_rc (pc->connection,
-				  response_code,
-				  ec,
-				  "Could not determine exchange URL for the given tip id");
+    return TALER_MHD_reply_with_error (pc->connection,
+                                       response_code,
+                                       ec,
+                                       "Could not determine exchange URL for the given tip id");
 
   }
   pc->fo = TMH_EXCHANGES_find_exchange (pc->exchange_url,
-					NULL,
-					&exchange_found_cb,
-					pc);
+                                        NULL,
+                                        &exchange_found_cb,
+                                        pc);
   if (NULL == pc->fo)
   {
-    return TMH_RESPONSE_reply_rc (pc->connection,
-				  MHD_HTTP_INTERNAL_SERVER_ERROR,
-				  TALER_EC_INTERNAL_INVARIANT_FAILURE,
-				  "consult server logs");
+    return TALER_MHD_reply_with_error (pc->connection,
+                                       MHD_HTTP_INTERNAL_SERVER_ERROR,
+                                       TALER_EC_INTERNAL_INVARIANT_FAILURE,
+                                       "consult server logs");
   }
   MHD_suspend_connection (pc->connection);
   return MHD_YES;
@@ -439,35 +460,35 @@ prepare_pickup (struct PickupContext *pc)
  */
 static int
 parse_planchet (struct MHD_Connection *connection,
-		const json_t *planchet,
-		struct PlanchetDetail *pd)
+                const json_t *planchet,
+                struct PlanchetDetail *pd)
 {
   int ret;
   struct GNUNET_JSON_Specification spec[] = {
     GNUNET_JSON_spec_fixed_auto ("denom_pub_hash",
-				 &pd->wr.h_denomination_pub),
+                                 &pd->wr.h_denomination_pub),
     GNUNET_JSON_spec_varsize ("coin_ev",
-			      (void **) &pd->coin_ev,
-			      &pd->coin_ev_size),
-    GNUNET_JSON_spec_end()
+                              (void **) &pd->coin_ev,
+                              &pd->coin_ev_size),
+    GNUNET_JSON_spec_end ()
   };
 
-  ret = TMH_PARSE_json_data (connection,
-			     planchet,
-			     spec);
+  ret = TALER_MHD_parse_json_data (connection,
+                                   planchet,
+                                   spec);
   if (GNUNET_OK != ret)
     return ret;
   pd->wr.purpose.purpose = htonl (TALER_SIGNATURE_WALLET_RESERVE_WITHDRAW);
   pd->wr.purpose.size = htonl (sizeof (struct TALER_WithdrawRequestPS));
   GNUNET_CRYPTO_hash (pd->coin_ev,
-		      pd->coin_ev_size,
-		      &pd->wr.h_coin_envelope);
+                      pd->coin_ev_size,
+                      &pd->wr.h_coin_envelope);
   return ret;
 }
 
 
 /**
- * Manages a /tip-pickup call, checking that the tip is authorized,
+ * Manages a POST /tip-pickup call, checking that the tip is authorized,
  * and if so, returning the withdrawal permissions.
  *
  * @param rh context of the handler
@@ -475,6 +496,7 @@ parse_planchet (struct MHD_Connection *connection,
  * @param[in,out] connection_cls the connection's closure (can be updated)
  * @param upload_data upload data
  * @param[in,out] upload_data_size number of bytes (left) in @a upload_data
+ * @param mi merchant backend instance, never NULL
  * @return MHD result code
  */
 int
@@ -482,15 +504,18 @@ MH_handler_tip_pickup (struct TMH_RequestHandler *rh,
                        struct MHD_Connection *connection,
                        void **connection_cls,
                        const char *upload_data,
-                       size_t *upload_data_size)
+                       size_t *upload_data_size,
+                       struct MerchantInstance *mi)
 {
   int res;
   struct GNUNET_HashCode tip_id;
   json_t *planchets;
   struct GNUNET_JSON_Specification spec[] = {
-    GNUNET_JSON_spec_fixed_auto ("tip_id", &tip_id),
-    GNUNET_JSON_spec_json ("planchets", &planchets),
-    GNUNET_JSON_spec_end()
+    GNUNET_JSON_spec_fixed_auto ("tip_id",
+                                 &tip_id),
+    GNUNET_JSON_spec_json ("planchets",
+                           &planchets),
+    GNUNET_JSON_spec_end ()
   };
   struct PickupContext *pc;
   json_t *root;
@@ -512,13 +537,13 @@ MH_handler_tip_pickup (struct TMH_RequestHandler *rh,
        after an exchange /keys' request to obtain the DKs
        (and not for each request). */
     return run_pickup (connection,
-		       pc);
+                       pc);
   }
-  res = TMH_PARSE_post_json (connection,
-                             &pc->json_parse_context,
-                             upload_data,
-                             upload_data_size,
-                             &root);
+  res = TALER_MHD_parse_post_json (connection,
+                                   &pc->json_parse_context,
+                                   upload_data,
+                                   upload_data_size,
+                                   &root);
   if (GNUNET_SYSERR == res)
     return MHD_NO;
   /* the POST's body has to be further fetched */
@@ -526,9 +551,9 @@ MH_handler_tip_pickup (struct TMH_RequestHandler *rh,
        (NULL == root) )
     return MHD_YES;
 
-  res = TMH_PARSE_json_data (connection,
-                             root,
-                             spec);
+  res = TALER_MHD_parse_json_data (connection,
+                                   root,
+                                   spec);
   if (GNUNET_YES != res)
   {
     GNUNET_break_op (0);
@@ -540,30 +565,30 @@ MH_handler_tip_pickup (struct TMH_RequestHandler *rh,
   {
     GNUNET_JSON_parse_free (spec);
     json_decref (root);
-    return TMH_RESPONSE_reply_rc (connection,
-				  MHD_HTTP_BAD_REQUEST,
-				  TALER_EC_TIP_PICKUP_EXCHANGE_TOO_MANY_PLANCHETS,
-				  "limit of 1024 planchets exceeded by request");
+    return TALER_MHD_reply_with_error (connection,
+                                       MHD_HTTP_BAD_REQUEST,
+                                       TALER_EC_TIP_PICKUP_EXCHANGE_TOO_MANY_PLANCHETS,
+                                       "limit of 1024 planchets exceeded by request");
   }
   if (0 == pc->planchets_len)
   {
     GNUNET_JSON_parse_free (spec);
     json_decref (root);
-    return TMH_RESPONSE_reply_rc (connection,
-				  MHD_HTTP_BAD_REQUEST,
-				  TALER_EC_PARAMETER_MALFORMED,
-				  "no planchets specified");
+    return TALER_MHD_reply_with_error (connection,
+                                       MHD_HTTP_BAD_REQUEST,
+                                       TALER_EC_PARAMETER_MALFORMED,
+                                       "no planchets specified");
   }
   db->preflight (db->cls);
   pc->planchets = GNUNET_new_array (pc->planchets_len,
-				    struct PlanchetDetail);
-  for (unsigned int i=0;i<pc->planchets_len;i++)
+                                    struct PlanchetDetail);
+  for (unsigned int i = 0; i<pc->planchets_len; i++)
   {
     if (GNUNET_OK !=
-	(res = parse_planchet (connection,
-			       json_array_get (planchets,
-					       i),
-			       &pc->planchets[i])))
+        (res = parse_planchet (connection,
+                               json_array_get (planchets,
+                                               i),
+                               &pc->planchets[i])))
     {
       GNUNET_JSON_parse_free (spec);
       json_decref (root);
@@ -576,3 +601,124 @@ MH_handler_tip_pickup (struct TMH_RequestHandler *rh,
   json_decref (root);
   return res;
 }
+
+
+/**
+ * Manages a GET /tip-pickup call, checking that the tip is authorized,
+ * and if so, returning the withdrawal permissions.
+ *
+ * @param rh context of the handler
+ * @param connection the MHD connection to handle
+ * @param[in,out] connection_cls the connection's closure (can be updated)
+ * @param upload_data upload data
+ * @param[in,out] upload_data_size number of bytes (left) in @a upload_data
+ * @param mi merchant backend instance, never NULL
+ * @return MHD result code
+ */
+int
+MH_handler_tip_pickup_get (struct TMH_RequestHandler *rh,
+                           struct MHD_Connection *connection,
+                           void **connection_cls,
+                           const char *upload_data,
+                           size_t *upload_data_size,
+                           struct MerchantInstance *mi)
+{
+  const char *tip_id_str;
+  char *exchange_url;
+  json_t *extra;
+  struct GNUNET_HashCode tip_id;
+  struct TALER_Amount tip_amount;
+  struct TALER_Amount tip_amount_left;
+  struct GNUNET_TIME_Absolute timestamp;
+  struct GNUNET_TIME_Absolute timestamp_expire;
+  int ret;
+  int qs;
+
+  tip_id_str = MHD_lookup_connection_value (connection,
+                                            MHD_GET_ARGUMENT_KIND,
+                                            "tip_id");
+
+  if (NULL == tip_id_str)
+  {
+    /* tip_id is required but missing */
+    GNUNET_break_op (0);
+    return TALER_MHD_reply_with_error (connection,
+                                       MHD_HTTP_BAD_REQUEST,
+                                       TALER_EC_PARAMETER_MISSING,
+                                       "tip_id required");
+  }
+
+  if (GNUNET_OK !=
+      GNUNET_CRYPTO_hash_from_string (tip_id_str,
+                                      &tip_id))
+  {
+    /* tip_id has wrong encoding */
+    GNUNET_break_op (0);
+    return TALER_MHD_reply_with_error (connection,
+                                       MHD_HTTP_BAD_REQUEST,
+                                       TALER_EC_PARAMETER_MALFORMED,
+                                       "tip_id malformed");
+  }
+
+  db->preflight (db->cls);
+  qs = db->lookup_tip_by_id (db->cls,
+                             &tip_id,
+                             &exchange_url,
+                             &extra,
+                             &tip_amount,
+                             &tip_amount_left,
+                             &timestamp);
+
+  if (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT != qs)
+  {
+    unsigned int response_code;
+    enum TALER_ErrorCode ec;
+
+    switch (qs)
+    {
+    case GNUNET_DB_STATUS_SUCCESS_NO_RESULTS:
+      ec = TALER_EC_TIP_PICKUP_TIP_ID_UNKNOWN;
+      response_code = MHD_HTTP_NOT_FOUND;
+      break;
+    case GNUNET_DB_STATUS_SOFT_ERROR:
+      ec = TALER_EC_TIP_PICKUP_DB_ERROR_SOFT;
+      response_code = MHD_HTTP_INTERNAL_SERVER_ERROR;
+      break;
+    case GNUNET_DB_STATUS_HARD_ERROR:
+      ec = TALER_EC_TIP_PICKUP_DB_ERROR_HARD;
+      response_code = MHD_HTTP_INTERNAL_SERVER_ERROR;
+      break;
+    default:
+      GNUNET_break (0);
+      ec = TALER_EC_INTERNAL_LOGIC_ERROR;
+      response_code = MHD_HTTP_INTERNAL_SERVER_ERROR;
+      break;
+    }
+    return TALER_MHD_reply_with_error (connection,
+                                       response_code,
+                                       ec,
+                                       "Could not determine exchange URL for the given tip id");
+  }
+
+  timestamp_expire = GNUNET_TIME_absolute_add (timestamp,
+                                               GNUNET_TIME_UNIT_DAYS);
+
+  ret = TALER_MHD_reply_json_pack (connection,
+                                   MHD_HTTP_OK,
+                                   "{s:s, s:o, s:o, s:o, s:o, s:o}",
+                                   "exchange_url", exchange_url,
+                                   "amount", TALER_JSON_from_amount (
+                                     &tip_amount),
+                                   "amount_left", TALER_JSON_from_amount (
+                                     &tip_amount_left),
+                                   "stamp_created",
+                                   GNUNET_JSON_from_time_abs (timestamp),
+                                   "stamp_expire",
+                                   GNUNET_JSON_from_time_abs (
+                                     timestamp_expire),
+                                   "extra", extra);
+
+  GNUNET_free (exchange_url);
+  json_decref (extra);
+  return ret;
+}
