@@ -24,11 +24,10 @@
 #include <taler/taler_signatures.h>
 #include <taler/taler_json_lib.h>
 #include "taler-merchant-httpd.h"
+#include "taler_merchant_service.h"
 #include "taler-merchant-httpd_mhd.h"
-#include "taler-merchant-httpd_parsing.h"
 #include "taler-merchant-httpd_auditors.h"
 #include "taler-merchant-httpd_exchanges.h"
-#include "taler-merchant-httpd_responses.h"
 #include "taler-merchant-httpd_track-transaction.h"
 
 
@@ -75,7 +74,8 @@ extern struct GNUNET_CONTAINER_MultiHashMap *by_id_map;
 /**
  * How long to wait before giving up processing with the exchange?
  */
-#define TRACK_TIMEOUT (GNUNET_TIME_relative_multiply (GNUNET_TIME_UNIT_SECONDS, 30))
+#define TRACK_TIMEOUT (GNUNET_TIME_relative_multiply (GNUNET_TIME_UNIT_SECONDS, \
+                                                      30))
 
 /**
  * How often do we retry the simple INSERT database transaction?
@@ -92,37 +92,44 @@ extern struct GNUNET_CONTAINER_MultiHashMap *by_id_map;
  */
 static struct MHD_Response *
 make_track_transaction_ok (unsigned int num_transfers,
-			   const struct TransactionWireTransfer *transfers)
+                           const struct TransactionWireTransfer *transfers)
 {
   struct MHD_Response *ret;
   json_t *j_transfers;
   struct TALER_Amount sum;
 
   j_transfers = json_array ();
-  for (unsigned int i=0;i<num_transfers;i++)
+  for (unsigned int i = 0; i<num_transfers; i++)
   {
     const struct TransactionWireTransfer *transfer = &transfers[i];
 
     sum = transfer->coins[0].amount_with_fee;
-    for (unsigned int j=1;j<transfer->num_coins;j++)
+    for (unsigned int j = 1; j<transfer->num_coins; j++)
     {
       const struct TALER_MERCHANT_CoinWireTransfer *coin = &transfer->coins[j];
 
       GNUNET_assert (GNUNET_SYSERR !=
-		     TALER_amount_add (&sum,
-				       &sum,
-				       &coin->amount_with_fee));
+                     TALER_amount_add (&sum,
+                                       &sum,
+                                       &coin->amount_with_fee));
     }
 
     GNUNET_assert (0 ==
                    json_array_append_new (j_transfers,
                                           json_pack ("{s:s, s:o, s:o, s:o}",
-                                                     "exchange", transfer->exchange_url,
-                                                     "wtid", GNUNET_JSON_from_data_auto (&transfer->wtid),
-                                                     "execution_time", GNUNET_JSON_from_time_abs (transfer->execution_time),
-                                                     "amount", TALER_JSON_from_amount (&sum))));
+                                                     "exchange",
+                                                     transfer->exchange_url,
+                                                     "wtid",
+                                                     GNUNET_JSON_from_data_auto (
+                                                       &transfer->wtid),
+                                                     "execution_time",
+                                                     GNUNET_JSON_from_time_abs (
+                                                       transfer->execution_time),
+                                                     "amount",
+                                                     TALER_JSON_from_amount (
+                                                       &sum))));
   }
-  ret = TMH_RESPONSE_make_json (j_transfers);
+  ret = TALER_MHD_make_json (j_transfers);
   json_decref (j_transfers);
   return ret;
 }
@@ -171,7 +178,7 @@ struct TrackCoinContext
   /**
    * Handle for the request to resolve the WTID for this coin.
    */
-  struct TALER_EXCHANGE_TrackTransactionHandle *dwh;
+  struct TALER_EXCHANGE_DepositGetHandle *dwh;
 
   /**
    * Wire transfer identifier for this coin.
@@ -252,7 +259,7 @@ struct TrackTransactionContext
   /**
    * Handle we use to resolve transactions for a given WTID.
    */
-  struct TALER_EXCHANGE_TrackTransferHandle *wdh;
+  struct TALER_EXCHANGE_TransfersGetHandle *wdh;
 
   /**
    * Response to return upon resume.
@@ -335,7 +342,7 @@ free_tctx (struct TrackTransactionContext *tctx)
                                  tcc);
     if (NULL != tcc->dwh)
     {
-      TALER_EXCHANGE_track_transaction_cancel (tcc->dwh);
+      TALER_EXCHANGE_deposits_get_cancel (tcc->dwh);
       tcc->dwh = NULL;
     }
     if (NULL != tcc->exchange_url)
@@ -347,7 +354,7 @@ free_tctx (struct TrackTransactionContext *tctx)
   }
   if (NULL != tctx->wdh)
   {
-    TALER_EXCHANGE_track_transfer_cancel (tctx->wdh);
+    TALER_EXCHANGE_transfers_get_cancel (tctx->wdh);
     tctx->wdh = NULL;
   }
   if (NULL != tctx->fo)
@@ -443,7 +450,7 @@ trace_coins (struct TrackTransactionContext *tctx);
 static void
 wire_deposits_cb (void *cls,
                   unsigned int http_status,
-		  enum TALER_ErrorCode ec,
+                  enum TALER_ErrorCode ec,
                   const struct TALER_ExchangePublicKeyP *exchange_pub,
                   const json_t *json,
                   const struct GNUNET_HashCode *h_wire,
@@ -462,22 +469,26 @@ wire_deposits_cb (void *cls,
     GNUNET_break_op (0);
     resume_track_transaction_with_response
       (tctx,
-       MHD_HTTP_FAILED_DEPENDENCY,
-       TMH_RESPONSE_make_json_pack ("{s:I, s:I, s:I, s:O}",
-				    "code", (json_int_t) TALER_EC_TRACK_TRANSACTION_WIRE_TRANSFER_TRACE_ERROR,
-                                    "exchange-http-status", (json_int_t) http_status,
-				    "exchange-code", (json_int_t) ec,
-                                    "details", json));
+      MHD_HTTP_FAILED_DEPENDENCY,
+      TALER_MHD_make_json_pack ("{s:I, s:I, s:I, s:O}",
+                                "code",
+                                (json_int_t)
+                                TALER_EC_TRACK_TRANSACTION_WIRE_TRANSFER_TRACE_ERROR,
+                                "exchange-http-status",
+                                (json_int_t) http_status,
+                                "exchange-code", (json_int_t) ec,
+                                "details", json));
     return;
   }
-  for (unsigned int i=0;i<MAX_RETRIES;i++)
+  for (unsigned int i = 0; i<MAX_RETRIES; i++)
   {
+    db->preflight (db->cls);
     qs = db->store_transfer_to_proof (db->cls,
-				      tctx->current_exchange,
-				      &tctx->current_wtid,
-				      tctx->current_execution_time,
-				      exchange_pub,
-				      json);
+                                      tctx->current_exchange,
+                                      &tctx->current_wtid,
+                                      tctx->current_execution_time,
+                                      exchange_pub,
+                                      json);
     if (GNUNET_DB_STATUS_SOFT_ERROR != qs)
       break;
   }
@@ -491,40 +502,40 @@ wire_deposits_cb (void *cls,
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                 "Failed to store transfer-to-proof mapping in DB\n");
   }
-  for (struct TrackCoinContext *tcc=tctx->tcc_head;
+  for (struct TrackCoinContext *tcc = tctx->tcc_head;
        NULL != tcc;
-       tcc=tcc->next)
+       tcc = tcc->next)
   {
     if (GNUNET_YES == tcc->have_wtid)
       continue;
-    for (unsigned int d=0;d<details_length;d++)
+    for (unsigned int d = 0; d<details_length; d++)
     {
 
-      if (0 == memcmp (&details[d].coin_pub,
-                       &tcc->coin_pub,
-                       sizeof (struct TALER_CoinSpendPublicKeyP)))
+      if (0 == GNUNET_memcmp (&details[d].coin_pub,
+                              &tcc->coin_pub))
       {
         tcc->wtid = tctx->current_wtid;
         tcc->execution_time = tctx->current_execution_time;
         tcc->have_wtid = GNUNET_YES;
       }
 
-      for (unsigned int i=0;i<MAX_RETRIES;i++)
+      for (unsigned int i = 0; i<MAX_RETRIES; i++)
       {
-	qs = db->store_coin_to_transfer (db->cls,
-					 &details[d].h_contract_terms,
-					 &details[d].coin_pub,
-					 &tctx->current_wtid);
-	if (GNUNET_DB_STATUS_SOFT_ERROR != qs)
-	  break;
+        db->preflight (db->cls);
+        qs = db->store_coin_to_transfer (db->cls,
+                                         &details[d].h_contract_terms,
+                                         &details[d].coin_pub,
+                                         &tctx->current_wtid);
+        if (GNUNET_DB_STATUS_SOFT_ERROR != qs)
+          break;
       }
       if (0 > qs)
       {
-	/* Not good, but not fatal either, log error and continue */
-	/* Special report if retries insufficient */
-	GNUNET_break (GNUNET_DB_STATUS_SOFT_ERROR != qs);
-	/* Always report on hard error as well to enable diagnostics */
-	GNUNET_break (GNUNET_DB_STATUS_HARD_ERROR == qs);
+        /* Not good, but not fatal either, log error and continue */
+        /* Special report if retries insufficient */
+        GNUNET_break (GNUNET_DB_STATUS_SOFT_ERROR != qs);
+        /* Always report on hard error as well to enable diagnostics */
+        GNUNET_break (GNUNET_DB_STATUS_HARD_ERROR == qs);
         GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                     "Failed to store coin-to-transfer mapping in DB\n");
       }
@@ -592,7 +603,7 @@ proof_cb (void *cls,
 static void
 wtid_cb (void *cls,
          unsigned int http_status,
-	 enum TALER_ErrorCode ec,
+         enum TALER_ErrorCode ec,
          const struct TALER_ExchangePublicKeyP *exchange_pub,
          const json_t *json,
          const struct TALER_WireTransferIdentifierRawP *wtid,
@@ -605,21 +616,36 @@ wtid_cb (void *cls,
   enum GNUNET_DB_QueryStatus qs;
 
   tcc->dwh = NULL;
+
   if (MHD_HTTP_OK != http_status)
   {
-    /* Transaction not resolved for one of the coins, report error!
-       We keep the status code for #MHD_HTTP_ACCEPTED, but box all the
-       others (as #MHD_HTTP_ACCEPTED is not an error). */
+    if (MHD_HTTP_ACCEPTED == http_status)
+    {
+      resume_track_transaction_with_response
+        (tcc->tctx,
+        MHD_HTTP_ACCEPTED,
+        /* Return verbatim what the exchange said.  */
+        TALER_MHD_make_json (json));
+
+      return;
+    }
+
+    /* Transaction not resolved for one of the
+       coins, report error! */
     resume_track_transaction_with_response
       (tcc->tctx,
-       (MHD_HTTP_ACCEPTED == http_status)
-       ? MHD_HTTP_ACCEPTED
-       : MHD_HTTP_FAILED_DEPENDENCY,
-       TMH_RESPONSE_make_json_pack ("{s:I, s:I, s:I, s:O}",
-                                    "code", (json_int_t) TALER_EC_TRACK_TRANSACTION_COIN_TRACE_ERROR,
-                                    "exchange-http-status", (json_int_t) http_status,
-                                    "exchange-code", (json_int_t) ec,
-                                    "details", json));
+      MHD_HTTP_FAILED_DEPENDENCY,
+      TALER_MHD_make_json_pack
+        ("{s:I, s:I, s:I, s:O}",
+        "code",
+        (json_int_t) TALER_EC_TRACK_TRANSACTION_COIN_TRACE_ERROR,
+        "exchange-http-status",
+        (json_int_t) http_status,
+        "exchange-code",
+        (json_int_t) ec,
+        "details",
+        json));
+
     return;
   }
   tctx->current_wtid = *wtid;
@@ -628,22 +654,25 @@ wtid_cb (void *cls,
   pcc.p_ret = NULL;
   /* attempt to find this wtid's track from our database,
      Will make pcc.p_ret point to a "proof", if one exists. */
+  db->preflight (db->cls);
   qs = db->find_proof_by_wtid (db->cls,
-			       tctx->current_exchange,
-			       wtid,
-			       &proof_cb,
-			       &pcc);
+                               tctx->current_exchange,
+                               wtid,
+                               &proof_cb,
+                               &pcc);
   if (0 > qs)
   {
-    /* Simple select queries should not cause serialization issues */
+    /* Simple select queries should not
+       cause serialization issues */
     GNUNET_break (GNUNET_DB_STATUS_SOFT_ERROR != qs);
     /* Always report on hard error as well to enable diagnostics */
     GNUNET_break (GNUNET_DB_STATUS_HARD_ERROR == qs);
     resume_track_transaction_with_response
       (tcc->tctx,
-       MHD_HTTP_INTERNAL_SERVER_ERROR,
-       TMH_RESPONSE_make_error (TALER_EC_TRACK_TRANSACTION_DB_FETCH_FAILED,
-				"Fail to query database about proofs"));
+      MHD_HTTP_INTERNAL_SERVER_ERROR,
+      TALER_MHD_make_error
+        (TALER_EC_TRACK_TRANSACTION_DB_FETCH_TRANSACTION_ERROR,
+        "Fail to query database about proofs"));
     return;
   }
 
@@ -656,20 +685,24 @@ wtid_cb (void *cls,
     GNUNET_break_op (0);
     resume_track_transaction_with_response
       (tcc->tctx,
-       MHD_HTTP_FAILED_DEPENDENCY,
-       TMH_RESPONSE_make_json_pack ("{s:I, s:s, s:O, s:o, s:o}",
-				    "code", (json_int_t) TALER_EC_TRACK_TRANSACTION_CONFLICTING_REPORTS,
-                                    "error", "conflicting transfer data from exchange",
-                                    "transaction_tracking_claim", json,
-                                    "wtid_tracking_claim", pcc.p_ret,
-                                    "coin_pub", GNUNET_JSON_from_data_auto (&tcc->coin_pub)));
+      MHD_HTTP_FAILED_DEPENDENCY,
+      TALER_MHD_make_json_pack ("{s:I, s:s, s:O, s:o, s:o}",
+                                "code",
+                                (json_int_t)
+                                TALER_EC_TRACK_TRANSACTION_CONFLICTING_REPORTS,
+                                "error",
+                                "conflicting transfer data from exchange",
+                                "transaction_tracking_claim", json,
+                                "wtid_tracking_claim", pcc.p_ret,
+                                "coin_pub", GNUNET_JSON_from_data_auto (
+                                  &tcc->coin_pub)));
     return;
   }
 
-  tctx->wdh = TALER_EXCHANGE_track_transfer (tctx->eh,
-                                             wtid,
-                                             &wire_deposits_cb,
-                                             tctx);
+  tctx->wdh = TALER_EXCHANGE_transfers_get (tctx->eh,
+                                            wtid,
+                                            &wire_deposits_cb,
+                                            tctx);
 }
 
 
@@ -693,12 +726,11 @@ generate_response (struct TrackTransactionContext *tctx)
     int found = GNUNET_NO;
 
     for (struct TrackCoinContext *tcc2 = tctx->tcc_head;
-	 tcc2 != tcc;
-	 tcc2 = tcc2->next)
+         tcc2 != tcc;
+         tcc2 = tcc2->next)
     {
-      if (0 == memcmp (&tcc->wtid,
-                       &tcc2->wtid,
-                       sizeof (struct TALER_WireTransferIdentifierRawP)))
+      if (0 == GNUNET_memcmp (&tcc->wtid,
+                              &tcc2->wtid))
       {
         found = GNUNET_YES;
         break;
@@ -721,12 +753,11 @@ generate_response (struct TrackTransactionContext *tctx)
       int found = GNUNET_NO;
 
       for (struct TrackCoinContext *tcc2 = tctx->tcc_head;
-	   tcc2 != tcc;
-	   tcc2 = tcc2->next)
+           tcc2 != tcc;
+           tcc2 = tcc2->next)
       {
-        if (0 == memcmp (&tcc->wtid,
-                         &tcc2->wtid,
-                         sizeof (struct TALER_WireTransferIdentifierRawP)))
+        if (0 == GNUNET_memcmp (&tcc->wtid,
+                                &tcc2->wtid))
         {
           found = GNUNET_YES;
           break;
@@ -739,17 +770,16 @@ generate_response (struct TrackTransactionContext *tctx)
 
         wt = &wts[wtid_off++];
         wt->wtid = tcc->wtid;
-	wt->exchange_url = tcc->exchange_url;
+        wt->exchange_url = tcc->exchange_url;
         wt->execution_time = tcc->execution_time;
         /* count number of coins with this wtid */
         num_coins = 0;
         for (struct TrackCoinContext *tcc2 = tctx->tcc_head;
-	     NULL != tcc2;
-	     tcc2 = tcc2->next)
+             NULL != tcc2;
+             tcc2 = tcc2->next)
         {
-          if (0 == memcmp (&wt->wtid,
-                           &tcc2->wtid,
-                           sizeof (struct TALER_WireTransferIdentifierRawP)))
+          if (0 == GNUNET_memcmp (&wt->wtid,
+                                  &tcc2->wtid))
             num_coins++;
         }
         /* initialize coins array */
@@ -758,14 +788,14 @@ generate_response (struct TrackTransactionContext *tctx)
                                       struct TALER_MERCHANT_CoinWireTransfer);
         num_coins = 0;
         for (struct TrackCoinContext *tcc2 = tctx->tcc_head;
-	     NULL != tcc2;
-	     tcc2 = tcc2->next)
+             NULL != tcc2;
+             tcc2 = tcc2->next)
         {
-          if (0 == memcmp (&wt->wtid,
-                           &tcc2->wtid,
-                           sizeof (struct TALER_WireTransferIdentifierRawP)))
+          if (0 == GNUNET_memcmp (&wt->wtid,
+                                  &tcc2->wtid))
           {
-            struct TALER_MERCHANT_CoinWireTransfer *coin = &wt->coins[num_coins++];
+            struct TALER_MERCHANT_CoinWireTransfer *coin =
+              &wt->coins[num_coins++];
 
             coin->coin_pub = tcc2->coin_pub;
             coin->amount_with_fee = tcc2->amount_with_fee;
@@ -777,8 +807,8 @@ generate_response (struct TrackTransactionContext *tctx)
     GNUNET_assert (wtid_off == num_wtid);
 
     resp = make_track_transaction_ok (num_wtid,
-				      wts);
-    for (wtid_off=0;wtid_off < num_wtid; wtid_off++)
+                                      wts);
+    for (wtid_off = 0; wtid_off < num_wtid; wtid_off++)
       GNUNET_free (wts[wtid_off].coins);
     resume_track_transaction_with_response (tctx,
                                             MHD_HTTP_OK,
@@ -820,7 +850,7 @@ trace_coins (struct TrackTransactionContext *tctx)
   if (NULL != tcc)
   {
     if (0 != strcmp (tcc->exchange_url,
-		     tctx->current_exchange))
+                     tctx->current_exchange))
     {
       /* exchange changed, find matching one first! */
       tctx->eh = NULL;
@@ -830,13 +860,13 @@ trace_coins (struct TrackTransactionContext *tctx)
     }
     /* we are not done requesting WTIDs from the current
        exchange; do the next one */
-    tcc->dwh = TALER_EXCHANGE_track_transaction (tctx->eh,
-                                                 &tctx->mi->privkey,
-                                                 &tctx->h_wire,
-                                                 &tctx->h_contract_terms,
-                                                 &tcc->coin_pub,
-                                                 &wtid_cb,
-                                                 tcc);
+    tcc->dwh = TALER_EXCHANGE_deposits_get (tctx->eh,
+                                            &tctx->mi->privkey,
+                                            &tctx->h_wire,
+                                            &tctx->h_contract_terms,
+                                            &tcc->coin_pub,
+                                            &wtid_cb,
+                                            tcc);
     return;
   }
   tctx->current_exchange = NULL;
@@ -890,8 +920,9 @@ handle_track_transaction_timeout (void *cls)
   }
   resume_track_transaction_with_response (tctx,
                                           MHD_HTTP_SERVICE_UNAVAILABLE,
-                                          TMH_RESPONSE_make_error (TALER_EC_PAY_EXCHANGE_TIMEOUT,
-									    "exchange not reachable"));
+                                          TALER_MHD_make_error (
+                                            TALER_EC_PAY_EXCHANGE_TIMEOUT,
+                                            "exchange not reachable"));
 }
 
 
@@ -914,7 +945,7 @@ handle_track_transaction_timeout (void *cls)
  */
 static void
 transfer_cb (void *cls,
-	     const struct GNUNET_HashCode *h_contract_terms,
+             const struct GNUNET_HashCode *h_contract_terms,
              const struct TALER_CoinSpendPublicKeyP *coin_pub,
              const struct TALER_WireTransferIdentifierRawP *wtid,
              struct GNUNET_TIME_Absolute execution_time,
@@ -922,9 +953,8 @@ transfer_cb (void *cls,
 {
   struct TrackCoinContext *tcc = cls;
 
-  if (0 != memcmp (coin_pub,
-                   &tcc->coin_pub,
-                   sizeof (struct TALER_CoinSpendPublicKeyP)))
+  if (0 != GNUNET_memcmp (coin_pub,
+                          &tcc->coin_pub))
     return;
   tcc->wtid = *wtid;
   tcc->execution_time = execution_time;
@@ -948,7 +978,7 @@ static void
 coin_cb (void *cls,
          const struct GNUNET_HashCode *h_contract_terms,
          const struct TALER_CoinSpendPublicKeyP *coin_pub,
-	 const char *exchange_url,
+         const char *exchange_url,
          const struct TALER_Amount *amount_with_fee,
          const struct TALER_Amount *deposit_fee,
          const struct TALER_Amount *refund_fee,
@@ -975,9 +1005,9 @@ coin_cb (void *cls,
      will then set the wtid for the "current coin"
      context. */
   qs = db->find_transfers_by_hash (db->cls,
-				   h_contract_terms,
-				   &transfer_cb,
-				   tcc);
+                                   h_contract_terms,
+                                   &transfer_cb,
+                                   tcc);
   if (0 > qs)
   {
     GNUNET_break (0);
@@ -997,15 +1027,16 @@ find_exchange (struct TrackTransactionContext *tctx)
   struct TrackCoinContext *tcc = tctx->tcc_head;
 
   while ( (NULL != tcc) &&
-	  (GNUNET_YES == tcc->have_wtid) )
+          (GNUNET_YES == tcc->have_wtid) )
     tcc = tcc->next;
   if (NULL != tcc)
   {
     tctx->current_exchange = tcc->exchange_url;
     tctx->fo = TMH_EXCHANGES_find_exchange (tctx->current_exchange,
-					    NULL,
-					    &process_track_transaction_with_exchange,
-					    tctx);
+                                            NULL,
+                                            &
+                                            process_track_transaction_with_exchange,
+                                            tctx);
 
   }
   else
@@ -1023,6 +1054,7 @@ find_exchange (struct TrackTransactionContext *tctx)
  * @param[in,out] connection_cls the connection's closure (can be updated)
  * @param upload_data upload data
  * @param[in,out] upload_data_size number of bytes (left) in @a upload_data
+ * @param mi merchant backend instance, never NULL
  * @return MHD result code
  */
 int
@@ -1030,16 +1062,13 @@ MH_handler_track_transaction (struct TMH_RequestHandler *rh,
                               struct MHD_Connection *connection,
                               void **connection_cls,
                               const char *upload_data,
-                              size_t *upload_data_size)
+                              size_t *upload_data_size,
+                              struct MerchantInstance *mi)
 {
   struct TrackTransactionContext *tctx;
   const char *order_id;
-  const char *instance;
-  int ret;
   enum GNUNET_DB_QueryStatus qs;
-  struct GNUNET_HashCode h_instance;
   struct json_t *contract_terms;
-  char *last_session_id;
 
   if (NULL == *connection_cls)
   {
@@ -1056,6 +1085,8 @@ MH_handler_track_transaction (struct TMH_RequestHandler *rh,
 
   if (0 != tctx->response_code)
   {
+    int ret;
+
     /* We are *done* processing the request, just queue the response (!) */
     if (UINT_MAX == tctx->response_code)
     {
@@ -1088,60 +1119,48 @@ MH_handler_track_transaction (struct TMH_RequestHandler *rh,
                                           MHD_GET_ARGUMENT_KIND,
                                           "order_id");
   if (NULL == order_id)
-    return TMH_RESPONSE_reply_arg_missing (connection,
-                                           TALER_EC_PARAMETER_MISSING,
-					   "order_id");
-  instance = MHD_lookup_connection_value (connection,
-                                          MHD_GET_ARGUMENT_KIND,
-                                          "instance");
-  if (NULL == instance)
-    instance = "default";
+    return TALER_MHD_reply_with_error (connection,
+                                       MHD_HTTP_BAD_REQUEST,
+                                       TALER_EC_PARAMETER_MISSING,
+                                       "order_id");
 
-  GNUNET_CRYPTO_hash (instance,
-                      strlen (instance),
-                      &h_instance);
-
-  tctx->mi = GNUNET_CONTAINER_multihashmap_get (by_id_map,
-                                                &h_instance);
+  tctx->mi = mi;
   GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
               "Tracking on behalf of instance '%s'\n",
-              instance);
+              mi->id);
+
 
-  if (NULL == tctx->mi)
-    return TMH_RESPONSE_reply_not_found (connection,
-					 TALER_EC_TRACK_TRANSACTION_INSTANCE_UNKNOWN,
-					 "unknown instance");
-  
   /* Map order id to contract terms; the objective is to get
      the contract term's hashcode so as to retrieve all the
      coins which have been deposited for it. */
+  db->preflight (db->cls);
   qs = db->find_contract_terms (db->cls,
-				&contract_terms,
-                                &last_session_id,
-				order_id,
-				&tctx->mi->pubkey);
+                                &contract_terms,
+                                order_id,
+                                &tctx->mi->pubkey);
   if (0 > qs)
   {
     GNUNET_break (GNUNET_DB_STATUS_SOFT_ERROR != qs);
-    return TMH_RESPONSE_reply_internal_error (connection,
-					      TALER_EC_TRACK_TRANSACTION_DB_FETCH_TRANSACTION_ERROR,
-                                              "Database error finding contract terms");
+    return TALER_MHD_reply_with_error (connection,
+                                       MHD_HTTP_INTERNAL_SERVER_ERROR,
+                                       TALER_EC_TRACK_TRANSACTION_DB_FETCH_TRANSACTION_ERROR,
+                                       "Database error finding contract terms");
   }
   if (GNUNET_DB_STATUS_SUCCESS_NO_RESULTS == qs)
-    return TMH_RESPONSE_reply_not_found (connection,
-					 TALER_EC_PROPOSAL_LOOKUP_NOT_FOUND,
-					 "Given order_id doesn't map to any proposal");
-
-  GNUNET_free (last_session_id);
+    return TALER_MHD_reply_with_error (connection,
+                                       MHD_HTTP_NOT_FOUND,
+                                       TALER_EC_PROPOSAL_LOOKUP_NOT_FOUND,
+                                       "Given order_id doesn't map to any proposal");
 
   if (GNUNET_OK !=
       TALER_JSON_hash (contract_terms,
                        &tctx->h_contract_terms))
   {
     json_decref (contract_terms);
-    return TMH_RESPONSE_reply_internal_error (connection,
-					      TALER_EC_INTERNAL_LOGIC_ERROR,
-                                              "Failed to hash contract terms");
+    return TALER_MHD_reply_with_error (connection,
+                                       MHD_HTTP_INTERNAL_SERVER_ERROR,
+                                       TALER_EC_INTERNAL_LOGIC_ERROR,
+                                       "Failed to hash contract terms");
   }
 
   {
@@ -1152,9 +1171,9 @@ MH_handler_track_transaction (struct TMH_RequestHandler *rh,
                                       &tctx->timestamp),
       TALER_JSON_spec_amount ("amount",
                               &tctx->total_amount),
-      GNUNET_JSON_spec_fixed_auto ("H_wire",
+      GNUNET_JSON_spec_fixed_auto ("h_wire",
                                    &tctx->h_wire),
-      GNUNET_JSON_spec_end()
+      GNUNET_JSON_spec_end ()
     };
 
     if (GNUNET_YES !=
@@ -1165,9 +1184,10 @@ MH_handler_track_transaction (struct TMH_RequestHandler *rh,
       GNUNET_break (0);
       GNUNET_JSON_parse_free (spec);
       json_decref (contract_terms);
-      return TMH_RESPONSE_reply_internal_error (connection,
-                                                TALER_EC_INTERNAL_LOGIC_ERROR,
-                                                "Failed to parse contract terms from DB");
+      return TALER_MHD_reply_with_error (connection,
+                                         MHD_HTTP_INTERNAL_SERVER_ERROR,
+                                         TALER_EC_INTERNAL_LOGIC_ERROR,
+                                         "Failed to parse contract terms from DB");
     }
     json_decref (contract_terms);
   }
@@ -1178,24 +1198,26 @@ MH_handler_track_transaction (struct TMH_RequestHandler *rh,
   /* Find coins which have been deposited for this contract,
      and retrieve the wtid for each one. */
   qs = db->find_payments (db->cls,
-			  &tctx->h_contract_terms,
-			  &tctx->mi->pubkey,
-			  &coin_cb,
-			  tctx);
+                          &tctx->h_contract_terms,
+                          &tctx->mi->pubkey,
+                          &coin_cb,
+                          tctx);
   if ( (0 > qs) ||
        (0 > tctx->qs) )
   {
     GNUNET_break (GNUNET_DB_STATUS_SOFT_ERROR != qs);
     GNUNET_break (GNUNET_DB_STATUS_SOFT_ERROR != tctx->qs);
-    return TMH_RESPONSE_reply_internal_error (connection,
-                                              TALER_EC_TRACK_TRANSACTION_DB_FETCH_PAYMENT_ERROR,
-					      "Database error: failed to find payment data");
+    return TALER_MHD_reply_with_error (connection,
+                                       MHD_HTTP_INTERNAL_SERVER_ERROR,
+                                       TALER_EC_TRACK_TRANSACTION_DB_FETCH_PAYMENT_ERROR,
+                                       "Database error: failed to find payment data");
   }
   if (GNUNET_DB_STATUS_SUCCESS_NO_RESULTS == qs)
   {
-    return TMH_RESPONSE_reply_not_found (connection,
-					 TALER_EC_TRACK_TRANSACTION_DB_NO_DEPOSITS_ERROR,
-                                         "deposit data not found");
+    return TALER_MHD_reply_with_error (connection,
+                                       MHD_HTTP_NOT_FOUND,
+                                       TALER_EC_TRACK_TRANSACTION_DB_NO_DEPOSITS_ERROR,
+                                       "deposit data not found");
   }
   *connection_cls = tctx;
 
@@ -1204,8 +1226,8 @@ MH_handler_track_transaction (struct TMH_RequestHandler *rh,
   MHD_suspend_connection (connection);
   tctx->timeout_task
     = GNUNET_SCHEDULER_add_delayed (TRACK_TIMEOUT,
-				    &handle_track_transaction_timeout,
-				    tctx);
+                                    &handle_track_transaction_timeout,
+                                    tctx);
   find_exchange (tctx);
   return MHD_YES;
 }
