@@ -25,17 +25,16 @@
 #include <taler/taler_json_lib.h>
 #include "taler-merchant-httpd.h"
 #include "taler-merchant-httpd_mhd.h"
-#include "taler-merchant-httpd_parsing.h"
 #include "taler-merchant-httpd_auditors.h"
 #include "taler-merchant-httpd_exchanges.h"
-#include "taler-merchant-httpd_responses.h"
 #include "taler-merchant-httpd_track-transfer.h"
 
 
 /**
  * How long to wait before giving up processing with the exchange?
  */
-#define TRACK_TIMEOUT (GNUNET_TIME_relative_multiply (GNUNET_TIME_UNIT_SECONDS, 30))
+#define TRACK_TIMEOUT (GNUNET_TIME_relative_multiply (GNUNET_TIME_UNIT_SECONDS, \
+                                                      30))
 
 /**
  * How often do we retry the simple INSERT database transaction?
@@ -61,7 +60,7 @@ struct TrackTransferContext
   /**
    * Handle for the /wire/transfers request.
    */
-  struct TALER_EXCHANGE_TrackTransferHandle *wdh;
+  struct TALER_EXCHANGE_TransfersGetHandle *wdh;
 
   /**
    * For which merchant instance is this tracking request?
@@ -180,7 +179,7 @@ free_transfer_track_context (struct TrackTransferContext *rctx)
   }
   if (NULL != rctx->wdh)
   {
-    TALER_EXCHANGE_track_transfer_cancel (rctx->wdh);
+    TALER_EXCHANGE_transfers_get_cancel (rctx->wdh);
     rctx->wdh = NULL;
   }
   if (NULL != rctx->url)
@@ -196,6 +195,7 @@ free_transfer_track_context (struct TrackTransferContext *rctx)
   GNUNET_free (rctx);
 }
 
+
 /**
  * Callback that frees all the elements in the hashmap
  *
@@ -238,6 +238,7 @@ build_deposits_response (void *cls,
   json_t *contract_terms;
   json_t *order_id;
 
+  db->preflight (db->cls);
   if (GNUNET_OK !=
       db->find_contract_terms_from_hash (db->cls,
                                          &contract_terms,
@@ -258,8 +259,10 @@ build_deposits_response (void *cls,
   }
   element = json_pack ("{s:O, s:o, s:o}",
                        "order_id", order_id,
-                       "deposit_value", TALER_JSON_from_amount (&entry->deposit_value),
-                       "deposit_fee", TALER_JSON_from_amount (&entry->deposit_fee));
+                       "deposit_value", TALER_JSON_from_amount (
+                         &entry->deposit_value),
+                       "deposit_fee", TALER_JSON_from_amount (
+                         &entry->deposit_fee));
   json_decref (contract_terms);
   if (NULL == element)
   {
@@ -322,8 +325,8 @@ transform_response (const json_t *result,
                                     &h_key);
 
     if (NULL != (current_entry =
-		 GNUNET_CONTAINER_multihashmap_get (map,
-						    &h_key)))
+                   GNUNET_CONTAINER_multihashmap_get (map,
+                                                      &h_key)))
     {
       /* The map already knows this h_contract_terms*/
       if ( (GNUNET_SYSERR ==
@@ -373,7 +376,7 @@ transform_response (const json_t *result,
                        "deposits_sums",
                        rctx->deposits_response);
   rctx->deposits_response = NULL;
- cleanup:
+cleanup:
   GNUNET_CONTAINER_multihashmap_iterate (map,
                                          &hashmap_free,
                                          NULL);
@@ -443,7 +446,7 @@ static void
 check_transfer (void *cls,
                 const struct GNUNET_HashCode *h_contract_terms,
                 const struct TALER_CoinSpendPublicKeyP *coin_pub,
-		const char *exchange_url,
+                const char *exchange_url,
                 const struct TALER_Amount *amount_with_fee,
                 const struct TALER_Amount *deposit_fee,
                 const struct TALER_Amount *refund_fee,
@@ -466,16 +469,25 @@ check_transfer (void *cls,
     rctx->check_transfer_result = GNUNET_SYSERR;
     /* Build the `TrackTransferConflictDetails` */
     rctx->response
-      = TMH_RESPONSE_make_json_pack ("{s:I, s:s, s:o, s:I, s:o, s:o, s:s, s:o, s:o}",
-                                     "code", (json_int_t) TALER_EC_TRACK_TRANSFER_CONFLICTING_REPORTS,
-                                     "hint", "disagreement about deposit valuation",
-                                     "exchange_deposit_proof", exchange_proof,
-                                     "conflict_offset", (json_int_t) rctx->current_offset,
-                                     "exchange_transfer_proof", rctx->original_response,
-                                     "coin_pub", GNUNET_JSON_from_data_auto (coin_pub),
-                                     "h_contract_terms", GNUNET_JSON_from_data_auto (&ttd->h_contract_terms),
-                                     "amount_with_fee", TALER_JSON_from_amount (amount_with_fee),
-                                     "deposit_fee", TALER_JSON_from_amount (deposit_fee));
+      = TALER_MHD_make_json_pack (
+          "{s:I, s:s, s:o, s:I, s:o, s:o, s:s, s:o, s:o}",
+          "code",
+          (json_int_t) TALER_EC_TRACK_TRANSFER_CONFLICTING_REPORTS,
+          "hint",
+          "disagreement about deposit valuation",
+          "exchange_deposit_proof", exchange_proof,
+          "conflict_offset",
+          (json_int_t) rctx->current_offset,
+          "exchange_transfer_proof",
+          rctx->original_response,
+          "coin_pub", GNUNET_JSON_from_data_auto (
+            coin_pub),
+          "h_contract_terms",
+          GNUNET_JSON_from_data_auto (&ttd->h_contract_terms),
+          "amount_with_fee", TALER_JSON_from_amount (
+            amount_with_fee),
+          "deposit_fee", TALER_JSON_from_amount (
+            deposit_fee));
     return;
   }
   rctx->check_transfer_result = GNUNET_OK;
@@ -500,9 +512,9 @@ check_transfer (void *cls,
  */
 static int
 check_wire_fee (struct TrackTransferContext *rctx,
-		const json_t *json,
-		struct GNUNET_TIME_Absolute execution_time,
-		const struct TALER_Amount *wire_fee)
+                const json_t *json,
+                struct GNUNET_TIME_Absolute execution_time,
+                const struct TALER_Amount *wire_fee)
 {
   const struct TALER_MasterPublicKeyP *master_pub;
   struct GNUNET_HashCode h_wire_method;
@@ -522,46 +534,56 @@ check_wire_fee (struct TrackTransferContext *rctx,
   }
   master_pub = &keys->master_pub;
   GNUNET_CRYPTO_hash (rctx->wire_method,
-		      strlen (rctx->wire_method) + 1,
-		      &h_wire_method);
+                      strlen (rctx->wire_method) + 1,
+                      &h_wire_method);
+  db->preflight (db->cls);
   qs = db->lookup_wire_fee (db->cls,
-			    master_pub,
-			    &h_wire_method,
-			    execution_time,
-			    &expected_fee,
-			    &closing_fee,
-			    &start_date,
-			    &end_date,
-			    &master_sig);
+                            master_pub,
+                            &h_wire_method,
+                            execution_time,
+                            &expected_fee,
+                            &closing_fee,
+                            &start_date,
+                            &end_date,
+                            &master_sig);
   if (0 >= qs)
   {
     GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
-		"Failed to find wire fee for `%s' and method `%s' at %s in DB, accepting blindly that the fee is %s\n",
-		TALER_B2S (master_pub),
-		rctx->wire_method,
-		GNUNET_STRINGS_absolute_time_to_string (execution_time),
-		TALER_amount2s (wire_fee));
+                "Failed to find wire fee for `%s' and method `%s' at %s in DB, accepting blindly that the fee is %s\n",
+                TALER_B2S (master_pub),
+                rctx->wire_method,
+                GNUNET_STRINGS_absolute_time_to_string (execution_time),
+                TALER_amount2s (wire_fee));
     return GNUNET_NO;
   }
   if (0 <= TALER_amount_cmp (&expected_fee,
-			     wire_fee))
+                             wire_fee))
     return GNUNET_OK; /* expected_fee >= wire_fee */
 
   /* Wire fee check failed, export proof to client */
   resume_track_transfer_with_response
     (rctx,
-     MHD_HTTP_INTERNAL_SERVER_ERROR,
-     TMH_RESPONSE_make_json_pack ("{s:I, s:o, s:o, s:o, s:o, s:o, s:o, s:o, s:o, s:O}",
-				  "code", (json_int_t) TALER_EC_TRACK_TRANSFER_JSON_BAD_WIRE_FEE,
-				  "wire_fee", TALER_JSON_from_amount (wire_fee),
-				  "execution_time", GNUNET_JSON_from_time_abs (execution_time),
-				  "expected_wire_fee", TALER_JSON_from_amount (&expected_fee),
-				  "expected_closing_fee", TALER_JSON_from_amount (&closing_fee),
-				  "start_date", GNUNET_JSON_from_time_abs (start_date),
-				  "end_date", GNUNET_JSON_from_time_abs (end_date),
-				  "master_sig", GNUNET_JSON_from_data_auto (&master_sig),
-				  "master_pub", GNUNET_JSON_from_data_auto (master_pub),
-				  "json", json));
+    MHD_HTTP_FAILED_DEPENDENCY,
+    TALER_MHD_make_json_pack (
+      "{s:I, s:o, s:o, s:o, s:o, s:o, s:o, s:o, s:o, s:O}",
+      "code",
+      (json_int_t) TALER_EC_TRACK_TRANSFER_JSON_BAD_WIRE_FEE,
+      "wire_fee", TALER_JSON_from_amount (wire_fee),
+      "execution_time", GNUNET_JSON_from_time_abs (
+        execution_time),
+      "expected_wire_fee", TALER_JSON_from_amount (
+        &expected_fee),
+      "expected_closing_fee",
+      TALER_JSON_from_amount (&closing_fee),
+      "start_date", GNUNET_JSON_from_time_abs (
+        start_date),
+      "end_date", GNUNET_JSON_from_time_abs (
+        end_date),
+      "master_sig", GNUNET_JSON_from_data_auto (
+        &master_sig),
+      "master_pub", GNUNET_JSON_from_data_auto (
+        master_pub),
+      "json", json));
   return GNUNET_SYSERR;
 }
 
@@ -587,7 +609,7 @@ check_wire_fee (struct TrackTransferContext *rctx,
 static void
 wire_transfer_cb (void *cls,
                   unsigned int http_status,
-		  enum TALER_ErrorCode ec,
+                  enum TALER_ErrorCode ec,
                   const struct TALER_ExchangePublicKeyP *exchange_pub,
                   const json_t *json,
                   const struct GNUNET_HashCode *h_wire,
@@ -609,22 +631,26 @@ wire_transfer_cb (void *cls,
   {
     resume_track_transfer_with_response
       (rctx,
-       MHD_HTTP_FAILED_DEPENDENCY,
-       TMH_RESPONSE_make_json_pack ("{s:I, s:I, s:I, s:O}",
-				    "code", (json_int_t) TALER_EC_TRACK_TRANSFER_EXCHANGE_ERROR,
-                                    "exchange-code", (json_int_t) ec,
-                                    "exchange-http-status", (json_int_t) http_status,
-                                    "details", json));
+      MHD_HTTP_FAILED_DEPENDENCY,
+      TALER_MHD_make_json_pack ("{s:I, s:I, s:I, s:O}",
+                                "code",
+                                (json_int_t)
+                                TALER_EC_TRACK_TRANSFER_EXCHANGE_ERROR,
+                                "exchange-code", (json_int_t) ec,
+                                "exchange-http-status",
+                                (json_int_t) http_status,
+                                "details", json));
     return;
   }
-  for (unsigned int i=0;i<MAX_RETRIES;i++)
+  for (unsigned int i = 0; i<MAX_RETRIES; i++)
   {
+    db->preflight (db->cls);
     qs = db->store_transfer_to_proof (db->cls,
-				      rctx->url,
-				      &rctx->wtid,
-				      execution_time,
-				      exchange_pub,
-				      json);
+                                      rctx->url,
+                                      &rctx->wtid,
+                                      execution_time,
+                                      exchange_pub,
+                                      json);
     if (GNUNET_DB_STATUS_SOFT_ERROR != qs)
       break;
   }
@@ -636,19 +662,22 @@ wire_transfer_cb (void *cls,
     GNUNET_break (GNUNET_DB_STATUS_HARD_ERROR == qs);
     resume_track_transfer_with_response
       (rctx,
-       MHD_HTTP_INTERNAL_SERVER_ERROR,
-       TMH_RESPONSE_make_json_pack ("{s:I, s:s}",
-				    "code", (json_int_t) TALER_EC_TRACK_TRANSFER_DB_STORE_TRANSFER_ERROR,
-                                    "details", "failed to store response from exchange to local database"));
+      MHD_HTTP_INTERNAL_SERVER_ERROR,
+      TALER_MHD_make_json_pack ("{s:I, s:s}",
+                                "code",
+                                (json_int_t)
+                                TALER_EC_TRACK_TRANSFER_DB_STORE_TRANSFER_ERROR,
+                                "details",
+                                "failed to store response from exchange to local database"));
     return;
   }
   rctx->original_response = json;
 
   if (GNUNET_SYSERR ==
       check_wire_fee (rctx,
-		      json,
-		      execution_time,
-		      wire_fee))
+                      json,
+                      execution_time,
+                      wire_fee))
     return;
 
   /* Now we want to double-check that any (Taler coin) deposit
@@ -658,32 +687,36 @@ wire_transfer_cb (void *cls,
    *
    * details_length is how many (Taler coin) deposits have been
    * aggregated into _this_ wire transfer.
-   */
-  for (unsigned int i=0;i<details_length;i++)
+   *///
+  for (unsigned int i = 0; i<details_length; i++)
   {
     rctx->current_offset = i;
     rctx->current_detail = &details[i];
     /* Set the coin as "never seen" before. */
     rctx->check_transfer_result = GNUNET_NO;
+    db->preflight (db->cls);
     qs = db->find_payments_by_hash_and_coin (db->cls,
-					     &details[i].h_contract_terms,
-					     &rctx->mi->pubkey,
-					     &details[i].coin_pub,
-					     &check_transfer,
-					     rctx);
+                                             &details[i].h_contract_terms,
+                                             &rctx->mi->pubkey,
+                                             &details[i].coin_pub,
+                                             &check_transfer,
+                                             rctx);
     if (0 > qs)
     {
       /* single, read-only SQL statements should never cause
-	 serialization problems */
+         serialization problems */
       GNUNET_break (GNUNET_DB_STATUS_SOFT_ERROR != qs);
       /* Always report on hard error as well to enable diagnostics */
       GNUNET_break (GNUNET_DB_STATUS_HARD_ERROR == qs);
       resume_track_transfer_with_response
         (rctx,
-         MHD_HTTP_INTERNAL_SERVER_ERROR,
-         TMH_RESPONSE_make_json_pack ("{s:I, s:s}",
-				      "code", (json_int_t) TALER_EC_TRACK_TRANSFER_DB_FETCH_DEPOSIT_ERROR,
-                                      "details", "failed to obtain deposit data from local database"));
+        MHD_HTTP_INTERNAL_SERVER_ERROR,
+        TALER_MHD_make_json_pack ("{s:I, s:s}",
+                                  "code",
+                                  (json_int_t)
+                                  TALER_EC_TRACK_TRANSFER_DB_FETCH_DEPOSIT_ERROR,
+                                  "details",
+                                  "failed to obtain deposit data from local database"));
       return;
     }
     if (GNUNET_DB_STATUS_SUCCESS_NO_RESULTS == qs)
@@ -702,12 +735,14 @@ wire_transfer_cb (void *cls,
       GNUNET_break (0);
       resume_track_transfer_with_response
         (rctx,
-         MHD_HTTP_INTERNAL_SERVER_ERROR,
-         TMH_RESPONSE_make_json_pack ("{s:I, s:s, s:I, s:s}",
-				      "code", (json_int_t) TALER_EC_TRACK_TRANSFER_DB_INTERNAL_LOGIC_ERROR,
-                                      "details", "internal logic error",
-                                      "line", (json_int_t) __LINE__,
-                                      "file", __FILE__));
+        MHD_HTTP_INTERNAL_SERVER_ERROR,
+        TALER_MHD_make_json_pack ("{s:I, s:s, s:I, s:s}",
+                                  "code",
+                                  (json_int_t)
+                                  TALER_EC_TRACK_TRANSFER_DB_INTERNAL_LOGIC_ERROR,
+                                  "details", "internal logic error",
+                                  "line", (json_int_t) __LINE__,
+                                  "file", __FILE__));
       return;
     }
     if (GNUNET_SYSERR == rctx->check_transfer_result)
@@ -717,21 +752,22 @@ wire_transfer_cb (void *cls,
       GNUNET_assert (NULL != rctx->response);
       resume_track_transfer_with_response
         (rctx,
-         MHD_HTTP_FAILED_DEPENDENCY,
-         rctx->response);
+        MHD_HTTP_FAILED_DEPENDENCY,
+        rctx->response);
       rctx->response = NULL;
       return;
     }
     /* Response is consistent with the /deposit we made,
        remember it for future reference */
-    for (unsigned int i=0;i<MAX_RETRIES;i++)
+    for (unsigned int r = 0; r<MAX_RETRIES; r++)
     {
+      db->preflight (db->cls);
       qs = db->store_coin_to_transfer (db->cls,
-				       &details[i].h_contract_terms,
-				       &details[i].coin_pub,
-				       &rctx->wtid);
+                                       &details[i].h_contract_terms,
+                                       &details[i].coin_pub,
+                                       &rctx->wtid);
       if (GNUNET_DB_STATUS_SOFT_ERROR != qs)
-	break;
+        break;
     }
     if (0 > qs)
     {
@@ -741,10 +777,13 @@ wire_transfer_cb (void *cls,
       GNUNET_break (GNUNET_DB_STATUS_HARD_ERROR == qs);
       resume_track_transfer_with_response
         (rctx,
-         MHD_HTTP_INTERNAL_SERVER_ERROR,
-         TMH_RESPONSE_make_json_pack ("{s:I, s:s}",
-				      "code", (json_int_t) TALER_EC_TRACK_TRANSFER_DB_STORE_COIN_ERROR,
-                                      "details", "failed to store response from exchange to local database"));
+        MHD_HTTP_INTERNAL_SERVER_ERROR,
+        TALER_MHD_make_json_pack ("{s:I, s:s}",
+                                  "code",
+                                  (json_int_t)
+                                  TALER_EC_TRACK_TRANSFER_DB_STORE_COIN_ERROR,
+                                  "details",
+                                  "failed to store response from exchange to local database"));
       return;
     }
   }
@@ -754,20 +793,20 @@ wire_transfer_cb (void *cls,
               "About to call tracks transformator.\n");
 
   if (NULL == (jresponse =
-	       transform_response (json,
-				   rctx)))
+                 transform_response (json,
+                                     rctx)))
   {
     resume_track_transfer_with_response
       (rctx,
-       MHD_HTTP_INTERNAL_SERVER_ERROR,
-       TMH_RESPONSE_make_error (TALER_EC_TRACK_TRANSFER_JSON_RESPONSE_ERROR,
-				"Fail to elaborate the response."));
+      MHD_HTTP_INTERNAL_SERVER_ERROR,
+      TALER_MHD_make_error (TALER_EC_TRACK_TRANSFER_JSON_RESPONSE_ERROR,
+                            "Fail to elaborate the response."));
     return;
   }
 
   resume_track_transfer_with_response (rctx,
                                        MHD_HTTP_OK,
-                                       TMH_RESPONSE_make_json (jresponse));
+                                       TALER_MHD_make_json (jresponse));
   json_decref (jresponse);
 }
 
@@ -790,19 +829,22 @@ process_track_transfer_with_exchange (void *cls,
 
   rctx->fo = NULL;
   rctx->eh = eh;
-  rctx->wdh = TALER_EXCHANGE_track_transfer (eh,
-                                             &rctx->wtid,
-                                             &wire_transfer_cb,
-                                             rctx);
+  rctx->wdh = TALER_EXCHANGE_transfers_get (eh,
+                                            &rctx->wtid,
+                                            &wire_transfer_cb,
+                                            rctx);
   if (NULL == rctx->wdh)
   {
     GNUNET_break (0);
     resume_track_transfer_with_response
       (rctx,
-       MHD_HTTP_INTERNAL_SERVER_ERROR,
-       TMH_RESPONSE_make_json_pack ("{s:I, s:s}",
-				    "code", (json_int_t) TALER_EC_TRACK_TRANSFER_REQUEST_ERROR,
-                                    "error", "failed to run /track/transfer on exchange"));
+      MHD_HTTP_INTERNAL_SERVER_ERROR,
+      TALER_MHD_make_json_pack ("{s:I, s:s}",
+                                "code",
+                                (json_int_t)
+                                TALER_EC_TRACK_TRANSFER_REQUEST_ERROR,
+                                "error",
+                                "failed to run /track/transfer on exchange"));
   }
 }
 
@@ -828,8 +870,9 @@ handle_track_transfer_timeout (void *cls)
   }
   resume_track_transfer_with_response (rctx,
                                        MHD_HTTP_SERVICE_UNAVAILABLE,
-                                       TMH_RESPONSE_make_error (TALER_EC_TRACK_TRANSFER_EXCHANGE_TIMEOUT,
-									 "exchange not reachable"));
+                                       TALER_MHD_make_error (
+                                         TALER_EC_TRACK_TRANSFER_EXCHANGE_TIMEOUT,
+                                         "exchange not reachable"));
 }
 
 
@@ -850,18 +893,18 @@ proof_cb (void *cls,
   json_t *transformed_response;
 
   if (NULL == (transformed_response =
-	       transform_response (proof,
-				   rctx)))
+                 transform_response (proof,
+                                     rctx)))
   {
     rctx->response_code = MHD_HTTP_INTERNAL_SERVER_ERROR;
     rctx->response
-      = TMH_RESPONSE_make_error (TALER_EC_TRACK_TRANSFER_JSON_RESPONSE_ERROR,
-				 "Fail to elaborate response.");
+      = TALER_MHD_make_error (TALER_EC_TRACK_TRANSFER_JSON_RESPONSE_ERROR,
+                              "Fail to elaborate response.");
     return;
   }
 
   rctx->response_code = MHD_HTTP_OK;
-  rctx->response = TMH_RESPONSE_make_json (transformed_response);
+  rctx->response = TALER_MHD_make_json (transformed_response);
   json_decref (transformed_response);
 }
 
@@ -876,6 +919,7 @@ proof_cb (void *cls,
  * @param[in,out] connection_cls the connection's closure (can be updated)
  * @param upload_data upload data
  * @param[in,out] upload_data_size number of bytes (left) in @a upload_data
+ * @param mi merchant backend instance, never NULL
  * @return MHD result code
  */
 int
@@ -883,12 +927,12 @@ MH_handler_track_transfer (struct TMH_RequestHandler *rh,
                            struct MHD_Connection *connection,
                            void **connection_cls,
                            const char *upload_data,
-                           size_t *upload_data_size)
+                           size_t *upload_data_size,
+                           struct MerchantInstance *mi)
 {
   struct TrackTransferContext *rctx;
   const char *str;
   const char *url;
-  const char *instance_str;
   const char *wire_method;
   int ret;
   enum GNUNET_DB_QueryStatus qs;
@@ -941,76 +985,72 @@ MH_handler_track_transfer (struct TMH_RequestHandler *rh,
                                      MHD_GET_ARGUMENT_KIND,
                                      "exchange");
   if (NULL == url)
-    return TMH_RESPONSE_reply_arg_missing (connection,
-					   TALER_EC_PARAMETER_MISSING,
-                                           "exchange");
+    return TALER_MHD_reply_with_error (connection,
+                                       MHD_HTTP_BAD_REQUEST,
+                                       TALER_EC_PARAMETER_MISSING,
+                                       "exchange");
   rctx->url = GNUNET_strdup (url);
 
+  /* FIXME: change again: we probably don't want the wire_method
+     but rather the _account_ (section) here! */
   wire_method = MHD_lookup_connection_value (connection,
-					     MHD_GET_ARGUMENT_KIND,
-					     "wire_method");
+                                             MHD_GET_ARGUMENT_KIND,
+                                             "wire_method");
   if (NULL == wire_method)
   {
     if (1)
     {
       /* temporary work-around until demo is adjusted... */
       GNUNET_break (0);
-      wire_method = "test";
+      wire_method = "x-taler-bank";
       GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-		  "Client needs fixing, see API change for #4943!\n");
+                  "Client needs fixing, see API change for #4943!\n");
     }
     else
-      return TMH_RESPONSE_reply_arg_missing (connection,
-					     TALER_EC_PARAMETER_MISSING,
-					     "wire_method");
+      return TALER_MHD_reply_with_error (connection,
+                                         MHD_HTTP_BAD_REQUEST,
+                                         TALER_EC_PARAMETER_MISSING,
+                                         "wire_method");
   }
   rctx->wire_method = GNUNET_strdup (wire_method);
-
-  instance_str = MHD_lookup_connection_value (connection,
-                                              MHD_GET_ARGUMENT_KIND,
-                                              "instance");
-  if (NULL == instance_str)
-    instance_str = "default";
-
-  rctx->mi = TMH_lookup_instance (instance_str);
-  if (NULL == rctx->mi)
-    return TMH_RESPONSE_reply_not_found (connection,
-					 TALER_EC_TRACK_TRANSFER_INSTANCE_UNKNOWN,
-                                         "instance unknown");
+  rctx->mi = mi;
   str = MHD_lookup_connection_value (connection,
                                      MHD_GET_ARGUMENT_KIND,
                                      "wtid");
   if (NULL == str)
-    return TMH_RESPONSE_reply_arg_missing (connection,
-					   TALER_EC_PARAMETER_MISSING,
-                                           "wtid");
+    return TALER_MHD_reply_with_error (connection,
+                                       MHD_HTTP_BAD_REQUEST,
+                                       TALER_EC_PARAMETER_MISSING,
+                                       "wtid");
   if (GNUNET_OK !=
       GNUNET_STRINGS_string_to_data (str,
                                      strlen (str),
                                      &rctx->wtid,
                                      sizeof (rctx->wtid)))
   {
-    return TMH_RESPONSE_reply_arg_invalid (connection,
-					   TALER_EC_PARAMETER_MALFORMED,
-                                           "wtid");
+    return TALER_MHD_reply_with_error (connection,
+                                       MHD_HTTP_BAD_REQUEST,
+                                       TALER_EC_PARAMETER_MALFORMED,
+                                       "wtid");
   }
 
   /* Check if reply is already in database! */
   db->preflight (db->cls);
   qs = db->find_proof_by_wtid (db->cls,
-			       rctx->url,
-			       &rctx->wtid,
-			       &proof_cb,
-			       rctx);
+                               rctx->url,
+                               &rctx->wtid,
+                               &proof_cb,
+                               rctx);
   if (0 > qs)
   {
     /* Simple select queries should not cause serialization issues */
     GNUNET_break (GNUNET_DB_STATUS_SOFT_ERROR != qs);
     /* Always report on hard error as well to enable diagnostics */
     GNUNET_break (GNUNET_DB_STATUS_HARD_ERROR == qs);
-    return TMH_RESPONSE_reply_internal_error (connection,
-					      TALER_EC_TRACK_TRANSFER_DB_FETCH_FAILED,
-					      "Fail to query database about proofs");
+    return TALER_MHD_reply_with_error (connection,
+                                       MHD_HTTP_INTERNAL_SERVER_ERROR,
+                                       TALER_EC_TRACK_TRANSFER_DB_FETCH_DEPOSIT_ERROR,
+                                       "Fail to query database about proofs");
   }
   if (0 != rctx->response_code)
   {
@@ -1043,4 +1083,5 @@ MH_handler_track_transfer (struct TMH_RequestHandler *rh,
   return MHD_YES;
 }
 
+
 /* end of taler-merchant-httpd_track-transfer.c */
