@@ -28,15 +28,14 @@
 /**
  * Initialize the plugin.
  *
- * @param cfg configuration to use
+ * @param[in,out] cfg configuration to use
  * @return #GNUNET_OK on success
  */
 struct TALER_MERCHANTDB_Plugin *
-TALER_MERCHANTDB_plugin_load (const struct GNUNET_CONFIGURATION_Handle *cfg)
+TALER_MERCHANTDB_plugin_load (struct GNUNET_CONFIGURATION_Handle *cfg)
 {
   char *plugin_name;
   char *lib_name;
-  struct GNUNET_CONFIGURATION_Handle *cfg_dup;
   struct TALER_MERCHANTDB_Plugin *plugin;
 
   if (GNUNET_SYSERR ==
@@ -54,13 +53,12 @@ TALER_MERCHANTDB_plugin_load (const struct GNUNET_CONFIGURATION_Handle *cfg)
                           "libtaler_plugin_merchantdb_%s",
                           plugin_name);
   GNUNET_free (plugin_name);
-  cfg_dup = GNUNET_CONFIGURATION_dup (cfg);
-  plugin = GNUNET_PLUGIN_load (lib_name, cfg_dup);
+  plugin = GNUNET_PLUGIN_load (lib_name,
+                               cfg);
   if (NULL != plugin)
     plugin->library_name = lib_name;
   else
     GNUNET_free (lib_name);
-  GNUNET_CONFIGURATION_destroy (cfg_dup);
   return plugin;
 }
 
@@ -104,8 +102,8 @@ plugin_init ()
   err = lt_dlinit ();
   if (err > 0)
   {
-    FPRINTF (stderr,
-             _("Initialization of plugin mechanism failed: %s!\n"),
+    fprintf (stderr,
+             _ ("Initialization of plugin mechanism failed: %s!\n"),
              lt_dlerror ());
     return;
   }
