@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  (C) 2014-2017 INRIA
+  (C) 2014-2017 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU Lesser General Public License as published by the Free Software
@@ -29,7 +29,7 @@
 
 #define FAILIF(cond)                            \
   do {                                          \
-    if (!(cond)){ break;}                       \
+    if (! (cond)) { break;}                       \
     GNUNET_break (0);                           \
     goto drop;                                  \
   } while (0)
@@ -206,14 +206,14 @@ static json_t *contract_terms_future;
  */
 static void
 refund_cb (void *cls,
-	   const struct TALER_CoinSpendPublicKeyP *coin_pub,
-	   uint64_t rtransaction_id,
-	   const char *reason,
-	   const struct TALER_Amount *refund_amount,
-	   const struct TALER_Amount *refund_fee)
+           const struct TALER_CoinSpendPublicKeyP *coin_pub,
+           uint64_t rtransaction_id,
+           const char *reason,
+           const struct TALER_Amount *refund_amount,
+           const struct TALER_Amount *refund_fee)
 {
   GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
-	      "refund_cb\n");
+              "refund_cb\n");
   /* FIXME, more logic here? */
 }
 
@@ -255,19 +255,17 @@ static void
 deposit_cb (void *cls,
             const struct GNUNET_HashCode *ah_contract_terms,
             const struct TALER_CoinSpendPublicKeyP *acoin_pub,
-	    const char *aexchange_url,
+            const char *aexchange_url,
             const struct TALER_Amount *aamount_with_fee,
             const struct TALER_Amount *adeposit_fee,
             const struct TALER_Amount *arefund_fee,
             const struct TALER_Amount *awire_fee,
             const json_t *aexchange_proof)
 {
-  CHECK ((0 == memcmp (ah_contract_terms,
-                       &h_contract_terms,
-                       sizeof (struct GNUNET_HashCode))));
-  CHECK (0 == memcmp (acoin_pub,
-                      &coin_pub,
-                      sizeof (struct TALER_CoinSpendPublicKeyP)));
+  CHECK (0 == GNUNET_memcmp (ah_contract_terms,
+                             &h_contract_terms));
+  CHECK (0 == GNUNET_memcmp (acoin_pub,
+                             &coin_pub));
   CHECK (0 == strcmp (aexchange_url,
                       EXCHANGE_URL));
   CHECK (0 == TALER_amount_cmp (aamount_with_fee,
@@ -306,16 +304,13 @@ transfer_cb (void *cls,
              struct GNUNET_TIME_Absolute execution_time,
              const json_t *exchange_proof)
 {
-  CHECK (0 == memcmp (ah_contract_terms,
-                      &h_contract_terms,
-                      sizeof (struct GNUNET_HashCode)));
+  CHECK (0 == GNUNET_memcmp (ah_contract_terms,
+                             &h_contract_terms));
 
-  CHECK (0 == memcmp (acoin_pub,
-                      &coin_pub,
-                      sizeof (struct TALER_CoinSpendPublicKeyP)));
-  CHECK (0 == memcmp (awtid,
-                      &wtid,
-                      sizeof (struct TALER_WireTransferIdentifierRawP)));
+  CHECK (0 == GNUNET_memcmp (acoin_pub,
+                             &coin_pub));
+  CHECK (0 == GNUNET_memcmp (awtid,
+                             &wtid));
   CHECK (1 == json_equal ((json_t *) exchange_proof,
                           transfer_proof));
 }
@@ -334,6 +329,8 @@ proof_cb (void *cls,
   CHECK (1 == json_equal ((json_t *) proof,
                           transfer_proof));
 }
+
+
 #undef CHECK
 
 
@@ -368,9 +365,9 @@ test_wire_fee ()
   date1 = GNUNET_TIME_absolute_get ();
   (void) GNUNET_TIME_round_abs (&date1);
   date2 = GNUNET_TIME_absolute_add (date1,
-				    GNUNET_TIME_UNIT_DAYS);
+                                    GNUNET_TIME_UNIT_DAYS);
   date3 = GNUNET_TIME_absolute_add (date2,
-				    GNUNET_TIME_UNIT_DAYS);
+                                    GNUNET_TIME_UNIT_DAYS);
   GNUNET_assert (GNUNET_OK ==
                  TALER_string_to_amount (CURRENCY ":5",
                                          &closing_fee1));
@@ -385,26 +382,26 @@ test_wire_fee ()
                                          &wire_fee2));
   if (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT !=
       plugin->store_wire_fee_by_exchange (plugin->cls,
-					  &exchange_pub,
-					  &h_wire_method,
-					  &wire_fee1,
-					  &closing_fee1,
-					  date1,
-					  date2,
-					  &exchange_sig))
+                                          &exchange_pub,
+                                          &h_wire_method,
+                                          &wire_fee1,
+                                          &closing_fee1,
+                                          date1,
+                                          date2,
+                                          &exchange_sig))
   {
     GNUNET_break (0);
     return GNUNET_SYSERR;
   }
   if (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT !=
       plugin->store_wire_fee_by_exchange (plugin->cls,
-					  &exchange_pub,
-					  &h_wire_method,
-					  &wire_fee2,
-					  &closing_fee2,
-					  date2,
-					  date3,
-					  &exchange_sig))
+                                          &exchange_pub,
+                                          &h_wire_method,
+                                          &wire_fee2,
+                                          &closing_fee2,
+                                          date2,
+                                          date3,
+                                          &exchange_sig))
   {
     GNUNET_break (0);
     return GNUNET_SYSERR;
@@ -412,56 +409,54 @@ test_wire_fee ()
   contract_date = date2; /* test inclusive/exclusive range */
   if (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT !=
       plugin->lookup_wire_fee (plugin->cls,
-			       &exchange_pub,
-			       &h_wire_method,
-			       contract_date,
-			       &wire_fee3,
-			       &closing_fee3,
-			       &start_date,
-			       &end_date,
-			       &exchange_sig2))
+                               &exchange_pub,
+                               &h_wire_method,
+                               contract_date,
+                               &wire_fee3,
+                               &closing_fee3,
+                               &start_date,
+                               &end_date,
+                               &exchange_sig2))
   {
     GNUNET_break (0);
     return GNUNET_SYSERR;
   }
   if ( (start_date.abs_value_us != date2.abs_value_us) ||
        (end_date.abs_value_us != date3.abs_value_us) ||
-       (0 != memcmp (&exchange_sig,
-		     &exchange_sig2,
-		     sizeof (exchange_sig))) ||
+       (0 != GNUNET_memcmp (&exchange_sig,
+                            &exchange_sig2)) ||
        (0 != TALER_amount_cmp (&wire_fee2,
-			       &wire_fee3)) ||
+                               &wire_fee3)) ||
        (0 != TALER_amount_cmp (&closing_fee2,
-			       &closing_fee3)) )
+                               &closing_fee3)) )
   {
     GNUNET_break (0);
     return GNUNET_SYSERR;
   }
   contract_date = GNUNET_TIME_absolute_add (date1,
-					    GNUNET_TIME_UNIT_SECONDS);
+                                            GNUNET_TIME_UNIT_SECONDS);
   if (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT !=
       plugin->lookup_wire_fee (plugin->cls,
-			       &exchange_pub,
-			       &h_wire_method,
-			       contract_date,
-			       &wire_fee3,
-			       &closing_fee3,
-			       &start_date,
-			       &end_date,
-			       &exchange_sig2))
+                               &exchange_pub,
+                               &h_wire_method,
+                               contract_date,
+                               &wire_fee3,
+                               &closing_fee3,
+                               &start_date,
+                               &end_date,
+                               &exchange_sig2))
   {
     GNUNET_break (0);
     return GNUNET_SYSERR;
   }
   if ( (start_date.abs_value_us != date1.abs_value_us) ||
        (end_date.abs_value_us != date2.abs_value_us) ||
-       (0 != memcmp (&exchange_sig,
-		     &exchange_sig2,
-		     sizeof (exchange_sig))) ||
+       (0 != GNUNET_memcmp (&exchange_sig,
+                            &exchange_sig2)) ||
        (0 != TALER_amount_cmp (&wire_fee1,
-			       &wire_fee3)) ||
+                               &wire_fee3)) ||
        (0 != TALER_amount_cmp (&closing_fee1,
-			       &closing_fee3)) )
+                               &closing_fee3)) )
   {
     GNUNET_break (0);
     return GNUNET_SYSERR;
@@ -469,14 +464,14 @@ test_wire_fee ()
   contract_date = date3; /* outside of valid range! */
   if (GNUNET_DB_STATUS_SUCCESS_NO_RESULTS !=
       plugin->lookup_wire_fee (plugin->cls,
-			       &exchange_pub,
-			       &h_wire_method,
-			       contract_date,
-			       &wire_fee3,
-			       &closing_fee3,
-			       &start_date,
-			       &end_date,
-			       &exchange_sig2))
+                               &exchange_pub,
+                               &h_wire_method,
+                               contract_date,
+                               &wire_fee3,
+                               &closing_fee3,
+                               &start_date,
+                               &end_date,
+                               &exchange_sig2))
   {
     GNUNET_break (0);
     return GNUNET_SYSERR;
@@ -507,13 +502,14 @@ test_tipping ()
 
   RND_BLK (&tip_reserve_priv);
   if (TALER_EC_TIP_AUTHORIZE_INSUFFICIENT_FUNDS !=
-      plugin->authorize_tip (plugin->cls,
-                             "testing tips reserve unknown",
-                             &amount,
-                             &tip_reserve_priv,
-			     "http://localhost:8081/",
-                             &tip_expiration,
-                             &tip_id))
+      plugin->authorize_tip_TR (plugin->cls,
+                                "testing tips reserve unknown",
+                                json_object (),
+                                &amount,
+                                &tip_reserve_priv,
+                                "http://localhost:8081/",
+                                &tip_expiration,
+                                &tip_id))
   {
     GNUNET_break (0);
     return GNUNET_SYSERR;
@@ -524,25 +520,26 @@ test_tipping ()
                                          &total));
   /* Pick short expiration, but long enough to
      run 2 DB interactions even on very slow systems. */
-  reserve_expiration = GNUNET_TIME_relative_to_absolute (GNUNET_TIME_relative_multiply (GNUNET_TIME_UNIT_SECONDS,
-                                                                                        2));
+  reserve_expiration = GNUNET_TIME_relative_to_absolute (
+    GNUNET_TIME_relative_multiply (GNUNET_TIME_UNIT_SECONDS,
+                                   2));
   if (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT !=
-      plugin->enable_tip_reserve (plugin->cls,
-                                  &tip_reserve_priv,
-                                  &tip_credit_uuid,
-                                  &total,
-                                  reserve_expiration))
+      plugin->enable_tip_reserve_TR (plugin->cls,
+                                     &tip_reserve_priv,
+                                     &tip_credit_uuid,
+                                     &total,
+                                     reserve_expiration))
   {
     GNUNET_break (0);
     return GNUNET_SYSERR;
   }
   /* check idempotency */
   if (GNUNET_DB_STATUS_SUCCESS_NO_RESULTS !=
-      plugin->enable_tip_reserve (plugin->cls,
-                                  &tip_reserve_priv,
-                                  &tip_credit_uuid,
-                                  &total,
-                                  reserve_expiration))
+      plugin->enable_tip_reserve_TR (plugin->cls,
+                                     &tip_reserve_priv,
+                                     &tip_credit_uuid,
+                                     &total,
+                                     reserve_expiration))
   {
     GNUNET_break (0);
     return GNUNET_SYSERR;
@@ -550,13 +547,14 @@ test_tipping ()
   /* Make sure it has expired, so at this point the value is back at ZERO! */
   sleep (3);
   if (TALER_EC_TIP_AUTHORIZE_RESERVE_EXPIRED !=
-      plugin->authorize_tip (plugin->cls,
-                             "testing tips too late",
-                             &amount,
-                             &tip_reserve_priv,
-			     "http://localhost:8081/",
-                             &tip_expiration,
-                             &tip_id))
+      plugin->authorize_tip_TR (plugin->cls,
+                                "testing tips too late",
+                                json_object (),
+                                &amount,
+                                &tip_reserve_priv,
+                                "http://localhost:8081/",
+                                &tip_expiration,
+                                &tip_id))
   {
     GNUNET_break (0);
     return GNUNET_SYSERR;
@@ -564,14 +562,15 @@ test_tipping ()
 
   /* Re-add some funds again */
   RND_BLK (&tip_credit_uuid);
-  reserve_expiration = GNUNET_TIME_relative_to_absolute (GNUNET_TIME_relative_multiply (GNUNET_TIME_UNIT_SECONDS,
-                                                                                        2));
+  reserve_expiration = GNUNET_TIME_relative_to_absolute (
+    GNUNET_TIME_relative_multiply (GNUNET_TIME_UNIT_SECONDS,
+                                   2));
   if (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT !=
-      plugin->enable_tip_reserve (plugin->cls,
-                                  &tip_reserve_priv,
-                                  &tip_credit_uuid,
-                                  &total,
-                                  reserve_expiration))
+      plugin->enable_tip_reserve_TR (plugin->cls,
+                                     &tip_reserve_priv,
+                                     &tip_credit_uuid,
+                                     &total,
+                                     reserve_expiration))
   {
     GNUNET_break (0);
     return GNUNET_SYSERR;
@@ -581,11 +580,11 @@ test_tipping ()
   RND_BLK (&tip_credit_uuid);
   reserve_expiration = GNUNET_TIME_relative_to_absolute (GNUNET_TIME_UNIT_DAYS);
   if (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT !=
-      plugin->enable_tip_reserve (plugin->cls,
-                                  &tip_reserve_priv,
-                                  &tip_credit_uuid,
-                                  &total,
-                                  reserve_expiration))
+      plugin->enable_tip_reserve_TR (plugin->cls,
+                                     &tip_reserve_priv,
+                                     &tip_credit_uuid,
+                                     &total,
+                                     reserve_expiration))
   {
     GNUNET_break (0);
     return GNUNET_SYSERR;
@@ -596,13 +595,14 @@ test_tipping ()
                  TALER_string_to_amount (CURRENCY ":4",
                                          &amount));
   if (TALER_EC_NONE !=
-      plugin->authorize_tip (plugin->cls,
-                             "testing tips",
-                             &amount,
-                             &tip_reserve_priv,
-			     "http://localhost:8081/",
-                             &tip_expiration,
-                             &tip_id))
+      plugin->authorize_tip_TR (plugin->cls,
+                                "testing tips",
+                                json_object (),
+                                &amount,
+                                &tip_reserve_priv,
+                                "http://localhost:8081/",
+                                &tip_expiration,
+                                &tip_id))
   {
     GNUNET_break (0);
     return GNUNET_SYSERR;
@@ -614,15 +614,15 @@ test_tipping ()
   }
   if (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT !=
       plugin->lookup_tip_by_id (plugin->cls,
-				      &tip_id,
-				      &url,
-                                      NULL, NULL))
+                                &tip_id,
+                                &url,
+                                NULL, NULL, NULL, NULL))
   {
     GNUNET_break (0);
     return GNUNET_SYSERR;
   }
   if (0 != strcmp ("http://localhost:8081/",
-		   url))
+                   url))
   {
     GNUNET_free (url);
     GNUNET_break (0);
@@ -630,13 +630,14 @@ test_tipping ()
   }
   GNUNET_free (url);
   if (TALER_EC_NONE !=
-      plugin->authorize_tip (plugin->cls,
-                             "testing tips more",
-                             &amount,
-                             &tip_reserve_priv,
-			     "http://localhost:8081/",
-                             &tip_expiration,
-                             &tip_id))
+      plugin->authorize_tip_TR (plugin->cls,
+                                "testing tips more",
+                                json_object (),
+                                &amount,
+                                &tip_reserve_priv,
+                                "http://localhost:8081/",
+                                &tip_expiration,
+                                &tip_id))
   {
     GNUNET_break (0);
     return GNUNET_SYSERR;
@@ -653,36 +654,34 @@ test_tipping ()
                                          &inc));
   RND_BLK (&pickup_id);
   if (TALER_EC_NONE !=
-      plugin->pickup_tip (plugin->cls,
-                          &inc,
-                          &tip_id,
-                          &pickup_id,
-                          &pres))
+      plugin->pickup_tip_TR (plugin->cls,
+                             &inc,
+                             &tip_id,
+                             &pickup_id,
+                             &pres))
   {
     GNUNET_break (0);
     return GNUNET_SYSERR;
   }
-  if (0 != memcmp (&pres,
-                   &tip_reserve_priv,
-                   sizeof (pres)))
+  if (0 != GNUNET_memcmp (&pres,
+                          &tip_reserve_priv))
   {
     GNUNET_break (0);
     return GNUNET_SYSERR;
   }
   RND_BLK (&pickup_id);
   if (TALER_EC_NONE !=
-      plugin->pickup_tip (plugin->cls,
-                          &inc,
-                          &tip_id,
-                          &pickup_id,
-                          &pres))
+      plugin->pickup_tip_TR (plugin->cls,
+                             &inc,
+                             &tip_id,
+                             &pickup_id,
+                             &pres))
   {
     GNUNET_break (0);
     return GNUNET_SYSERR;
   }
-  if (0 != memcmp (&pres,
-                   &tip_reserve_priv,
-                   sizeof (pres)))
+  if (0 != GNUNET_memcmp (&pres,
+                          &tip_reserve_priv))
   {
     GNUNET_break (0);
     return GNUNET_SYSERR;
@@ -691,11 +690,11 @@ test_tipping ()
   /* Third attempt should fail, as we've picked up 4/4 in amount */
   RND_BLK (&pickup_id);
   if (TALER_EC_TIP_PICKUP_NO_FUNDS !=
-      plugin->pickup_tip (plugin->cls,
-                          &inc,
-                          &tip_id,
-                          &pickup_id,
-                          &pres))
+      plugin->pickup_tip_TR (plugin->cls,
+                             &inc,
+                             &tip_id,
+                             &pickup_id,
+                             &pres))
   {
     GNUNET_break (0);
     return GNUNET_SYSERR;
@@ -703,13 +702,14 @@ test_tipping ()
 
   /* We authorized 8 out of 10, so going for another 4 should fail with insufficient funds */
   if (TALER_EC_TIP_AUTHORIZE_INSUFFICIENT_FUNDS !=
-      plugin->authorize_tip (plugin->cls,
-                             "testing tips insufficient funds",
-                             &amount,
-                             &tip_reserve_priv,
-			     "http://localhost:8081/",
-                             &tip_expiration,
-                             &tip_id))
+      plugin->authorize_tip_TR (plugin->cls,
+                                "testing tips insufficient funds",
+                                json_object (),
+                                &amount,
+                                &tip_reserve_priv,
+                                "http://localhost:8081/",
+                                &tip_expiration,
+                                &tip_id))
   {
     GNUNET_break (0);
     return GNUNET_SYSERR;
@@ -719,11 +719,11 @@ test_tipping ()
   RND_BLK (&tip_id);
   RND_BLK (&pickup_id);
   if (TALER_EC_TIP_PICKUP_TIP_ID_UNKNOWN !=
-      plugin->pickup_tip (plugin->cls,
-                          &inc,
-                          &tip_id,
-                          &pickup_id,
-                          &pres))
+      plugin->pickup_tip_TR (plugin->cls,
+                             &inc,
+                             &tip_id,
+                             &pickup_id,
+                             &pres))
   {
     GNUNET_break (0);
     return GNUNET_SYSERR;
@@ -751,7 +751,6 @@ run (void *cls)
     result = 77;
     return;
   }
-
   if (GNUNET_OK != plugin->drop_tables (plugin->cls))
   {
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
@@ -759,7 +758,8 @@ run (void *cls)
     result = 77;
     return;
   }
-  if (GNUNET_OK != plugin->initialize (plugin->cls))
+  TALER_MERCHANTDB_plugin_unload (plugin);
+  if (NULL == (plugin = TALER_MERCHANTDB_plugin_load (cfg)))
   {
     result = 77;
     return;
@@ -777,7 +777,7 @@ run (void *cls)
   (void) GNUNET_TIME_round_abs (&timestamp);
   delta = GNUNET_TIME_UNIT_MINUTES;
   fake_now = GNUNET_TIME_absolute_add (timestamp, delta);
-  refund_deadline = GNUNET_TIME_absolute_get();
+  refund_deadline = GNUNET_TIME_absolute_get ();
   (void) GNUNET_TIME_round_abs (&refund_deadline);
   GNUNET_assert (GNUNET_OK ==
                  TALER_string_to_amount (CURRENCY ":5",
@@ -807,12 +807,12 @@ run (void *cls)
   deposit_proof = json_object ();
   GNUNET_assert (0 ==
                  json_object_set_new (deposit_proof,
-                                      "test",
+                                      "x-taler-bank",
                                       json_string ("backenddb test A")));
   transfer_proof = json_object ();
   GNUNET_assert (0 ==
                  json_object_set_new (transfer_proof,
-                                      "test",
+                                      "x-taler-bank",
                                       json_string ("backenddb test B")));
   contract = json_object ();
   contract_terms = json_object ();
@@ -833,10 +833,10 @@ run (void *cls)
 
   FAILIF (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT !=
           plugin->insert_contract_terms (plugin->cls,
-					 order_id,
-					 &merchant_pub,
-					 timestamp,
-					 contract_terms));
+                                         order_id,
+                                         &merchant_pub,
+                                         timestamp,
+                                         contract_terms));
 
   FAILIF (GNUNET_DB_STATUS_SUCCESS_NO_RESULTS !=
           plugin->find_paid_contract_terms_from_hash (plugin->cls,
@@ -846,47 +846,21 @@ run (void *cls)
   FAILIF (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT !=
           plugin->mark_proposal_paid (plugin->cls,
                                       &h_contract_terms,
-                                      &merchant_pub,
-                                      "my-session-123"));
+                                      &merchant_pub));
 
 
-  {
-    char *last_session_id;
-    FAILIF (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT !=
-            plugin->find_contract_terms (plugin->cls,
-                                         &out,
-                                         &last_session_id,
-                                         order_id,
-                                         &merchant_pub));
-    FAILIF (0 != strcmp (last_session_id, "my-session-123"));
-    GNUNET_free (last_session_id);
-  }
-
   FAILIF (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT !=
           plugin->mark_proposal_paid (plugin->cls,
                                       &h_contract_terms,
-                                      &merchant_pub,
-                                      NULL));
+                                      &merchant_pub));
 
 
-  {
-    char *last_session_id;
-    FAILIF (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT !=
-            plugin->find_contract_terms (plugin->cls,
-                                         &out,
-                                         &last_session_id,
-                                         order_id,
-                                         &merchant_pub));
-    FAILIF (0 != strcmp (last_session_id, ""));
-    GNUNET_free (last_session_id);
-  }
-
   FAILIF (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT !=
-	  plugin->find_contract_terms_history (plugin->cls,
-					       order_id,
-					       &merchant_pub,
-					       &pd_cb,
-					       NULL));
+          plugin->find_contract_terms_history (plugin->cls,
+                                               order_id,
+                                               &merchant_pub,
+                                               &pd_cb,
+                                               NULL));
 
   FAILIF (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT !=
           plugin->find_paid_contract_terms_from_hash (plugin->cls,
@@ -895,27 +869,28 @@ run (void *cls)
                                                       &merchant_pub));
   FAILIF (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT !=
           plugin->find_contract_terms_from_hash (plugin->cls,
-						 &out,
-						 &h_contract_terms,
-						 &merchant_pub));
+                                                 &out,
+                                                 &h_contract_terms,
+                                                 &merchant_pub));
   FAILIF (1 !=
           plugin->find_contract_terms_by_date_and_range (plugin->cls,
-							 fake_now,
-							 &merchant_pub,
-							 2,
-							 1,
-							 GNUNET_NO,
-							 &pd_cb,
-							 NULL));
+                                                         fake_now,
+                                                         &merchant_pub,
+                                                         2,
+                                                         1,
+                                                         GNUNET_YES,
+                                                         GNUNET_NO,
+                                                         &pd_cb,
+                                                         NULL));
   timestamp = GNUNET_TIME_absolute_get ();
   (void) GNUNET_TIME_round_abs (&timestamp);
 
   FAILIF (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT !=
           plugin->insert_contract_terms (plugin->cls,
-					 order_id_future,
-					 &merchant_pub,
-					 timestamp,
-					 contract_terms_future));
+                                         order_id_future,
+                                         &merchant_pub,
+                                         timestamp,
+                                         contract_terms_future));
 
   fake_now = GNUNET_TIME_absolute_subtract (timestamp, delta);
 
@@ -926,36 +901,36 @@ run (void *cls)
   FAILIF (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT !=
           plugin->mark_proposal_paid (plugin->cls,
                                       &h_contract_terms_future,
-                                      &merchant_pub,
-                                      "hello"));
+                                      &merchant_pub));
   FAILIF (2 !=
           plugin->find_contract_terms_by_date_and_range (plugin->cls,
-							 fake_now,
-							 &merchant_pub,
-							 0,
-							 5,
-							 GNUNET_YES,
-							 &pd_cb,
-							 NULL));
+                                                         fake_now,
+                                                         &merchant_pub,
+                                                         0,
+                                                         5,
+                                                         GNUNET_NO,
+                                                         GNUNET_NO,
+                                                         &pd_cb,
+                                                         NULL));
 
   FAILIF (0 !=
           plugin->find_contract_terms_by_date (plugin->cls,
-					       fake_now,
-					       &merchant_pub,
-					       1,
-					       &pd_cb,
-					       NULL));
+                                               fake_now,
+                                               &merchant_pub,
+                                               1,
+                                               &pd_cb,
+                                               NULL));
 
   FAILIF (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT !=
           plugin->store_deposit (plugin->cls,
                                  &h_contract_terms,
-				 &merchant_pub,
+                                 &merchant_pub,
                                  &coin_pub,
-				 EXCHANGE_URL,
+                                 EXCHANGE_URL,
                                  &amount_with_fee,
                                  &deposit_fee,
                                  &refund_fee,
-				 &wire_fee,
+                                 &wire_fee,
                                  &signkey_pub,
                                  deposit_proof));
   FAILIF (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT !=
@@ -1000,54 +975,54 @@ run (void *cls)
                                                         &refund_cb,
                                                         NULL));
   FAILIF (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT !=
-          plugin->increase_refund_for_contract (plugin->cls,
-                                                &h_contract_terms,
-                                                &merchant_pub,
-                                                &refund_amount,
-                                                "refund testing"));
+          plugin->increase_refund_for_contract_NT (plugin->cls,
+                                                   &h_contract_terms,
+                                                   &merchant_pub,
+                                                   &refund_amount,
+                                                   "refund testing"));
 
   FAILIF (GNUNET_DB_STATUS_SUCCESS_NO_RESULTS !=
-          plugin->increase_refund_for_contract (plugin->cls,
-                                                &h_contract_terms,
-                                                &merchant_pub,
-                                                &refund_amount,
-                                                "same refund amount as "
-                                                "the previous one, should succeed without changes (0)"));
+          plugin->increase_refund_for_contract_NT (plugin->cls,
+                                                   &h_contract_terms,
+                                                   &merchant_pub,
+                                                   &refund_amount,
+                                                   "same refund amount as "
+                                                   "the previous one, should succeed without changes (0)"));
 
   /*Should fail as this refund a lesser amount respect to the previous one*/
   FAILIF (GNUNET_DB_STATUS_SUCCESS_NO_RESULTS !=
-          plugin->increase_refund_for_contract (plugin->cls,
-                                                &h_contract_terms,
-                                                &merchant_pub,
-                                                &little_refund_amount,
-                                                "lower refund amount as the previous one, should succeed without changes (0)"));
+          plugin->increase_refund_for_contract_NT (plugin->cls,
+                                                   &h_contract_terms,
+                                                   &merchant_pub,
+                                                   &little_refund_amount,
+                                                   "lower refund amount as the previous one, should succeed without changes (0)"));
   FAILIF (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT !=
-          plugin->increase_refund_for_contract (plugin->cls,
-                                                &h_contract_terms,
-                                                &merchant_pub,
-                                                &right_second_refund_amount,
-                                                "right refund increase"));
+          plugin->increase_refund_for_contract_NT (plugin->cls,
+                                                   &h_contract_terms,
+                                                   &merchant_pub,
+                                                   &right_second_refund_amount,
+                                                   "right refund increase"));
 
   FAILIF (GNUNET_DB_STATUS_HARD_ERROR !=
-          plugin->increase_refund_for_contract (plugin->cls,
-                                                &h_contract_terms,
-                                                &merchant_pub,
-                                                &too_big_refund_amount,
-                                                "make refund testing fail due"
-                                                " to too big refund amount"));
+          plugin->increase_refund_for_contract_NT (plugin->cls,
+                                                   &h_contract_terms,
+                                                   &merchant_pub,
+                                                   &too_big_refund_amount,
+                                                   "make refund testing fail due"
+                                                   " to too big refund amount"));
 
   FAILIF (GNUNET_OK !=
-	  test_wire_fee ());
+          test_wire_fee ());
   FAILIF (GNUNET_OK !=
-	  test_tipping ());
+          test_tipping ());
 
 
   if (-1 == result)
     result = 0;
 
- drop:
+drop:
   GNUNET_break (GNUNET_OK ==
-		plugin->drop_tables (plugin->cls));
+                plugin->drop_tables (plugin->cls));
   TALER_MERCHANTDB_plugin_unload (plugin);
   plugin = NULL;
   if (NULL != deposit_proof)
@@ -1090,11 +1065,13 @@ main (int argc,
     GNUNET_free (testname);
     return 2;
   }
-  GNUNET_SCHEDULER_run (&run, cfg);
+  GNUNET_SCHEDULER_run (&run,
+                        cfg);
   GNUNET_CONFIGURATION_destroy (cfg);
   GNUNET_free (config_filename);
   GNUNET_free (testname);
   return result;
 }
 
+
 /* end of test_merchantdb.c */
