@@ -47,9 +47,9 @@ struct TALER_MERCHANT_RefundLookupOperation;
  */
 typedef void
 (*TALER_MERCHANT_RefundLookupCallback) (void *cls,
-					unsigned int http_status,
-					enum TALER_ErrorCode ec,
-					const json_t *obj);
+                                        unsigned int http_status,
+                                        enum TALER_ErrorCode ec,
+                                        const json_t *obj);
 
 /**
  * Does a GET /refund.
@@ -65,7 +65,6 @@ struct TALER_MERCHANT_RefundLookupOperation *
 TALER_MERCHANT_refund_lookup (struct GNUNET_CURL_Context *ctx,
                               const char *backend_url,
                               const char *order_id,
-                              const char *instance,
                               TALER_MERCHANT_RefundLookupCallback cb,
                               void *cb_cls);
 
@@ -75,7 +74,8 @@ TALER_MERCHANT_refund_lookup (struct GNUNET_CURL_Context *ctx,
  * @param rlo the refund increasing operation to cancel
  */
 void
-TALER_MERCHANT_refund_lookup_cancel (struct TALER_MERCHANT_RefundLookupOperation *rlo);
+TALER_MERCHANT_refund_lookup_cancel (struct
+                                     TALER_MERCHANT_RefundLookupOperation *rlo);
 
 
 /**
@@ -107,7 +107,6 @@ typedef void
  * @param order_id id of the order whose refund is to be increased
  * @param refund amount to which increase the refund
  * @param reason human-readable reason justifying the refund
- * @param instance id of the merchant instance issuing the request
  * @param cb callback processing the response from /refund
  * @param cb_cls closure for cb
  */
@@ -117,7 +116,6 @@ TALER_MERCHANT_refund_increase (struct GNUNET_CURL_Context *ctx,
                                 const char *order_id,
                                 const struct TALER_Amount *refund,
                                 const char *reason,
-                                const char *instance,
                                 TALER_MERCHANT_RefundIncreaseCallback cb,
                                 void *cb_cls);
 
@@ -127,7 +125,9 @@ TALER_MERCHANT_refund_increase (struct GNUNET_CURL_Context *ctx,
  * @param rio the refund increasing operation to cancel
  */
 void
-TALER_MERCHANT_refund_increase_cancel (struct TALER_MERCHANT_RefundIncreaseOperation *rio);
+TALER_MERCHANT_refund_increase_cancel (struct
+                                       TALER_MERCHANT_RefundIncreaseOperation *
+                                       rio);
 
 
 /* *********************  /proposal *********************** */
@@ -152,7 +152,7 @@ struct TALER_MERCHANT_ProposalOperation;
 typedef void
 (*TALER_MERCHANT_ProposalCallback) (void *cls,
                                     unsigned int http_status,
-				    enum TALER_ErrorCode ec,
+                                    enum TALER_ErrorCode ec,
                                     const json_t *obj,
                                     const char *order_id);
 
@@ -205,8 +205,10 @@ typedef void
                                                    unsigned int http_status,
                                                    const json_t *body,
                                                    const json_t *contract_terms,
-                                                   const struct TALER_MerchantSignatureP *sig,
-                                                   const struct GNUNET_HashCode *hash);
+                                                   const struct
+                                                   TALER_MerchantSignatureP *sig,
+                                                   const struct
+                                                   GNUNET_HashCode *hash);
 
 
 /**
@@ -226,9 +228,10 @@ struct TALER_MERCHANT_ProposalLookupOperation *
 TALER_MERCHANT_proposal_lookup (struct GNUNET_CURL_Context *ctx,
                                 const char *backend_url,
                                 const char *order_id,
-                                const char *instance,
-                                const struct GNUNET_CRYPTO_EddsaPublicKey *nonce,
-                                TALER_MERCHANT_ProposalLookupOperationCallback plo_cb,
+                                const struct
+                                GNUNET_CRYPTO_EddsaPublicKey *nonce,
+                                TALER_MERCHANT_ProposalLookupOperationCallback
+                                plo_cb,
                                 void *plo_cb_cls);
 
 
@@ -238,8 +241,9 @@ TALER_MERCHANT_proposal_lookup (struct GNUNET_CURL_Context *ctx,
  * @param plo handle to the request to be canceled
  */
 void
-TALER_MERCHANT_proposal_lookup_cancel (struct TALER_MERCHANT_ProposalLookupOperation *plo);
-
+TALER_MERCHANT_proposal_lookup_cancel (struct
+                                       TALER_MERCHANT_ProposalLookupOperation *
+                                       plo);
 
 
 /* *********************  /pay *********************** */
@@ -272,7 +276,7 @@ struct TALER_MERCHANT_Pay;
 typedef void
 (*TALER_MERCHANT_PayCallback) (void *cls,
                                unsigned int http_status,
-			       enum TALER_ErrorCode ec,
+                               enum TALER_ErrorCode ec,
                                const json_t *obj);
 
 
@@ -330,7 +334,6 @@ struct TALER_MERCHANT_PayCoin
  *
  * @param ctx execution context
  * @param merchant_url base URL of the merchant
- * @param instance which merchant instance will receive this payment
  * @param h_wire hash of the merchant’s account details
  * @param h_contract hash of the contact of the merchant with the customer
  * @param transaction_id transaction id for the transaction between merchant and customer
@@ -350,8 +353,7 @@ struct TALER_MERCHANT_PayCoin
  */
 struct TALER_MERCHANT_Pay *
 TALER_MERCHANT_pay_wallet (struct GNUNET_CURL_Context *ctx,
-			   const char *merchant_url,
-			   const char *instance,
+                           const char *merchant_url,
                            const struct GNUNET_HashCode *h_contract,
                            const struct TALER_Amount *amount,
                            const struct TALER_Amount *max_fee,
@@ -408,13 +410,15 @@ struct TALER_MERCHANT_RefundEntry
  */
 typedef void
 (*TALER_MERCHANT_PayRefundCallback) (void *cls,
-				     unsigned int http_status,
-				     enum TALER_ErrorCode ec,
-				     const struct TALER_MerchantPublicKeyP *merchant_pub,
-				     const struct GNUNET_HashCode *h_contract,
-				     unsigned int num_refunds,
-				     const struct TALER_MERCHANT_RefundEntry *res,
-				     const json_t *obj);
+                                     unsigned int http_status,
+                                     enum TALER_ErrorCode ec,
+                                     const struct
+                                     TALER_MerchantPublicKeyP *merchant_pub,
+                                     const struct GNUNET_HashCode *h_contract,
+                                     unsigned int num_refunds,
+                                     const struct
+                                     TALER_MERCHANT_RefundEntry *res,
+                                     const json_t *obj);
 
 
 /**
@@ -423,7 +427,6 @@ typedef void
  *
  * @param ctx execution context
  * @param merchant_url base URL of the merchant
- * @param instance which merchant instance will receive this payment
  * @param h_wire hash of the merchant’s account details
  * @param h_contract hash of the contact of the merchant with the customer
  * @param transaction_id transaction id for the transaction between merchant and customer
@@ -443,22 +446,21 @@ typedef void
  */
 struct TALER_MERCHANT_Pay *
 TALER_MERCHANT_pay_abort (struct GNUNET_CURL_Context *ctx,
-			  const char *merchant_url,
-			  const char *instance,
-			  const struct GNUNET_HashCode *h_contract,
-			  const struct TALER_Amount *amount,
-			  const struct TALER_Amount *max_fee,
-			  const struct TALER_MerchantPublicKeyP *merchant_pub,
-			  const struct TALER_MerchantSignatureP *merchant_sig,
-			  struct GNUNET_TIME_Absolute timestamp,
-			  struct GNUNET_TIME_Absolute refund_deadline,
-			  struct GNUNET_TIME_Absolute pay_deadline,
-			  const struct GNUNET_HashCode *h_wire,
-			  const char *order_id,
-			  unsigned int num_coins,
-			  const struct TALER_MERCHANT_PayCoin *coins,
-			  TALER_MERCHANT_PayRefundCallback payref_cb,
-			  void *payref_cb_cls);
+                          const char *merchant_url,
+                          const struct GNUNET_HashCode *h_contract,
+                          const struct TALER_Amount *amount,
+                          const struct TALER_Amount *max_fee,
+                          const struct TALER_MerchantPublicKeyP *merchant_pub,
+                          const struct TALER_MerchantSignatureP *merchant_sig,
+                          struct GNUNET_TIME_Absolute timestamp,
+                          struct GNUNET_TIME_Absolute refund_deadline,
+                          struct GNUNET_TIME_Absolute pay_deadline,
+                          const struct GNUNET_HashCode *h_wire,
+                          const char *order_id,
+                          unsigned int num_coins,
+                          const struct TALER_MERCHANT_PayCoin *coins,
+                          TALER_MERCHANT_PayRefundCallback payref_cb,
+                          void *payref_cb_cls);
 
 
 /**
@@ -523,7 +525,6 @@ struct TALER_MERCHANT_PaidCoin
  *
  * @param ctx execution context
  * @param merchant_url base URL of the merchant
- * @param instance which merchant instance will receive this payment
  * @param h_contract hash of the contact of the merchant with the customer
  * @param amount total value of the contract to be paid to the merchant
  * @param max_fee maximum fee covered by the merchant (according to the contract)
@@ -542,9 +543,10 @@ struct TALER_MERCHANT_PaidCoin
  */
 struct TALER_MERCHANT_Pay *
 TALER_MERCHANT_pay_frontend (struct GNUNET_CURL_Context *ctx,
-			     const char *merchant_url,
-                             const struct TALER_MerchantPublicKeyP *merchant_pub,
-			     const char *order_id,
+                             const char *merchant_url,
+                             const struct
+                             TALER_MerchantPublicKeyP *merchant_pub,
+                             const char *order_id,
                              unsigned int num_coins,
                              const struct TALER_MERCHANT_PaidCoin *coins,
                              TALER_MERCHANT_PayCallback pay_cb,
@@ -615,13 +617,17 @@ struct TALER_MERCHANT_TrackTransferDetails
 typedef void
 (*TALER_MERCHANT_TrackTransferCallback) (void *cls,
                                          unsigned int http_status,
-					 enum TALER_ErrorCode ec,
-                                         const struct TALER_ExchangePublicKeyP *sign_key,
+                                         enum TALER_ErrorCode ec,
+                                         const struct
+                                         TALER_ExchangePublicKeyP *sign_key,
                                          const json_t *json,
                                          const struct GNUNET_HashCode *h_wire,
-                                         const struct TALER_Amount *total_amount,
+                                         const struct
+                                         TALER_Amount *total_amount,
                                          unsigned int details_length,
-                                         const struct TALER_MERCHANT_TrackTransferDetails *details);
+                                         const struct
+                                         TALER_MERCHANT_TrackTransferDetails *
+                                         details);
 
 
 /**
@@ -629,7 +635,6 @@ typedef void
  *
  * @param ctx execution context
  * @param backend_url base URL of the backend
- * @param instance which merchant instance is going to be tracked
  * @param wire_method wire method used for the wire transfer
  * @param wtid base32 string indicating a wtid
  * @param exchange base URL of the exchange in charge of returning the wanted information
@@ -640,11 +645,12 @@ typedef void
 struct TALER_MERCHANT_TrackTransferHandle *
 TALER_MERCHANT_track_transfer (struct GNUNET_CURL_Context *ctx,
                                const char *backend_url,
-                               const char *instance,
-			       const char *wire_method,
-                               const struct TALER_WireTransferIdentifierRawP *wtid,
+                               const char *wire_method,
+                               const struct
+                               TALER_WireTransferIdentifierRawP *wtid,
                                const char *exchange_url,
-                               TALER_MERCHANT_TrackTransferCallback track_transfer_cb,
+                               TALER_MERCHANT_TrackTransferCallback
+                               track_transfer_cb,
                                void *track_transfer_cb_cls);
 
 
@@ -655,7 +661,8 @@ TALER_MERCHANT_track_transfer (struct GNUNET_CURL_Context *ctx,
  * @param co the deposit's tracking operation
  */
 void
-TALER_MERCHANT_track_transfer_cancel (struct TALER_MERCHANT_TrackTransferHandle *tdo);
+TALER_MERCHANT_track_transfer_cancel (struct
+                                      TALER_MERCHANT_TrackTransferHandle *tdo);
 
 
 /* ********************* /track/transaction *********************** */
@@ -704,7 +711,7 @@ struct TALER_MERCHANT_CoinWireTransfer
 typedef void
 (*TALER_MERCHANT_TrackTransactionCallback) (void *cls,
                                             unsigned int http_status,
-					    enum TALER_ErrorCode ec,
+                                            enum TALER_ErrorCode ec,
                                             const json_t *json);
 
 
@@ -713,7 +720,6 @@ typedef void
  *
  * @param ctx execution context
  * @param backend_url base URL of the backend
- * @param instance which merchant instance is going to be tracked
  * @param transaction_id which transaction should we trace
  * @param track_transaction_cb the callback to call when a reply for this request is available
  * @param track_transaction_cb_cls closure for @a track_transaction_cb
@@ -722,9 +728,9 @@ typedef void
 struct TALER_MERCHANT_TrackTransactionHandle *
 TALER_MERCHANT_track_transaction (struct GNUNET_CURL_Context *ctx,
                                   const char *backend_url,
-                                  const char *instance,
                                   const char *order_id,
-                                  TALER_MERCHANT_TrackTransactionCallback track_transaction_cb,
+                                  TALER_MERCHANT_TrackTransactionCallback
+                                  track_transaction_cb,
                                   void *track_transaction_cb_cls);
 
 
@@ -735,7 +741,9 @@ TALER_MERCHANT_track_transaction (struct GNUNET_CURL_Context *ctx,
  * @param co the deposit's tracking operation
  */
 void
-TALER_MERCHANT_track_transaction_cancel (struct TALER_MERCHANT_TrackTransactionHandle *tdo);
+TALER_MERCHANT_track_transaction_cancel (struct
+                                         TALER_MERCHANT_TrackTransactionHandle *
+                                         tdo);
 
 /* ********************* /history *********************** */
 
@@ -754,7 +762,7 @@ struct TALER_MERCHANT_HistoryOperation;
 typedef void
 (*TALER_MERCHANT_HistoryOperationCallback) (void *cls,
                                             unsigned int http_status,
-					    enum TALER_ErrorCode ec,
+                                            enum TALER_ErrorCode ec,
                                             const json_t *json);
 
 /**
@@ -762,7 +770,6 @@ typedef void
  *
  * @param ctx execution context
  * @param backend_url base URL of the merchant backend
- * @param instance which merchant instance is performing this call
  * @param start return @a delta records starting from position @a start
  * @param delta return @a delta records starting from position @a start
  * @param date only transactions younger than/equals to date will be returned
@@ -773,13 +780,33 @@ typedef void
 struct TALER_MERCHANT_HistoryOperation *
 TALER_MERCHANT_history (struct GNUNET_CURL_Context *ctx,
                         const char *backend_url,
-                        const char *instance,
-                        unsigned int start,
-                        unsigned int delta,
+                        unsigned long long start,
+                        long long delta,
                         struct GNUNET_TIME_Absolute date,
                         TALER_MERCHANT_HistoryOperationCallback history_cb,
                         void *history_cb_cls);
 
+/**
+ * Issue a /history request to the backend.
+ *
+ * @param ctx execution context
+ * @param backend_url base URL of the merchant backend
+ * @param start return `delta` records starting from position `start`.
+ * If given as zero, then no initial skip of `start` records is done.
+ * @param delta return `delta` records starting from position `start`
+ * @param date only transactions younger than/equals to date will be returned
+ * @param history_cb callback which will work the response gotten from the backend
+ * @param history_cb_cls closure to pass to @a history_cb
+ * @return handle for this operation, NULL upon errors
+ */
+struct TALER_MERCHANT_HistoryOperation *
+TALER_MERCHANT_history_default_start (struct GNUNET_CURL_Context *ctx,
+                                      const char *backend_url,
+                                      long long delta,
+                                      struct GNUNET_TIME_Absolute date,
+                                      TALER_MERCHANT_HistoryOperationCallback
+                                      history_cb,
+                                      void *history_cb_cls);
 
 
 /**
@@ -806,17 +833,15 @@ struct TALER_MERCHANT_TipAuthorizeOperation;
  * @param cls closure
  * @param http_status HTTP status returned by the merchant backend
  * @param ec taler-specific error code
+ * @param taler_tip_uri tip URI
  * @param tip_id which tip ID should be used to pickup the tip
- * @param tip_expiration when does the tip expire (needs to be picked up before this time)
- * @param exchange_url at what exchange can the tip be picked up
  */
 typedef void
 (*TALER_MERCHANT_TipAuthorizeCallback) (void *cls,
                                         unsigned int http_status,
                                         enum TALER_ErrorCode ec,
-                                        const struct GNUNET_HashCode *tip_id,
-                                        struct GNUNET_TIME_Absolute tip_expiration,
-                                        const char *exchange_url);
+                                        const char *taler_tip_uri,
+                                        struct GNUNET_HashCode *tip_id);
 
 
 /**
@@ -828,7 +853,6 @@ typedef void
  * @param pickup_url frontend URL for where the tip can be picked up
  * @param next_url where the browser should proceed after picking up the tip
  * @param amount amount to be handed out as a tip
- * @param instance which backend instance should create the tip (identifies the reserve and exchange)
  * @param justification which justification should be stored (human-readable reason for the tip)
  * @param authorize_cb callback which will work the response gotten from the backend
  * @param authorize_cb_cls closure to pass to @a authorize_cb
@@ -840,20 +864,19 @@ TALER_MERCHANT_tip_authorize (struct GNUNET_CURL_Context *ctx,
                               const char *pickup_url,
                               const char *next_url,
                               const struct TALER_Amount *amount,
-                              const char *instance,
                               const char *justification,
                               TALER_MERCHANT_TipAuthorizeCallback authorize_cb,
                               void *authorize_cb_cls);
 
 
-
 /**
  * Cancel a pending /tip-authorize request
  *
  * @param ta handle from the operation to cancel
  */
 void
-TALER_MERCHANT_tip_authorize_cancel (struct TALER_MERCHANT_TipAuthorizeOperation *ta);
+TALER_MERCHANT_tip_authorize_cancel (struct
+                                     TALER_MERCHANT_TipAuthorizeOperation *ta);
 
 /* ********************** /tip-pickup ************************* */
 
@@ -880,9 +903,11 @@ typedef void
 (*TALER_MERCHANT_TipPickupCallback) (void *cls,
                                      unsigned int http_status,
                                      enum TALER_ErrorCode ec,
-                                     const struct TALER_ReservePublicKeyP *reserve_pub,
+                                     const struct
+                                     TALER_ReservePublicKeyP *reserve_pub,
                                      unsigned int num_reserve_sigs,
-                                     const struct TALER_ReserveSignatureP *reserve_sigs,
+                                     const struct
+                                     TALER_ReserveSignatureP *reserve_sigs,
                                      const json_t *json);
 
 
@@ -940,17 +965,17 @@ struct TALER_MERCHANT_CheckPaymentOperation;
  *        #GNUNET_NO if refunded, #GNUNET_SYSERR or error
  * @param refunded_amount amount that was refunded, NULL if there
  *        was no refund
- * @param payment_redirect_url URL to redirect the browser to in order to
- *        execute or re-play the payment (NULL if not applicable)
+ * @param taler_pay_uri the URI that instructs the wallets to process
+ *                      the payment
  */
 typedef void
 (*TALER_MERCHANT_CheckPaymentCallback) (void *cls,
-					unsigned int http_status,
-					const json_t *obj,
+                                        unsigned int http_status,
+                                        const json_t *obj,
                                         int paid,
                                         int refunded,
                                         struct TALER_Amount *refund_amount,
-                                        const char *payment_redirect_url);
+                                        const char *taler_pay_uri);
 
 
 /**
@@ -959,12 +984,12 @@ typedef void
  *
  * @param ctx execution context
  * @param backend_url base URL of the merchant backend
- * @param instance instance used for the transaction
  * @param order_id order id to identify the payment
- * @parem resource_url resource URL to identify duplicate payments (can be NULL)
- * @parem session_id sesion id for the payment (or NULL if the payment is not bound to a session) 
- * @parem session_id sesion signature for the payment (or NULL if the payment
- *        is not bound to a session or the session is not signed yet) 
+ * @parem session_id sesion id for the payment (or NULL if the payment is not bound to a session)
+ * @param timeout timeout to use in long polling (how long may the server wait to reply
+ *        before generating an unpaid response). Note that this is just provided to
+ *        the server, we as client will block until the response comes back or until
+ *        #TALER_MERCHANT_check_payment_cancel() is called.
  * @param check_payment_cb callback which will work the response gotten from the backend
  * @param check_payment_cb_cls closure to pass to @a check_payment_cb
  * @return handle for this operation, NULL upon errors
@@ -972,12 +997,11 @@ typedef void
 struct TALER_MERCHANT_CheckPaymentOperation *
 TALER_MERCHANT_check_payment (struct GNUNET_CURL_Context *ctx,
                               const char *backend_url,
-                              const char *instance,
                               const char *order_id,
-                              const char *resource_url,
                               const char *session_id,
-                              const char *session_sig,
-                              TALER_MERCHANT_CheckPaymentCallback check_payment_cb,
+                              struct GNUNET_TIME_Relative timeout,
+                              TALER_MERCHANT_CheckPaymentCallback
+                              check_payment_cb,
                               void *check_payment_cls);
 
 /**
@@ -986,7 +1010,8 @@ TALER_MERCHANT_check_payment (struct GNUNET_CURL_Context *ctx,
  * @param cpo handle to the request to be canceled
  */
 void
-TALER_MERCHANT_check_payment_cancel (struct TALER_MERCHANT_CheckPaymentOperation *cpo);
+TALER_MERCHANT_check_payment_cancel (struct
+                                     TALER_MERCHANT_CheckPaymentOperation *cpo);
 
 
 /* ********************** /tip-query ************************* */
@@ -1004,13 +1029,19 @@ struct TALER_MERCHANT_TipQueryOperation;
  * @param http_status HTTP status code for this request
  * @param ec Taler-specific error code
  * @param raw raw response body
+ * @param reserve_expiration when the tip reserve will expire
+ * @param reserve_pub tip reserve public key
+ * @param amount_authorized total amount authorized on tip reserve
+ * @param amount_available total amount still available on tip reserve
+ * @param amount_picked_up total amount picked up from tip reserve
  */
 typedef void
 (*TALER_MERCHANT_TipQueryCallback) (void *cls,
                                     unsigned int http_status,
                                     enum TALER_ErrorCode ec,
                                     const json_t *raw,
-                                    struct GNUNET_TIME_Absolute reserve_expiration,
+                                    struct GNUNET_TIME_Absolute
+                                    reserve_expiration,
                                     struct TALER_ReservePublicKeyP *reserve_pub,
                                     struct TALER_Amount *amount_authorized,
                                     struct TALER_Amount *amount_available,
@@ -1032,13 +1063,11 @@ TALER_MERCHANT_tip_query_cancel (struct TALER_MERCHANT_TipQueryOperation *tqo);
  *
  * @param ctx execution context
  * @param backend_url base URL of the merchant backend
- * @param instance instance to query
  * @return handle for this operation, NULL upon errors
  */
 struct TALER_MERCHANT_TipQueryOperation *
 TALER_MERCHANT_tip_query (struct GNUNET_CURL_Context *ctx,
                           const char *backend_url,
-                          const char *instance,
                           TALER_MERCHANT_TipQueryCallback query_cb,
                           void *query_cb_cls);
 
@@ -1052,4 +1081,78 @@ void
 TALER_MERCHANT_tip_query_cancel (struct TALER_MERCHANT_TipQueryOperation *tqh);
 
 
+/* ********************** /public/poll-payment ************************* */
+
+
+/**
+ * Handle for a /public/poll-payment operation.
+ */
+struct TALER_MERCHANT_PollPaymentOperation;
+
+
+/**
+ * Callback to process a GET /poll-payment request
+ *
+ * @param cls closure
+ * @param http_status HTTP status code for this request
+ * @param obj raw response body
+ * @param paid #GNUNET_YES if the payment is settled, #GNUNET_NO if not
+ *        settled, $GNUNET_SYSERR on error
+ *        (note that refunded payments are returned as paid!)
+ * @param refunded #GNUNET_YES if there is at least on refund on this payment,
+ *        #GNUNET_NO if refunded, #GNUNET_SYSERR or error
+ * @param refunded_amount amount that was refunded, NULL if there
+ *        was no refund
+ * @param taler_pay_uri the URI that instructs the wallets to process
+ *                      the payment
+ */
+typedef void
+(*TALER_MERCHANT_PollPaymentCallback) (void *cls,
+                                       unsigned int http_status,
+                                       const json_t *obj,
+                                       int paid,
+                                       int refunded,
+                                       struct TALER_Amount *refund_amount,
+                                       const char *taler_pay_uri);
+
+
+/**
+ * Issue a /poll-payment request to the backend.  Polls the status
+ * of a payment.
+ *
+ * @param ctx execution context
+ * @param backend_url base URL of the merchant backend
+ * @param order_id order id to identify the payment
+ * @param h_contract hash of the contract for @a order_id
+ * @parem session_id sesion id for the payment (or NULL if the payment is not bound to a session)
+ * @param timeout timeout to use in long polling (how long may the server wait to reply
+ *        before generating an unpaid response). Note that this is just provided to
+ *        the server, we as client will block until the response comes back or until
+ *        #TALER_MERCHANT_poll_payment_cancel() is called.
+ * @param poll_payment_cb callback which will work the response gotten from the backend
+ * @param poll_payment_cb_cls closure to pass to @a poll_payment_cb
+ * @return handle for this operation, NULL upon errors
+ */
+struct TALER_MERCHANT_PollPaymentOperation *
+TALER_MERCHANT_poll_payment (struct GNUNET_CURL_Context *ctx,
+                             const char *backend_url,
+                             const char *order_id,
+                             const struct GNUNET_HashCode *h_contract,
+                             const char *session_id,
+                             struct GNUNET_TIME_Relative timeout,
+                             TALER_MERCHANT_PollPaymentCallback
+                             poll_payment_cb,
+                             void *poll_payment_cls);
+
+
+/**
+ * Cancel a GET /public/poll-payment request.
+ *
+ * @param cpo handle to the request to be canceled
+ */
+void
+TALER_MERCHANT_poll_payment_cancel (struct
+                                    TALER_MERCHANT_PollPaymentOperation *cpo);
+
+
 #endif  /* _TALER_MERCHANT_SERVICE_H */
