@@ -40,11 +40,11 @@ struct TALER_MERCHANTDB_Plugin;
  * @param row_id serial numer of the transaction in the table,
  * @param contract_terms proposal data related to order id
  */
- typedef void
- (*TALER_MERCHANTDB_ProposalDataCallback)(void *cls,
-                                          const char *order_id,
-                                          uint64_t row_id,
-                                          const json_t *contract_terms);
+typedef void
+(*TALER_MERCHANTDB_ProposalDataCallback)(void *cls,
+                                         const char *order_id,
+                                         uint64_t row_id,
+                                         const json_t *contract_terms);
 
 /**
  * Function called with information about a transaction.
@@ -59,12 +59,15 @@ struct TALER_MERCHANTDB_Plugin;
  */
 typedef void
 (*TALER_MERCHANTDB_TransactionCallback)(void *cls,
-					const struct TALER_MerchantPublicKeyP *merchant_pub,
-                                        const struct GNUNET_HashCode *h_contract_terms,
+                                        const struct
+                                        TALER_MerchantPublicKeyP *merchant_pub,
+                                        const struct
+                                        GNUNET_HashCode *h_contract_terms,
                                         const struct GNUNET_HashCode *h_wire,
                                         struct GNUNET_TIME_Absolute timestamp,
                                         struct GNUNET_TIME_Absolute refund,
-                                        const struct TALER_Amount *total_amount);
+                                        const struct
+                                        TALER_Amount *total_amount);
 
 
 /**
@@ -83,13 +86,16 @@ typedef void
  */
 typedef void
 (*TALER_MERCHANTDB_CoinDepositCallback)(void *cls,
-                                        const struct GNUNET_HashCode *h_contract_terms,
-                                        const struct TALER_CoinSpendPublicKeyP *coin_pub,
-					const char *exchange_url,
-                                        const struct TALER_Amount *amount_with_fee,
+                                        const struct
+                                        GNUNET_HashCode *h_contract_terms,
+                                        const struct
+                                        TALER_CoinSpendPublicKeyP *coin_pub,
+                                        const char *exchange_url,
+                                        const struct
+                                        TALER_Amount *amount_with_fee,
                                         const struct TALER_Amount *deposit_fee,
                                         const struct TALER_Amount *refund_fee,
-					const struct TALER_Amount *wire_fee,
+                                        const struct TALER_Amount *wire_fee,
                                         const json_t *exchange_proof);
 
 
@@ -112,9 +118,12 @@ typedef void
  */
 typedef void
 (*TALER_MERCHANTDB_TransferCallback)(void *cls,
-                                     const struct GNUNET_HashCode *h_contract_terms,
-                                     const struct TALER_CoinSpendPublicKeyP *coin_pub,
-                                     const struct TALER_WireTransferIdentifierRawP *wtid,
+                                     const struct
+                                     GNUNET_HashCode *h_contract_terms,
+                                     const struct
+                                     TALER_CoinSpendPublicKeyP *coin_pub,
+                                     const struct
+                                     TALER_WireTransferIdentifierRawP *wtid,
                                      struct GNUNET_TIME_Absolute execution_time,
                                      const json_t *exchange_proof);
 
@@ -142,7 +151,8 @@ typedef void
  */
 typedef void
 (*TALER_MERCHANTDB_RefundCallback)(void *cls,
-                                   const struct TALER_CoinSpendPublicKeyP *coin_pub,
+                                   const struct
+                                   TALER_CoinSpendPublicKeyP *coin_pub,
                                    uint64_t rtransaction_id,
                                    const char *reason,
                                    const struct TALER_Amount *refund_amount,
@@ -151,6 +161,12 @@ typedef void
 
 /**
  * Handle to interact with the database.
+ *
+ * Functions ending with "_TR" run their OWN transaction scope
+ * and MUST NOT be called from within a transaction setup by the
+ * caller.  Functions ending with "_NT" require the caller to
+ * setup a transaction scope.  Functions without a suffix are
+ * simple, single SQL queries that MAY be used either way.
  */
 struct TALER_MERCHANTDB_Plugin
 {
@@ -175,15 +191,6 @@ struct TALER_MERCHANTDB_Plugin
   int
   (*drop_tables) (void *cls);
 
-  /**
-   * Initialize merchant tables
-   *
-   * @param cls closure
-   * @return #GNUNET_OK upon success; #GNUNET_SYSERR upon failure
-   */
-  int
-  (*initialize) (void *cls);
-
   /**
    * Insert order into db.
    *
@@ -195,11 +202,11 @@ struct TALER_MERCHANTDB_Plugin
    * @return transaction status
    */
   enum GNUNET_DB_QueryStatus
-  (*insert_order) (void *cls,
-                   const char *order_id,
-                   const struct TALER_MerchantPublicKeyP *merchant_pub,
-                   struct GNUNET_TIME_Absolute timestamp,
-                   const json_t *contract_terms);
+  (*insert_order)(void *cls,
+                  const char *order_id,
+                  const struct TALER_MerchantPublicKeyP *merchant_pub,
+                  struct GNUNET_TIME_Absolute timestamp,
+                  const json_t *contract_terms);
 
 
   /**
@@ -214,11 +221,11 @@ struct TALER_MERCHANTDB_Plugin
    * @return transaction status
    */
   enum GNUNET_DB_QueryStatus
-  (*insert_contract_terms) (void *cls,
-                            const char *order_id,
-                            const struct TALER_MerchantPublicKeyP *merchant_pub,
-                            struct GNUNET_TIME_Absolute timestamp,
-                            const json_t *contract_terms);
+  (*insert_contract_terms)(void *cls,
+                           const char *order_id,
+                           const struct TALER_MerchantPublicKeyP *merchant_pub,
+                           struct GNUNET_TIME_Absolute timestamp,
+                           const json_t *contract_terms);
 
   /**
    * Mark contract terms as payed.  Needed by /history as only payed
@@ -233,30 +240,62 @@ struct TALER_MERCHANTDB_Plugin
    * @param cls closure
    * @param h_contract_terms hash of the contract that is now paid
    * @param merchant_pub merchant's public key
-   * @param last_session_id session id used for the payment, NULL
-   *        if payment was not session-bound
    * @return transaction status
    */
   enum GNUNET_DB_QueryStatus
-  (*mark_proposal_paid) (void *cls,
-                         const struct GNUNET_HashCode *h_contract_terms,
-                         const struct TALER_MerchantPublicKeyP *merchant_pub,
-                         const char *last_session_id);
+  (*mark_proposal_paid)(void *cls,
+                        const struct GNUNET_HashCode *h_contract_terms,
+                        const struct TALER_MerchantPublicKeyP *merchant_pub);
+
+  /**
+   * Store the order ID that was used to pay for a resource within a session.
+   *
+   * @param cls closure
+   * @param session_id session id
+   * @param fulfillment_url URL that canonically identifies the resource
+   *        being paid for
+   * @param order_id the order ID that was used when paying for the resource URL
+   * @param merchant_pub public key of the merchant, identifying the instance
+   * @return transaction status
+   */
+  enum GNUNET_DB_QueryStatus
+  (*insert_session_info)(void *cls,
+                         const char *session_id,
+                         const char *fulfillment_url,
+                         const char *order_id,
+                         const struct TALER_MerchantPublicKeyP *merchant_pub);
+
+  /**
+   * Retrieve the order ID that was used to pay for a resource within a session.
+   *
+   * @param cls closure
+   * @param[out] order_id location to store the order ID that was used when
+   *             paying for the resource URL
+   * @param session_id session id
+   * @param fulfillment_url URL that canonically identifies the resource
+   *        being paid for
+   * @param merchant_pub public key of the merchant, identifying the instance
+   * @return transaction status
+   */
+  enum GNUNET_DB_QueryStatus
+  (*find_session_info)(void *cls,
+                       char **order_id,
+                       const char *session_id,
+                       const char *fulfillment_url,
+                       const struct TALER_MerchantPublicKeyP *merchant_pub);
 
   /**
    * Retrieve proposal data given its order ID.
    *
    * @param cls closure
    * @param[out] contract_terms where to store the result
-   * @param[out] last_session_id where to store the result
    * @param order_id order_id used to lookup.
    * @param merchant_pub instance's public key.
    * @return transaction status
    */
   enum GNUNET_DB_QueryStatus
-  (*find_contract_terms) (void *cls,
+  (*find_contract_terms)(void *cls,
                          json_t **contract_terms,
-                         char **last_session_id,
                          const char *order_id,
                          const struct TALER_MerchantPublicKeyP *merchant_pub);
 
@@ -270,10 +309,10 @@ struct TALER_MERCHANTDB_Plugin
    * @return transaction status
    */
   enum GNUNET_DB_QueryStatus
-  (*find_order) (void *cls,
-                 json_t **contract_terms,
-                 const char *order_id,
-                 const struct TALER_MerchantPublicKeyP *merchant_pub);
+  (*find_order)(void *cls,
+                json_t **contract_terms,
+                const char *order_id,
+                const struct TALER_MerchantPublicKeyP *merchant_pub);
 
 
   /**
@@ -286,10 +325,12 @@ struct TALER_MERCHANTDB_Plugin
    * @return transaction status
    */
   enum GNUNET_DB_QueryStatus
-  (*find_contract_terms_from_hash) (void *cls,
+  (*find_contract_terms_from_hash)(void *cls,
                                    json_t **contract_terms,
-                                   const struct GNUNET_HashCode *h_contract_terms,
-                                   const struct TALER_MerchantPublicKeyP *merchant_pub);
+                                   const struct
+                                   GNUNET_HashCode *h_contract_terms,
+                                   const struct
+                                   TALER_MerchantPublicKeyP *merchant_pub);
 
 
   /**
@@ -302,39 +343,46 @@ struct TALER_MERCHANTDB_Plugin
    * @return transaction status
    */
   enum GNUNET_DB_QueryStatus
-  (*find_paid_contract_terms_from_hash) (void *cls,
-                                         json_t **contract_terms,
-                                         const struct GNUNET_HashCode *h_contract_terms,
-                                         const struct TALER_MerchantPublicKeyP *merchant_pub);
+  (*find_paid_contract_terms_from_hash)(void *cls,
+                                        json_t **contract_terms,
+                                        const struct
+                                        GNUNET_HashCode *h_contract_terms,
+                                        const struct
+                                        TALER_MerchantPublicKeyP *merchant_pub);
 
 
   /**
-   * Return proposals whose timestamp are older than `date`.
+   * Return proposals whose timestamps are younger than `date`.
    * Among those proposals, only those ones being between the
    * start-th and (start-nrows)-th record are returned.  The rows
    * are sorted having the youngest first.
    *
    * @param cls our plugin handle.
-   * @param date only results older than this date are returned.
+   * @param date only results younger than this date are returned.
    * @param merchant_pub instance's public key; only rows related to this
    * instance are returned.
    * @param start only rows with serial id less than start are returned.
    * @param nrows only nrows rows are returned.
-   * @param future if set to #GNUNET_YES, retrieves rows younger than `date`.
+   * @param past if set to #GNUNET_YES, retrieves rows older than `date`.
+   * @param ascending if GNUNET_YES, then results will be sorted with youngest first.
    * This is typically used to show live updates on the merchant's backoffice
    * @param cb function to call with transaction data, can be NULL.
    * @param cb_cls closure for @a cb
    * @return transaction status
    */
   enum GNUNET_DB_QueryStatus
-  (*find_contract_terms_by_date_and_range) (void *cls,
-					    struct GNUNET_TIME_Absolute date,
-					    const struct TALER_MerchantPublicKeyP *merchant_pub,
-					    uint64_t start,
-					    uint64_t nrows,
-					    int future,
-					    TALER_MERCHANTDB_ProposalDataCallback cb,
-					    void *cb_cls);
+  (*find_contract_terms_by_date_and_range)(void *cls,
+                                           struct GNUNET_TIME_Absolute date,
+                                           const struct
+                                           TALER_MerchantPublicKeyP *
+                                           merchant_pub,
+                                           uint64_t start,
+                                           uint64_t nrows,
+                                           int past,
+                                           unsigned int ascending,
+                                           TALER_MERCHANTDB_ProposalDataCallback
+                                           cb,
+                                           void *cb_cls);
 
   /**
    * Lookup for a proposal, respecting the signature used by the
@@ -348,9 +396,10 @@ struct TALER_MERCHANTDB_Plugin
    * @return transaction status
    */
   enum GNUNET_DB_QueryStatus
-  (*find_contract_terms_history) (void *cls,
+  (*find_contract_terms_history)(void *cls,
                                  const char *order_id,
-                                 const struct TALER_MerchantPublicKeyP *merchant_pub,
+                                 const struct
+                                 TALER_MerchantPublicKeyP *merchant_pub,
                                  TALER_MERCHANTDB_ProposalDataCallback cb,
                                  void *cb_cls);
 
@@ -369,12 +418,13 @@ struct TALER_MERCHANTDB_Plugin
    * @return transaction status
    */
   enum GNUNET_DB_QueryStatus
-  (*find_contract_terms_by_date) (void *cls,
-				  struct GNUNET_TIME_Absolute date,
-				  const struct TALER_MerchantPublicKeyP *merchant_pub,
-				  uint64_t nrows,
-				  TALER_MERCHANTDB_ProposalDataCallback cb,
-				  void *cb_cls);
+  (*find_contract_terms_by_date)(void *cls,
+                                 struct GNUNET_TIME_Absolute date,
+                                 const struct
+                                 TALER_MerchantPublicKeyP *merchant_pub,
+                                 uint64_t nrows,
+                                 TALER_MERCHANTDB_ProposalDataCallback cb,
+                                 void *cb_cls);
 
 
   /**
@@ -393,17 +443,17 @@ struct TALER_MERCHANTDB_Plugin
    * @return transaction status
    */
   enum GNUNET_DB_QueryStatus
-  (*store_deposit) (void *cls,
-                    const struct GNUNET_HashCode *h_contract_terms,
-                    const struct TALER_MerchantPublicKeyP *merchant_pub,
-                    const struct TALER_CoinSpendPublicKeyP *coin_pub,
-		    const char *exchange_url,
-                    const struct TALER_Amount *amount_with_fee,
-                    const struct TALER_Amount *deposit_fee,
-                    const struct TALER_Amount *refund_fee,
-                    const struct TALER_Amount *wire_fee,
-                    const struct TALER_ExchangePublicKeyP *signkey_pub,
-                    const json_t *exchange_proof);
+  (*store_deposit)(void *cls,
+                   const struct GNUNET_HashCode *h_contract_terms,
+                   const struct TALER_MerchantPublicKeyP *merchant_pub,
+                   const struct TALER_CoinSpendPublicKeyP *coin_pub,
+                   const char *exchange_url,
+                   const struct TALER_Amount *amount_with_fee,
+                   const struct TALER_Amount *deposit_fee,
+                   const struct TALER_Amount *refund_fee,
+                   const struct TALER_Amount *wire_fee,
+                   const struct TALER_ExchangePublicKeyP *signkey_pub,
+                   const json_t *exchange_proof);
 
 
   /**
@@ -418,10 +468,11 @@ struct TALER_MERCHANTDB_Plugin
    * @return transaction status
    */
   enum GNUNET_DB_QueryStatus
-  (*store_coin_to_transfer) (void *cls,
-                             const struct GNUNET_HashCode *h_contract_terms,
-                             const struct TALER_CoinSpendPublicKeyP *coin_pub,
-                             const struct TALER_WireTransferIdentifierRawP *wtid);
+  (*store_coin_to_transfer)(void *cls,
+                            const struct GNUNET_HashCode *h_contract_terms,
+                            const struct TALER_CoinSpendPublicKeyP *coin_pub,
+                            const struct
+                            TALER_WireTransferIdentifierRawP *wtid);
 
 
   /**
@@ -436,12 +487,13 @@ struct TALER_MERCHANTDB_Plugin
    * @return transaction status
    */
   enum GNUNET_DB_QueryStatus
-  (*store_transfer_to_proof) (void *cls,
-                              const char *exchange_url,
-                              const struct TALER_WireTransferIdentifierRawP *wtid,
-                              struct GNUNET_TIME_Absolute execution_time,
-                              const struct TALER_ExchangePublicKeyP *signkey_pub,
-                              const json_t *exchange_proof);
+  (*store_transfer_to_proof)(void *cls,
+                             const char *exchange_url,
+                             const struct
+                             TALER_WireTransferIdentifierRawP *wtid,
+                             struct GNUNET_TIME_Absolute execution_time,
+                             const struct TALER_ExchangePublicKeyP *signkey_pub,
+                             const json_t *exchange_proof);
 
 
   /**
@@ -460,15 +512,16 @@ struct TALER_MERCHANTDB_Plugin
    * @return transaction status code
    */
   enum GNUNET_DB_QueryStatus
-  (*store_wire_fee_by_exchange) (void *cls,
-				 const struct TALER_MasterPublicKeyP *exchange_pub,
-				 const struct GNUNET_HashCode *h_wire_method,
-				 const struct TALER_Amount *wire_fee,
-				 const struct TALER_Amount *closing_fee,
-				 struct GNUNET_TIME_Absolute start_date,
-				 struct GNUNET_TIME_Absolute end_date,
-				 const struct TALER_MasterSignatureP *exchange_sig);
-
+  (*store_wire_fee_by_exchange)(void *cls,
+                                const struct
+                                TALER_MasterPublicKeyP *exchange_pub,
+                                const struct GNUNET_HashCode *h_wire_method,
+                                const struct TALER_Amount *wire_fee,
+                                const struct TALER_Amount *closing_fee,
+                                struct GNUNET_TIME_Absolute start_date,
+                                struct GNUNET_TIME_Absolute end_date,
+                                const struct
+                                TALER_MasterSignatureP *exchange_sig);
 
 
   /**
@@ -483,11 +536,11 @@ struct TALER_MERCHANTDB_Plugin
    * @return transaction status
    */
   enum GNUNET_DB_QueryStatus
-  (*find_payments) (void *cls,
-                    const struct GNUNET_HashCode *h_contract_terms,
-                    const struct TALER_MerchantPublicKeyP *merchant_pub,
-                    TALER_MERCHANTDB_CoinDepositCallback cb,
-                    void *cb_cls);
+  (*find_payments)(void *cls,
+                   const struct GNUNET_HashCode *h_contract_terms,
+                   const struct TALER_MerchantPublicKeyP *merchant_pub,
+                   TALER_MERCHANTDB_CoinDepositCallback cb,
+                   void *cb_cls);
 
 
   /**
@@ -503,12 +556,15 @@ struct TALER_MERCHANTDB_Plugin
    * @return transaction status
    */
   enum GNUNET_DB_QueryStatus
-  (*find_payments_by_hash_and_coin) (void *cls,
-                                     const struct GNUNET_HashCode *h_contract_terms,
-                                     const struct TALER_MerchantPublicKeyP *merchant_pub,
-                                     const struct TALER_CoinSpendPublicKeyP *coin_pub,
-                                     TALER_MERCHANTDB_CoinDepositCallback cb,
-                                     void *cb_cls);
+  (*find_payments_by_hash_and_coin)(void *cls,
+                                    const struct
+                                    GNUNET_HashCode *h_contract_terms,
+                                    const struct
+                                    TALER_MerchantPublicKeyP *merchant_pub,
+                                    const struct
+                                    TALER_CoinSpendPublicKeyP *coin_pub,
+                                    TALER_MERCHANTDB_CoinDepositCallback cb,
+                                    void *cb_cls);
 
 
   /**
@@ -525,10 +581,10 @@ struct TALER_MERCHANTDB_Plugin
    * @return transaction status
    */
   enum GNUNET_DB_QueryStatus
-  (*find_transfers_by_hash) (void *cls,
-                             const struct GNUNET_HashCode *h_contract_terms,
-                             TALER_MERCHANTDB_TransferCallback cb,
-                             void *cb_cls);
+  (*find_transfers_by_hash)(void *cls,
+                            const struct GNUNET_HashCode *h_contract_terms,
+                            TALER_MERCHANTDB_TransferCallback cb,
+                            void *cb_cls);
 
 
   /**
@@ -541,10 +597,10 @@ struct TALER_MERCHANTDB_Plugin
    * @return transaction status
    */
   enum GNUNET_DB_QueryStatus
-  (*find_deposits_by_wtid) (void *cls,
-                            const struct TALER_WireTransferIdentifierRawP *wtid,
-                            TALER_MERCHANTDB_CoinDepositCallback cb,
-                            void *cb_cls);
+  (*find_deposits_by_wtid)(void *cls,
+                           const struct TALER_WireTransferIdentifierRawP *wtid,
+                           TALER_MERCHANTDB_CoinDepositCallback cb,
+                           void *cb_cls);
 
 
   /**
@@ -558,11 +614,11 @@ struct TALER_MERCHANTDB_Plugin
    * @return transaction status
    */
   enum GNUNET_DB_QueryStatus
-  (*find_proof_by_wtid) (void *cls,
-                         const char *exchange_url,
-                         const struct TALER_WireTransferIdentifierRawP *wtid,
-                         TALER_MERCHANTDB_ProofCallback cb,
-                         void *cb_cls);
+  (*find_proof_by_wtid)(void *cls,
+                        const char *exchange_url,
+                        const struct TALER_WireTransferIdentifierRawP *wtid,
+                        TALER_MERCHANTDB_ProofCallback cb,
+                        void *cb_cls);
 
 
   /**
@@ -582,20 +638,22 @@ struct TALER_MERCHANTDB_Plugin
    * @return transaction status code
    */
   enum GNUNET_DB_QueryStatus
-  (*lookup_wire_fee) (void *cls,
-		      const struct TALER_MasterPublicKeyP *exchange_pub,
-		      const struct GNUNET_HashCode *h_wire_method,
-		      struct GNUNET_TIME_Absolute contract_date,
-		      struct TALER_Amount *wire_fee,
-		      struct TALER_Amount *closing_fee,
-		      struct GNUNET_TIME_Absolute *start_date,
-		      struct GNUNET_TIME_Absolute *end_date,
-		      struct TALER_MasterSignatureP *exchange_sig);
+  (*lookup_wire_fee)(void *cls,
+                     const struct TALER_MasterPublicKeyP *exchange_pub,
+                     const struct GNUNET_HashCode *h_wire_method,
+                     struct GNUNET_TIME_Absolute contract_date,
+                     struct TALER_Amount *wire_fee,
+                     struct TALER_Amount *closing_fee,
+                     struct GNUNET_TIME_Absolute *start_date,
+                     struct GNUNET_TIME_Absolute *end_date,
+                     struct TALER_MasterSignatureP *exchange_sig);
 
 
   /**
    * Function called when some backoffice staff decides to award or
-   * increase the refund on an existing contract.
+   * increase the refund on an existing contract.  This function
+   * MUST be called from within a transaction scope setup by the
+   * caller as it executes multiple SQL statements (NT).
    *
    * @param cls closure
    * @param merchant_pub merchant's instance public key
@@ -607,11 +665,13 @@ struct TALER_MERCHANTDB_Plugin
    * @return transaction status
    */
   enum GNUNET_DB_QueryStatus
-  (*increase_refund_for_contract)(void *cls,
-                                  const struct GNUNET_HashCode *h_contract_terms,
-                                  const struct TALER_MerchantPublicKeyP *merchant_pub,
-                                  const struct TALER_Amount *refund,
-                                  const char *reason);
+  (*increase_refund_for_contract_NT)(void *cls,
+                                     const struct
+                                     GNUNET_HashCode *h_contract_terms,
+                                     const struct
+                                     TALER_MerchantPublicKeyP *merchant_pub,
+                                     const struct TALER_Amount *refund,
+                                     const char *reason);
 
 
   /**
@@ -626,8 +686,10 @@ struct TALER_MERCHANTDB_Plugin
    */
   enum GNUNET_DB_QueryStatus
   (*get_refunds_from_contract_terms_hash)(void *cls,
-                                          const struct TALER_MerchantPublicKeyP *merchant_pub,
-                                          const struct GNUNET_HashCode *h_contract_terms,
+                                          const struct
+                                          TALER_MerchantPublicKeyP *merchant_pub,
+                                          const struct
+                                          GNUNET_HashCode *h_contract_terms,
                                           TALER_MERCHANTDB_RefundCallback rc,
                                           void *rc_cls);
 
@@ -648,11 +710,11 @@ struct TALER_MERCHANTDB_Plugin
    *      #GNUNET_DB_STATUS_SUCCESS_NO_RESULTS if @a credit_uuid already known
    */
   enum GNUNET_DB_QueryStatus
-  (*enable_tip_reserve)(void *cls,
-                        const struct TALER_ReservePrivateKeyP *reserve_priv,
-                        const struct GNUNET_HashCode *credit_uuid,
-                        const struct TALER_Amount *credit,
-                        struct GNUNET_TIME_Absolute expiration);
+  (*enable_tip_reserve_TR)(void *cls,
+                           const struct TALER_ReservePrivateKeyP *reserve_priv,
+                           const struct GNUNET_HashCode *credit_uuid,
+                           const struct TALER_Amount *credit,
+                           struct GNUNET_TIME_Absolute expiration);
 
 
   /**
@@ -662,6 +724,7 @@ struct TALER_MERCHANTDB_Plugin
    *
    * @param cls closure, typically a connection to the db
    * @param justification why was the tip approved
+   * @param extra extra data that will be given to the customer's wallet
    * @param amount how high is the tip (with fees)
    * @param reserve_priv which reserve is debited
    * @param exchange_url which exchange manages the tip
@@ -676,13 +739,14 @@ struct TALER_MERCHANTDB_Plugin
    *      #TALER_EC_NONE upon success
    */
   enum TALER_ErrorCode
-  (*authorize_tip)(void *cls,
-                   const char *justification,
-                   const struct TALER_Amount *amount,
-                   const struct TALER_ReservePrivateKeyP *reserve_priv,
-		   const char *exchange_url,
-                   struct GNUNET_TIME_Absolute *expiration,
-                   struct GNUNET_HashCode *tip_id);
+  (*authorize_tip_TR)(void *cls,
+                      const char *justification,
+                      const json_t *extra,
+                      const struct TALER_Amount *amount,
+                      const struct TALER_ReservePrivateKeyP *reserve_priv,
+                      const char *exchange_url,
+                      struct GNUNET_TIME_Absolute *expiration,
+                      struct GNUNET_HashCode *tip_id);
 
   /**
    * Get the total amount of authorized tips for a tipping reserve.
@@ -697,7 +761,8 @@ struct TALER_MERCHANTDB_Plugin
    */
   enum GNUNET_DB_QueryStatus
   (*get_authorized_tip_amount)(void *cls,
-                               const struct TALER_ReservePrivateKeyP *reserve_priv,
+                               const struct
+                               TALER_ReservePrivateKeyP *reserve_priv,
                                struct TALER_Amount *authorized_amount);
 
 
@@ -707,7 +772,9 @@ struct TALER_MERCHANTDB_Plugin
    * @param cls closure, typically a connection to the d
    * @param tip_id the unique ID for the tip
    * @param[out] exchange_url set to the URL of the exchange (unless NULL)
+   * @param[out] extra extra data to pass to the wallet
    * @param[out] amount set to the authorized amount (unless NULL)
+   * @param[out] amount_left set to the amount left (unless NULL)
    * @param[out] timestamp set to the timestamp of the tip authorization (unless NULL)
    * @return transaction status, usually
    *      #GNUNET_DB_STATUS_SUCCESS_ONE_RESULT for success
@@ -717,7 +784,9 @@ struct TALER_MERCHANTDB_Plugin
   (*lookup_tip_by_id)(void *cls,
                       const struct GNUNET_HashCode *tip_id,
                       char **exchange_url,
+                      json_t **extra,
                       struct TALER_Amount *amount,
+                      struct TALER_Amount *amount_left,
                       struct GNUNET_TIME_Absolute *timestamp);
 
 
@@ -737,11 +806,11 @@ struct TALER_MERCHANTDB_Plugin
    *      #TALER_EC_NONE upon success (@a reserve_priv was set)
    */
   enum TALER_ErrorCode
-  (*pickup_tip)(void *cls,
-                const struct TALER_Amount *amount,
-                const struct GNUNET_HashCode *tip_id,
-                const struct GNUNET_HashCode *pickup_id,
-                struct TALER_ReservePrivateKeyP *reserve_priv);
+  (*pickup_tip_TR)(void *cls,
+                   const struct TALER_Amount *amount,
+                   const struct GNUNET_HashCode *tip_id,
+                   const struct GNUNET_HashCode *pickup_id,
+                   struct TALER_ReservePrivateKeyP *reserve_priv);
 
 
   /**
@@ -785,7 +854,7 @@ struct TALER_MERCHANTDB_Plugin
    * @return transaction status code
    */
   enum GNUNET_DB_QueryStatus
-  (*commit) (void *cls);
+  (*commit)(void *cls);
 
 };
 
