@@ -2,22 +2,26 @@
   This file is part of TALER
   Copyright (C) 2014, 2015, 2016, 2017 GNUnet e.V. and INRIA
 
-  TALER is free software; you can redistribute it and/or modify it under the
-  terms of the GNU Lesser General Public License as published by the Free Software
-  Foundation; either version 2.1, or (at your option) any later version.
+  TALER is free software; you can redistribute it and/or modify
+  it under the terms of the GNU Lesser General Public License as
+  published by the Free Software Foundation; either version 2.1,
+  or (at your option) any later version.
 
-  TALER is distributed in the hope that it will be useful, but WITHOUT ANY
-  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.
+  TALER is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU Lesser General Public License for more details.
 
-  You should have received a copy of the GNU Lesser General Public License along with
-  TALER; see the file COPYING.LGPL.  If not, see
-  <http://www.gnu.org/licenses/>
+  You should have received a copy of the GNU Lesser General
+  Public License along with TALER; see the file COPYING.LGPL.
+  If not, see <http://www.gnu.org/licenses/>
 */
 /**
  * @file lib/merchant_api_pay.c
- * @brief Implementation of the /pay request of the merchant's HTTP API
+ * @brief Implementation of the /pay request
+ *        of the merchant's HTTP API
  * @author Christian Grothoff
+ * @author Marcello Stanisci
  */
 #include "platform.h"
 #include <curl/curl.h>
@@ -29,6 +33,7 @@
 #include <taler/taler_json_lib.h>
 #include <taler/taler_signatures.h>
 #include <taler/taler_exchange_service.h>
+#include <taler/taler_curl_lib.h>
 
 
 /**
@@ -42,11 +47,6 @@ struct TALER_MERCHANT_Pay
    */
   char *url;
 
-  /**
-   * JSON encoding of the request to POST.
-   */
-  char *json_enc;
-
   /**
    * Handle for the request.
    */
@@ -82,6 +82,11 @@ struct TALER_MERCHANT_Pay
    */
   struct GNUNET_CURL_Context *ctx;
 
+  /**
+   * Minor context that holds body and headers.
+   */
+  struct TALER_CURL_PostContext post_ctx;
+
   /**
    * The coins we are paying with.
    */
@@ -111,7 +116,7 @@ struct TALER_MERCHANT_Pay
  */
 static int
 check_abort_refund (struct TALER_MERCHANT_Pay *ph,
-		    const json_t *json)
+                    const json_t *json)
 {
   json_t *refunds;
   unsigned int num_refunds;
@@ -119,7 +124,7 @@ check_abort_refund (struct TALER_MERCHANT_Pay *ph,
   struct GNUNET_JSON_Specification spec[] = {
     GNUNET_JSON_spec_json ("refund_permissions", &refunds),
     GNUNET_JSON_spec_fixed_auto ("merchant_pub", &merchant_pub),
-    GNUNET_JSON_spec_end()
+    GNUNET_JSON_spec_end ()
   };
 
   if (GNUNET_OK !=
@@ -132,20 +137,20 @@ check_abort_refund (struct TALER_MERCHANT_Pay *ph,
   }
   num_refunds = json_array_size (refunds);
   {
-    struct TALER_MERCHANT_RefundEntry res[GNUNET_NZL(num_refunds)];
+    struct TALER_MERCHANT_RefundEntry res[GNUNET_NZL (num_refunds)];
 
-    for (unsigned int i=0;i<num_refunds;i++)
+    for (unsigned int i = 0; i<num_refunds; i++)
     {
       struct TALER_MerchantSignatureP *sig = &res[i].merchant_sig;
       json_t *refund = json_array_get (refunds, i);
       struct GNUNET_JSON_Specification spec_detail[] = {
         GNUNET_JSON_spec_fixed_auto ("merchant_sig",
-				     sig),
+                                     sig),
         GNUNET_JSON_spec_fixed_auto ("coin_pub",
-				     &res[i].coin_pub),
-	GNUNET_JSON_spec_uint64 ("rtransaction_id",
-				 &res[i].rtransaction_id),
-        GNUNET_JSON_spec_end()
+                                     &res[i].coin_pub),
+        GNUNET_JSON_spec_uint64 ("rtransaction_id",
+                                 &res[i].rtransaction_id),
+        GNUNET_JSON_spec_end ()
       };
       struct TALER_RefundRequestPS rr;
       int found;
@@ -156,54 +161,60 @@ check_abort_refund (struct TALER_MERCHANT_Pay *ph,
                              NULL, NULL))
       {
         GNUNET_break_op (0);
-	GNUNET_JSON_parse_free (spec);
+        GNUNET_JSON_parse_free (spec);
         return GNUNET_SYSERR;
       }
 
-      rr.purpose.purpose = htonl (TALER_SIGNATURE_MERCHANT_REFUND);
-      rr.purpose.size = htonl (sizeof (struct TALER_RefundRequestPS));
+      rr.purpose.purpose = htonl
+                             (TALER_SIGNATURE_MERCHANT_REFUND);
+      rr.purpose.size = htonl
+                          (sizeof (struct TALER_RefundRequestPS));
       rr.h_contract_terms = ph->h_contract_terms;
       rr.coin_pub = res[i].coin_pub;
       rr.merchant = merchant_pub;
       rr.rtransaction_id = GNUNET_htonll (res[i].rtransaction_id);
       found = -1;
-      for (unsigned int j=0;j<ph->num_coins;j++)
+      for (unsigned int j = 0; j<ph->num_coins; j++)
       {
-	if (0 == memcmp (&ph->coins[j].coin_pub,
-			 &res[i].coin_pub,
-			 sizeof (struct TALER_CoinSpendPublicKeyP)))
-	{
-	  TALER_amount_hton (&rr.refund_amount,
-			     &ph->coins[j].amount_with_fee);
-	  TALER_amount_hton (&rr.refund_fee,
-			     &ph->coins[j].refund_fee);
-	  found = j;
-	}
+        if (0 == memcmp (&ph->coins[j].coin_pub,
+                         &res[i].coin_pub,
+                         sizeof
+                         (struct TALER_CoinSpendPublicKeyP)))
+        {
+          TALER_amount_hton (&rr.refund_amount,
+                             &ph->coins[j].amount_with_fee);
+          TALER_amount_hton (&rr.refund_fee,
+                             &ph->coins[j].refund_fee);
+          found = j;
+        }
       }
       if (-1 == found)
       {
-	GNUNET_break_op (0);
-	return GNUNET_SYSERR;
+        GNUNET_break_op (0);
+        GNUNET_JSON_parse_free (spec);
+        return GNUNET_SYSERR;
       }
 
       if (GNUNET_OK !=
-	  GNUNET_CRYPTO_eddsa_verify (TALER_SIGNATURE_MERCHANT_REFUND,
-				      &rr.purpose,
-				      &sig->eddsa_sig,
-				      &merchant_pub.eddsa_pub))
+          GNUNET_CRYPTO_eddsa_verify
+            (TALER_SIGNATURE_MERCHANT_REFUND,
+            &rr.purpose,
+            &sig->eddsa_sig,
+            &merchant_pub.eddsa_pub))
       {
-	GNUNET_break_op (0);
-	return GNUNET_SYSERR;
+        GNUNET_break_op (0);
+        GNUNET_JSON_parse_free (spec);
+        return GNUNET_SYSERR;
       }
     }
     ph->abort_cb (ph->abort_cb_cls,
-		  MHD_HTTP_OK,
-		  TALER_EC_NONE,
-		  &merchant_pub,
-		  &ph->h_contract_terms,
-		  num_refunds,
-		  res,
-		  json);
+                  MHD_HTTP_OK,
+                  TALER_EC_NONE,
+                  &merchant_pub,
+                  &ph->h_contract_terms,
+                  num_refunds,
+                  res,
+                  json);
     ph->abort_cb = NULL;
   }
   GNUNET_JSON_parse_free (spec);
@@ -217,8 +228,8 @@ check_abort_refund (struct TALER_MERCHANT_Pay *ph,
  * coin was actually already spent!
  *
  * @param pc handle of the original coin we paid with
- * @param json cryptographic proof of coin's transaction history as
- *        was returned by the exchange/merchant
+ * @param json cryptographic proof of coin's transaction
+ *        history as was returned by the exchange/merchant
  * @return #GNUNET_OK if proof checks out
  */
 static int
@@ -229,7 +240,8 @@ check_coin_history (const struct TALER_MERCHANT_PaidCoin *pc,
   struct TALER_Amount spent_plus_contrib;
 
   if (GNUNET_OK !=
-      TALER_EXCHANGE_verify_coin_history (pc->amount_with_fee.currency,
+      TALER_EXCHANGE_verify_coin_history (NULL, /* do not verify fees */
+                                          pc->amount_with_fee.currency,
                                           &pc->coin_pub,
                                           json,
                                           &spent))
@@ -262,24 +274,25 @@ check_coin_history (const struct TALER_MERCHANT_PaidCoin *pc,
 
 
 /**
- * We got a 403 response back from the exchange (or the merchant).
+ * We got a 409 response back from the exchange (or the merchant).
  * Now we need to check the provided cryptographic proof that the
  * coin was actually already spent!
  *
  * @param ph handle of the original pay operation
- * @param json cryptographic proof returned by the exchange/merchant
+ * @param json cryptographic proof returned by the
+ *        exchange/merchant
  * @return #GNUNET_OK if proof checks out
  */
 static int
-check_forbidden (struct TALER_MERCHANT_Pay *ph,
-                 const json_t *json)
+check_conflict (struct TALER_MERCHANT_Pay *ph,
+                const json_t *json)
 {
   json_t *history;
   struct TALER_CoinSpendPublicKeyP coin_pub;
   struct GNUNET_JSON_Specification spec[] = {
     GNUNET_JSON_spec_json ("history", &history),
     GNUNET_JSON_spec_fixed_auto ("coin_pub", &coin_pub),
-    GNUNET_JSON_spec_end()
+    GNUNET_JSON_spec_end ()
   };
   int ret;
 
@@ -291,7 +304,7 @@ check_forbidden (struct TALER_MERCHANT_Pay *ph,
     GNUNET_break_op (0);
     return GNUNET_SYSERR;
   }
-  for (unsigned int i=0;i<ph->num_coins;i++)
+  for (unsigned int i = 0; i<ph->num_coins; i++)
   {
     if (0 == memcmp (&ph->coins[i].coin_pub,
                      &coin_pub,
@@ -321,119 +334,153 @@ check_forbidden (struct TALER_MERCHANT_Pay *ph,
 static void
 handle_pay_finished (void *cls,
                      long response_code,
-                     const json_t *json)
+                     const void *response)
 {
   struct TALER_MERCHANT_Pay *ph = cls;
+  const json_t *json = response;
+  enum TALER_ErrorCode ec;
 
   ph->job = NULL;
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
-	      "/pay completed with response code %u\n",
-	      (unsigned int) response_code);
+              "/pay completed with response code %u\n",
+              (unsigned int) response_code);
   if (0 == strcasecmp (ph->mode,
-		       "pay"))
+                       "pay"))
   {
     switch (response_code)
     {
     case 0:
+      ec = TALER_JSON_get_error_code (json);
       break;
     case MHD_HTTP_OK:
+      ec = TALER_EC_NONE;
+      break;
+    /* Tolerating Not Acceptable because sometimes
+     * - especially in tests - we might want to POST
+     * coins one at a time.  */
+    case MHD_HTTP_NOT_ACCEPTABLE:
+      ec = TALER_JSON_get_error_code (json);
       break;
     case MHD_HTTP_BAD_REQUEST:
-      /* This should never happen, either us or the merchant is buggy
-	 (or API version conflict); just pass JSON reply to the application */
+      ec = TALER_JSON_get_error_code (json);
+      /* This should never happen, either us
+       * or the merchant is buggy (or API version conflict);
+       * just pass JSON reply to the application */
       break;
-    case MHD_HTTP_FORBIDDEN:
-      if (GNUNET_OK != check_forbidden (ph,
-					json))
+    case MHD_HTTP_CONFLICT:
+      ec = TALER_JSON_get_error_code (json);
+      if (GNUNET_OK != check_conflict (ph,
+                                       json))
       {
-	GNUNET_break_op (0);
-	response_code = 0;
+        GNUNET_break_op (0);
+        response_code = 0;
       }
       break;
-    case MHD_HTTP_UNAUTHORIZED:
-      /* Nothing really to verify, merchant says one of the signatures is
-	 invalid; as we checked them, this should never happen, we
-	 should pass the JSON reply to the application */
+    case MHD_HTTP_FORBIDDEN:
+      ec = TALER_JSON_get_error_code (json);
+      /* Nothing really to verify, merchant says one of the
+       * signatures is invalid; as we checked them, this
+       * should never happen, we should pass the JSON reply
+       * to the application */
       break;
     case MHD_HTTP_NOT_FOUND:
+      ec = TALER_JSON_get_error_code (json);
       /* Nothing really to verify, this should never
-	 happen, we should pass the JSON reply to the application */
+   happen, we should pass the JSON reply to the
+         application */
       break;
     case MHD_HTTP_INTERNAL_SERVER_ERROR:
-      /* Server had an internal issue; we should retry, but this API
-	 leaves this to the application */
+      ec = TALER_JSON_get_error_code (json);
+      /* Server had an internal issue; we should retry,
+         but this API leaves this to the application */
+      break;
+    case MHD_HTTP_SERVICE_UNAVAILABLE:
+      ec = TALER_JSON_get_error_code (json);
+      /* Exchange couldn't respond properly; the retry is
+         left to the application */
       break;
     default:
+      ec = TALER_JSON_get_error_code (json);
       /* unexpected response code */
       GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-		  "Unexpected response code %u\n",
-		  (unsigned int) response_code);
+                  "Unexpected response code %u\n",
+                  (unsigned int) response_code);
       GNUNET_break (0);
       response_code = 0;
       break;
     }
     ph->pay_cb (ph->pay_cb_cls,
-		response_code,
-		TALER_JSON_get_error_code (json),
-		json);
+                response_code,
+                ec,
+                json);
   }
   else
   {
     GNUNET_assert (0 == strcasecmp (ph->mode,
-				    "abort-refund"));
+                                    "abort-refund"));
 
     switch (response_code)
     {
     case 0:
+      ec = TALER_JSON_get_error_code (json);
       break;
     case MHD_HTTP_OK:
       if (GNUNET_OK ==
-	  check_abort_refund (ph,
-			      json))
+          check_abort_refund (ph,
+                              json))
       {
-	TALER_MERCHANT_pay_cancel (ph);
-	return;
+        TALER_MERCHANT_pay_cancel (ph);
+        return;
       }
       response_code = 0;
+      ec = TALER_EC_NONE;
       break;
     case MHD_HTTP_BAD_REQUEST:
-      /* This should never happen, either us or the merchant is buggy
-	 (or API version conflict); just pass JSON reply to the application */
+      ec = TALER_JSON_get_error_code (json);
+      /* This should never happen, either us or the
+         merchant is buggy (or API version conflict); just
+         pass JSON reply to the application */
+      break;
+    case MHD_HTTP_CONFLICT:
+      ec = TALER_JSON_get_error_code (json);
       break;
     case MHD_HTTP_FORBIDDEN:
-      break;
-    case MHD_HTTP_UNAUTHORIZED:
-      /* Nothing really to verify, merchant says one of the signatures is
-	 invalid; as we checked them, this should never happen, we
-	 should pass the JSON reply to the application */
+      ec = TALER_JSON_get_error_code (json);
+      /* Nothing really to verify, merchant says one of
+         the signatures is invalid; as we checked them,
+         this should never happen, we should pass the JSON
+         reply to the application */
       break;
     case MHD_HTTP_NOT_FOUND:
+      ec = TALER_JSON_get_error_code (json);
       /* Nothing really to verify, this should never
-	 happen, we should pass the JSON reply to the application */
+   happen, we should pass the JSON reply to the
+         application */
       break;
     case MHD_HTTP_INTERNAL_SERVER_ERROR:
-      /* Server had an internal issue; we should retry, but this API
-	 leaves this to the application */
+      ec = TALER_JSON_get_error_code (json);
+      /* Server had an internal issue; we should retry,
+         but this API leaves this to the application */
       break;
     default:
+      ec = TALER_JSON_get_error_code (json);
       /* unexpected response code */
       GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-		  "Unexpected response code %u\n",
-		  (unsigned int) response_code);
+                  "Unexpected response code %u\n",
+                  (unsigned int) response_code);
       GNUNET_break (0);
       response_code = 0;
       break;
     }
     ph->abort_cb (ph->abort_cb_cls,
-		  response_code,
-		  TALER_JSON_get_error_code (json),
-		  NULL,
-		  NULL,
-		  0,
-		  NULL,
-		  json);
+                  response_code,
+                  ec,
+                  NULL,
+                  NULL,
+                  0,
+                  NULL,
+                  json);
   }
-
   TALER_MERCHANT_pay_cancel (ph);
 }
 
@@ -448,24 +495,26 @@ handle_pay_finished (void *cls,
  * @param num_coins number of coins used to pay
  * @param coins array of coins we use to pay
  * @param mode mode string to use ("pay" or "abort-refund").
- * @param pay_cb the callback to call when a reply for this request is available
+ * @param pay_cb the callback to call when a reply for this
+ *        request is available
  * @param pay_cb_cls closure for @a pay_cb
  * @param abort_cb callback to call for the abort-refund variant
  * @param abort_cb_cls closure for @a abort_cb
  * @return a handle for this request
  */
 static struct TALER_MERCHANT_Pay *
-request_pay_generic (struct GNUNET_CURL_Context *ctx,
-		     const char *merchant_url,
-		     const struct TALER_MerchantPublicKeyP *merchant_pub,
-		     const char *order_id,
-		     unsigned int num_coins,
-		     const struct TALER_MERCHANT_PaidCoin *coins,
-		     const char *mode,
-		     TALER_MERCHANT_PayCallback pay_cb,
-		     void *pay_cb_cls,
-		     TALER_MERCHANT_PayRefundCallback abort_cb,
-		     void *abort_cb_cls)
+request_pay_generic
+  (struct GNUNET_CURL_Context *ctx,
+  const char *merchant_url,
+  const struct TALER_MerchantPublicKeyP *merchant_pub,
+  const char *order_id,
+  unsigned int num_coins,
+  const struct TALER_MERCHANT_PaidCoin *coins,
+  const char *mode,
+  TALER_MERCHANT_PayCallback pay_cb,
+  void *pay_cb_cls,
+  TALER_MERCHANT_PayRefundCallback abort_cb,
+  void *abort_cb_cls)
 {
   struct TALER_MERCHANT_Pay *ph;
   json_t *pay_obj;
@@ -480,19 +529,19 @@ request_pay_generic (struct GNUNET_CURL_Context *ctx,
     return NULL;
   }
   j_coins = json_array ();
-  for (unsigned int i=0;i<num_coins;i++)
+  for (unsigned int i = 0; i<num_coins; i++)
   {
     json_t *j_coin;
     const struct TALER_MERCHANT_PaidCoin *pc = &coins[i];
     struct TALER_Amount fee;
 
     if (GNUNET_SYSERR ==
-	TALER_amount_subtract (&fee,
-			       &pc->amount_with_fee,
-			       &pc->amount_without_fee))
+        TALER_amount_subtract (&fee,
+                               &pc->amount_with_fee,
+                               &pc->amount_without_fee))
     {
       /* Integer underflow, fee larger than total amount?
-	 This should not happen (client violated API!) */
+         This should not happen (client violated API!) */
       GNUNET_break (0);
       json_decref (j_coins);
       return NULL;
@@ -505,32 +554,36 @@ request_pay_generic (struct GNUNET_CURL_Context *ctx,
     else
     {
       if ( (GNUNET_OK !=
-	    TALER_amount_add (&total_fee,
-			      &total_fee,
-			      &fee)) ||
-	   (GNUNET_OK !=
-	    TALER_amount_add (&total_amount,
-			      &total_amount,
-			      &pc->amount_with_fee)) )
+            TALER_amount_add (&total_fee,
+                              &total_fee,
+                              &fee)) ||
+           (GNUNET_OK !=
+            TALER_amount_add (&total_amount,
+                              &total_amount,
+                              &pc->amount_with_fee)) )
       {
-	/* integer overflow */
-	GNUNET_break (0);
-	json_decref (j_coins);
-	return NULL;
+        /* integer overflow */
+        GNUNET_break (0);
+        json_decref (j_coins);
+        return NULL;
       }
     }
 
     /* create JSON for this coin */
-    j_coin = json_pack ("{s:o, s:o," /* contribution/coin_pub */
-			" s:s, s:o," /* exchange_url / denom_pub */
-			" s:o, s:o}", /* ub_sig / coin_sig */
-			"contribution", TALER_JSON_from_amount (&pc->amount_with_fee),
-			"coin_pub", GNUNET_JSON_from_data_auto (&pc->coin_pub),
-			"exchange_url", pc->exchange_url,
-			"denom_pub", GNUNET_JSON_from_rsa_public_key (pc->denom_pub.rsa_public_key),
-			"ub_sig", GNUNET_JSON_from_rsa_signature (pc->denom_sig.rsa_signature),
-			"coin_sig", GNUNET_JSON_from_data_auto (&pc->coin_sig)
-			);
+    j_coin = json_pack
+               ("{s:o, s:o," /* contribution/coin_pub */
+               " s:s, s:o," /* exchange_url / denom_pub */
+               " s:o, s:o}", /* ub_sig / coin_sig */
+               "contribution", TALER_JSON_from_amount
+                 (&pc->amount_with_fee),
+               "coin_pub", GNUNET_JSON_from_data_auto
+                 (&pc->coin_pub),
+               "exchange_url", pc->exchange_url,
+               "denom_pub", GNUNET_JSON_from_rsa_public_key
+                 (pc->denom_pub.rsa_public_key),
+               "ub_sig", GNUNET_JSON_from_rsa_signature
+                 (pc->denom_sig.rsa_signature),
+               "coin_sig", GNUNET_JSON_from_data_auto (&pc->coin_sig));
     if (0 !=
         json_array_append_new (j_coins,
                                j_coin))
@@ -542,15 +595,16 @@ request_pay_generic (struct GNUNET_CURL_Context *ctx,
   }
 
   pay_obj = json_pack ("{"
-		       " s:s," /* mode */
+                       " s:s," /* mode */
                        " s:o," /* coins */
                        " s:s," /* order_id */
                        " s:o," /* merchant_pub */
                        "}",
-		       "mode", mode,
-		       "coins", j_coins,
+                       "mode", mode,
+                       "coins", j_coins,
                        "order_id", order_id,
-                       "merchant_pub", GNUNET_JSON_from_data_auto (merchant_pub));
+                       "merchant_pub", GNUNET_JSON_from_data_auto
+                         (merchant_pub));
   if (NULL == pay_obj)
   {
     GNUNET_break (0);
@@ -563,7 +617,15 @@ request_pay_generic (struct GNUNET_CURL_Context *ctx,
   ph->abort_cb_cls = abort_cb_cls;
   ph->pay_cb = pay_cb;
   ph->pay_cb_cls = pay_cb_cls;
-  ph->url = TALER_url_join (merchant_url, "/public/pay", NULL);
+  ph->url = TALER_url_join (merchant_url, "pay", NULL);
+  if (NULL == ph->url)
+  {
+    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                "Could not construct request URL.\n");
+    json_decref (pay_obj);
+    GNUNET_free (ph);
+    return NULL;
+  }
   ph->num_coins = num_coins;
   ph->coins = GNUNET_new_array (num_coins,
                                 struct TALER_MERCHANT_PaidCoin);
@@ -572,77 +634,83 @@ request_pay_generic (struct GNUNET_CURL_Context *ctx,
           num_coins * sizeof (struct TALER_MERCHANT_PaidCoin));
 
   eh = curl_easy_init ();
-  GNUNET_assert (NULL != (ph->json_enc =
-                          json_dumps (pay_obj,
-                                      JSON_COMPACT)));
+
+  if (GNUNET_OK != TALER_curl_easy_post (&ph->post_ctx,
+                                         eh,
+                                         pay_obj))
+  {
+    GNUNET_break (0);
+    json_decref (pay_obj);
+    GNUNET_free (ph);
+    return NULL;
+  }
+
   json_decref (pay_obj);
-  GNUNET_assert (CURLE_OK ==
-                 curl_easy_setopt (eh,
-                                   CURLOPT_URL,
-                                   ph->url));
-  GNUNET_assert (CURLE_OK ==
-                 curl_easy_setopt (eh,
-                                   CURLOPT_POSTFIELDS,
-                                   ph->json_enc));
-  GNUNET_assert (CURLE_OK ==
-                 curl_easy_setopt (eh,
-                                   CURLOPT_POSTFIELDSIZE,
-                                   strlen (ph->json_enc)));
-  ph->job = GNUNET_CURL_job_add (ctx,
-                                 eh,
-                                 GNUNET_YES,
-                                 &handle_pay_finished,
-                                 ph);
+  GNUNET_assert (CURLE_OK == curl_easy_setopt (eh,
+                                               CURLOPT_URL,
+                                               ph->url));
+  ph->job = GNUNET_CURL_job_add2 (ctx,
+                                  eh,
+                                  ph->post_ctx.headers,
+                                  &handle_pay_finished,
+                                  ph);
   return ph;
 }
 
 
 /**
- * Pay a merchant.  API for wallets that have the coin's private keys.
- * _NOTE_: this function does NOT calculate each coin amount in order
- * to match the contract total price.  This calculation is to be made
- * by the logic using this library.
+ * Pay a merchant.  API for wallets that have the coin's private
+ * keys.
+ * _NOTE_: this function does NOT calculate each coin amount in
+ * order to match the contract total price.  This calculation is
+ * to be made by the logic using this library.
  *
  * @param ctx the execution loop context
  * @param merchant_url base URL of the merchant's backend
- * @param instance which merchant instance will receive this payment
  * @param h_contract_terms hashcode of the proposal being paid
- * @param amount total value of the contract to be paid to the merchant
- * @param max_fee maximum fee covered by the merchant (according to the contract)
- * @param merchant_pub the public key of the merchant (used to identify the merchant for refund requests)
- * @param merchant_sig signature from the merchant over the original contract
- * @param timestamp timestamp when the contract was finalized, must match approximately the current time of the merchant
- * @param refund_deadline date until which the merchant can issue a refund to the customer via the merchant (can be zero if refunds are not allowed)
+ * @param amount total value of the contract to be paid to the
+ *        merchant
+ * @param max_fee maximum fee covered by the merchant
+ *        (according to the contract)
+ * @param merchant_pub the public key of the merchant
+ *        (used to identify the merchant for refund requests)
+ * @param merchant_sig signature from the merchant over the
+ *        original contract
+ * @param timestamp timestamp when the contract was finalized,
+ *        must match approximately the current time of the merchant
+ * @param refund_deadline date until which the merchant can issue
+ *        a refund to the customer via the merchant (can be zero
+ *        if refunds are not allowed)
  * @param pay_deadline maximum time limit to pay for this contract
  * @param h_wire hash of the merchant’s account details
  * @param order_id order id of the proposal being paid
  * @param num_coins number of coins used to pay
  * @param coins array of coins we use to pay
- * @param pay_cb the callback to call when a reply for this request is available
+ * @param pay_cb the callback to call when a reply for this
+ *        request is available
  * @param pay_cb_cls closure for @a pay_cb
  * @return a handle for this request
  */
 static struct TALER_MERCHANT_Pay *
 prepare_pay_generic (struct GNUNET_CURL_Context *ctx,
-		     const char *merchant_url,
-		     const char *instance,
-		     const struct GNUNET_HashCode *h_contract_terms,
-		     const struct TALER_Amount *amount,
-		     const struct TALER_Amount *max_fee,
-		     const struct TALER_MerchantPublicKeyP *merchant_pub,
-		     const struct TALER_MerchantSignatureP *merchant_sig,
-		     struct GNUNET_TIME_Absolute timestamp,
-		     struct GNUNET_TIME_Absolute refund_deadline,
-		     struct GNUNET_TIME_Absolute pay_deadline,
-		     const struct GNUNET_HashCode *h_wire,
-		     const char *order_id,
-		     unsigned int num_coins,
-		     const struct TALER_MERCHANT_PayCoin *coins,
-		     const char *mode,
-		     TALER_MERCHANT_PayCallback pay_cb,
-		     void *pay_cb_cls,
-		     TALER_MERCHANT_PayRefundCallback abort_cb,
-		     void *abort_cb_cls)
+                     const char *merchant_url,
+                     const struct GNUNET_HashCode *h_contract_terms,
+                     const struct TALER_Amount *amount,
+                     const struct TALER_Amount *max_fee,
+                     const struct TALER_MerchantPublicKeyP *merchant_pub,
+                     const struct TALER_MerchantSignatureP *merchant_sig,
+                     struct GNUNET_TIME_Absolute timestamp,
+                     struct GNUNET_TIME_Absolute refund_deadline,
+                     struct GNUNET_TIME_Absolute pay_deadline,
+                     const struct GNUNET_HashCode *h_wire,
+                     const char *order_id,
+                     unsigned int num_coins,
+                     const struct TALER_MERCHANT_PayCoin *coins,
+                     const char *mode,
+                     TALER_MERCHANT_PayCallback pay_cb,
+                     void *pay_cb_cls,
+                     TALER_MERCHANT_PayRefundCallback abort_cb,
+                     void *abort_cb_cls)
 {
   struct TALER_DepositRequestPS dr;
   struct TALER_MERCHANT_PaidCoin pc[num_coins];
@@ -667,7 +735,7 @@ prepare_pay_generic (struct GNUNET_CURL_Context *ctx,
   dr.refund_deadline = GNUNET_TIME_absolute_hton (refund_deadline);
   dr.merchant = *merchant_pub;
 
-  for (unsigned int i=0;i<num_coins;i++)
+  for (unsigned int i = 0; i<num_coins; i++)
   {
     const struct TALER_MERCHANT_PayCoin *coin = &coins[i]; // coin priv.
     struct TALER_MERCHANT_PaidCoin *p = &pc[i]; // coin pub.
@@ -675,21 +743,21 @@ prepare_pay_generic (struct GNUNET_CURL_Context *ctx,
 
     /* prepare 'dr' for this coin to generate coin signature */
     GNUNET_CRYPTO_eddsa_key_get_public (&coin->coin_priv.eddsa_priv,
-					&dr.coin_pub.eddsa_pub);
+                                        &dr.coin_pub.eddsa_pub);
     TALER_amount_hton (&dr.amount_with_fee,
-		       &coin->amount_with_fee);
+                       &coin->amount_with_fee);
     if (GNUNET_SYSERR ==
-	TALER_amount_subtract (&fee,
-			       &coin->amount_with_fee,
-			       &coin->amount_without_fee))
+        TALER_amount_subtract (&fee,
+                               &coin->amount_with_fee,
+                               &coin->amount_without_fee))
     {
       /* Integer underflow, fee larger than total amount?
-	 This should not happen (client violated API!) */
+         This should not happen (client violated API!) */
       GNUNET_break (0);
       return NULL;
     }
     TALER_amount_hton (&dr.deposit_fee,
-		       &fee);
+                       &fee);
     {
       TALER_LOG_DEBUG ("... amount_with_fee was %s\n",
                        TALER_amount2s (&coin->amount_with_fee));
@@ -697,9 +765,10 @@ prepare_pay_generic (struct GNUNET_CURL_Context *ctx,
                        TALER_amount2s (&fee));
     }
 
-    GNUNET_CRYPTO_eddsa_sign (&coin->coin_priv.eddsa_priv,
-			      &dr.purpose,
-			      &p->coin_sig.eddsa_signature);
+    GNUNET_assert (GNUNET_OK ==
+                   GNUNET_CRYPTO_eddsa_sign (&coin->coin_priv.eddsa_priv,
+                                             &dr.purpose,
+                                             &p->coin_sig.eddsa_signature));
     p->denom_pub = coin->denom_pub;
     p->denom_sig = coin->denom_sig;
     p->denom_value = coin->denom_value;
@@ -710,16 +779,16 @@ prepare_pay_generic (struct GNUNET_CURL_Context *ctx,
     p->exchange_url = coin->exchange_url;
   }
   return request_pay_generic (ctx,
-			      merchant_url,
-			      merchant_pub,
-			      order_id,
-			      num_coins,
-			      pc,
-			      mode,
-			      pay_cb,
-			      pay_cb_cls,
-			      abort_cb,
-			      abort_cb_cls);
+                              merchant_url,
+                              merchant_pub,
+                              order_id,
+                              num_coins,
+                              pc,
+                              mode,
+                              pay_cb,
+                              pay_cb_cls,
+                              abort_cb,
+                              abort_cb_cls);
 }
 
 
@@ -731,7 +800,6 @@ prepare_pay_generic (struct GNUNET_CURL_Context *ctx,
  *
  * @param ctx the execution loop context
  * @param merchant_url base URL of the merchant's backend
- * @param instance which merchant instance will receive this payment
  * @param h_contract_terms hashcode of the proposal being paid
  * @param amount total value of the contract to be paid to the merchant
  * @param max_fee maximum fee covered by the merchant (according to the contract)
@@ -750,11 +818,10 @@ prepare_pay_generic (struct GNUNET_CURL_Context *ctx,
  */
 struct TALER_MERCHANT_Pay *
 TALER_MERCHANT_pay_wallet (struct GNUNET_CURL_Context *ctx,
-			   const char *merchant_url,
-			   const char *instance,
+                           const char *merchant_url,
                            const struct GNUNET_HashCode *h_contract_terms,
-			   const struct TALER_Amount *amount,
-			   const struct TALER_Amount *max_fee,
+                           const struct TALER_Amount *amount,
+                           const struct TALER_Amount *max_fee,
                            const struct TALER_MerchantPublicKeyP *merchant_pub,
                            const struct TALER_MerchantSignatureP *merchant_sig,
                            struct GNUNET_TIME_Absolute timestamp,
@@ -768,25 +835,24 @@ TALER_MERCHANT_pay_wallet (struct GNUNET_CURL_Context *ctx,
                            void *pay_cb_cls)
 {
   return prepare_pay_generic (ctx,
-			      merchant_url,
-			      instance,
-			      h_contract_terms,
-			      amount,
-			      max_fee,
-			      merchant_pub,
-			      merchant_sig,
-			      timestamp,
-			      refund_deadline,
-			      pay_deadline,
-			      h_wire,
-			      order_id,
-			      num_coins,
-			      coins,
-			      "pay",
-			      pay_cb,
-			      pay_cb_cls,
-			      NULL,
-			      NULL);
+                              merchant_url,
+                              h_contract_terms,
+                              amount,
+                              max_fee,
+                              merchant_pub,
+                              merchant_sig,
+                              timestamp,
+                              refund_deadline,
+                              pay_deadline,
+                              h_wire,
+                              order_id,
+                              num_coins,
+                              coins,
+                              "pay",
+                              pay_cb,
+                              pay_cb_cls,
+                              NULL,
+                              NULL);
 }
 
 
@@ -796,7 +862,6 @@ TALER_MERCHANT_pay_wallet (struct GNUNET_CURL_Context *ctx,
  *
  * @param ctx execution context
  * @param merchant_url base URL of the merchant
- * @param instance which merchant instance will receive this payment
  * @param h_wire hash of the merchant’s account details
  * @param h_contract hash of the contact of the merchant with the customer
  * @param transaction_id transaction id for the transaction between merchant and customer
@@ -816,45 +881,45 @@ TALER_MERCHANT_pay_wallet (struct GNUNET_CURL_Context *ctx,
  */
 struct TALER_MERCHANT_Pay *
 TALER_MERCHANT_pay_abort (struct GNUNET_CURL_Context *ctx,
-			  const char *merchant_url,
-			  const char *instance,
-			  const struct GNUNET_HashCode *h_contract,
-			  const struct TALER_Amount *amount,
-			  const struct TALER_Amount *max_fee,
-			  const struct TALER_MerchantPublicKeyP *merchant_pub,
-			  const struct TALER_MerchantSignatureP *merchant_sig,
-			  struct GNUNET_TIME_Absolute timestamp,
-			  struct GNUNET_TIME_Absolute refund_deadline,
-			  struct GNUNET_TIME_Absolute pay_deadline,
-			  const struct GNUNET_HashCode *h_wire,
-			  const char *order_id,
-			  unsigned int num_coins,
-			  const struct TALER_MERCHANT_PayCoin *coins,
-			  TALER_MERCHANT_PayRefundCallback payref_cb,
-			  void *payref_cb_cls)
+                          const char *merchant_url,
+                          const struct GNUNET_HashCode *h_contract,
+                          const struct TALER_Amount *amount,
+                          const struct TALER_Amount *max_fee,
+                          const struct TALER_MerchantPublicKeyP *merchant_pub,
+                          const struct TALER_MerchantSignatureP *merchant_sig,
+                          struct GNUNET_TIME_Absolute timestamp,
+                          struct GNUNET_TIME_Absolute refund_deadline,
+                          struct GNUNET_TIME_Absolute pay_deadline,
+                          const struct GNUNET_HashCode *h_wire,
+                          const char *order_id,
+                          unsigned int num_coins,
+                          const struct TALER_MERCHANT_PayCoin *coins,
+                          TALER_MERCHANT_PayRefundCallback payref_cb,
+                          void *payref_cb_cls)
 {
   struct TALER_MERCHANT_Pay *ph;
 
   ph = prepare_pay_generic (ctx,
-			    merchant_url,
-			    instance,
-			    h_contract,
-			    amount,
-			    max_fee,
-			    merchant_pub,
-			    merchant_sig,
-			    timestamp,
-			    refund_deadline,
-			    pay_deadline,
-			    h_wire,
-			    order_id,
-			    num_coins,
-			    coins,
-			    "abort-refund",
-			    NULL,
-			    NULL,
-			    payref_cb,
-			    payref_cb_cls);
+                            merchant_url,
+                            h_contract,
+                            amount,
+                            max_fee,
+                            merchant_pub,
+                            merchant_sig,
+                            timestamp,
+                            refund_deadline,
+                            pay_deadline,
+                            h_wire,
+                            order_id,
+                            num_coins,
+                            coins,
+                            "abort-refund",
+                            NULL,
+                            NULL,
+                            payref_cb,
+                            payref_cb_cls);
+  if (NULL == ph)
+    return NULL;
   ph->h_contract_terms = *h_contract;
   return ph;
 }
@@ -877,8 +942,9 @@ TALER_MERCHANT_pay_abort (struct GNUNET_CURL_Context *ctx,
  */
 struct TALER_MERCHANT_Pay *
 TALER_MERCHANT_pay_frontend (struct GNUNET_CURL_Context *ctx,
-			     const char *merchant_url,
-                             const struct TALER_MerchantPublicKeyP *merchant_pub,
+                             const char *merchant_url,
+                             const struct
+                             TALER_MerchantPublicKeyP *merchant_pub,
                              const char *order_id,
                              unsigned int num_coins,
                              const struct TALER_MERCHANT_PaidCoin *coins,
@@ -886,16 +952,16 @@ TALER_MERCHANT_pay_frontend (struct GNUNET_CURL_Context *ctx,
                              void *pay_cb_cls)
 {
   return request_pay_generic (ctx,
-			      merchant_url,
-			      merchant_pub,
-			      order_id,
-			      num_coins,
-			      coins,
-			      "pay",
-			      pay_cb,
-			      pay_cb_cls,
-			      NULL,
-			      NULL);
+                              merchant_url,
+                              merchant_pub,
+                              order_id,
+                              num_coins,
+                              coins,
+                              "pay",
+                              pay_cb,
+                              pay_cb_cls,
+                              NULL,
+                              NULL);
 }
 
 
@@ -913,9 +979,9 @@ TALER_MERCHANT_pay_cancel (struct TALER_MERCHANT_Pay *pay)
     GNUNET_CURL_job_cancel (pay->job);
     pay->job = NULL;
   }
+  TALER_curl_easy_post_finished (&pay->post_ctx);
   GNUNET_free (pay->coins);
   GNUNET_free (pay->url);
-  GNUNET_free (pay->json_enc);
   GNUNET_free (pay);
 }
 
