@@ -1,19 +1,22 @@
 /*
   This file is part of TALER
-  Copyright (C) 2014-2017 GNUnet e.V. and INRIA
+  Copyright (C) 2014-2020 Taler Systems SA
 
-  TALER is free software; you can redistribute it and/or modify it under the
-  terms of the GNU Lesser General Public License as published by the Free Software
-  Foundation; either version 2.1, or (at your option) any later version.
+  TALER is free software; you can redistribute it and/or modify
+  it under the terms of the GNU Lesser General Public License as
+  published by the Free Software Foundation; either version 2.1,
+  or (at your option) any later version.
 
-  TALER is distributed in the hope that it will be useful, but WITHOUT ANY
-  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-  A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.
+  TALER is distributed in the hope that it will be useful, but
+  WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU Lesser General Public License for more details.
 
-  You should have received a copy of the GNU Lesser General Public License along with
-  TALER; see the file COPYING.LGPL.  If not, see
-  <http://www.gnu.org/licenses/>
+  You should have received a copy of the GNU Lesser General Public
+  License along with TALER; see the file COPYING.LGPL.  If not,
+  see <http://www.gnu.org/licenses/>
 */
+
 /**
  * @file lib/merchant_api_proposal.c
  * @brief Implementation of the /proposal POST and GET
@@ -29,6 +32,7 @@
 #include "taler_merchant_service.h"
 #include <taler/taler_json_lib.h>
 #include <taler/taler_signatures.h>
+#include <taler/taler_curl_lib.h>
 
 
 /**
@@ -42,11 +46,6 @@ struct TALER_MERCHANT_ProposalOperation
    */
   char *url;
 
-  /**
-   * JSON encoding of the request to POST.
-   */
-  char *json_enc;
-
   /**
    * Handle for the request.
    */
@@ -66,6 +65,11 @@ struct TALER_MERCHANT_ProposalOperation
    * Reference to the execution context.
    */
   struct GNUNET_CURL_Context *ctx;
+
+  /**
+   * Minor context that holds body and headers.
+   */
+  struct TALER_CURL_PostContext post_ctx;
 };
 
 /**
@@ -122,14 +126,16 @@ struct TALER_MERCHANT_ProposalLookupOperation
 static void
 handle_proposal_finished (void *cls,
                           long response_code,
-                          const json_t *json)
+                          const void *response)
 {
   struct TALER_MERCHANT_ProposalOperation *po = cls;
   const char *order_id;
+  const json_t *json = response;
+  enum TALER_ErrorCode ec;
   struct GNUNET_JSON_Specification spec[] = {
     GNUNET_JSON_spec_string ("order_id",
                              &order_id),
-    GNUNET_JSON_spec_end()
+    GNUNET_JSON_spec_end ()
   };
 
   po->job = NULL;
@@ -152,28 +158,40 @@ handle_proposal_finished (void *cls,
       {
         GNUNET_break_op (0);
         response_code = 0;
+        ec = TALER_JSON_get_error_code (json);
         break;
       }
+      else
+      {
+        ec = TALER_EC_NONE;
+      }
     }
     break;
   case MHD_HTTP_BAD_REQUEST:
-      /* This should never happen, either us or the merchant is buggy
-         (or API version conflict); just pass JSON reply to the application */
+    ec = TALER_JSON_get_error_code (json);
+    /* This should never happen, either us or
+       the merchant is buggy (or API version conflict);
+       just pass JSON reply to the application */
+    break;
+  case MHD_HTTP_CONFLICT:
+    ec = TALER_JSON_get_error_code (json);
     break;
   case MHD_HTTP_FORBIDDEN:
-    break;
-  case MHD_HTTP_UNAUTHORIZED:
-    /* Nothing really to verify, merchant says one of the signatures is
-       invalid; as we checked them, this should never happen, we
-       should pass the JSON reply to the application */
+    /* Nothing really to verify, merchant says one
+       of the signatures is invalid; as we checked them,
+       this should never happen, we should pass the JSON
+       reply to the application */
+    ec = TALER_JSON_get_error_code (json);
     break;
   case MHD_HTTP_NOT_FOUND:
     /* Nothing really to verify, this should never
        happen, we should pass the JSON reply to the application */
+    ec = TALER_JSON_get_error_code (json);
     break;
   case MHD_HTTP_INTERNAL_SERVER_ERROR:
-    /* Server had an internal issue; we should retry, but this API
-       leaves this to the application */
+    /* Server had an internal issue; we should retry,
+       but this API leaves this to the application */
+    ec = TALER_JSON_get_error_code (json);
     break;
   default:
     /* unexpected response code */
@@ -182,10 +200,12 @@ handle_proposal_finished (void *cls,
                 (unsigned int) response_code);
     GNUNET_break (0);
     response_code = 0;
+    ec = TALER_JSON_get_error_code (json);
+    break;
   }
   po->cb (po->cb_cls,
           response_code,
-	  TALER_JSON_get_error_code (json),
+          ec,
           json,
           order_id);
   GNUNET_JSON_parse_free (spec);
@@ -198,10 +218,10 @@ handle_proposal_finished (void *cls,
  *
  * @param ctx execution context
  * @param backend_url URL of the backend
- * @param order basic information about this purchase, to be extended by the
- * backend
- * @param proposal_cb the callback to call when a reply for this request is
- * available
+ * @param order basic information about this purchase,
+ *        to be extended by the backend
+ * @param proposal_cb the callback to call when a reply
+ *        for this request is available
  * @param proposal_cb_cls closure for @a proposal_cb
  * @return a handle for this request, NULL on error
  */
@@ -220,36 +240,30 @@ TALER_MERCHANT_order_put (struct GNUNET_CURL_Context *ctx,
   po->ctx = ctx;
   po->cb = proposal_cb;
   po->cb_cls = proposal_cb_cls;
-  po->url = TALER_url_join (backend_url, "/order", NULL);
+  po->url = TALER_url_join (backend_url, "order", NULL);
   req = json_pack ("{s:O}",
                    "order", (json_t *) order);
   eh = curl_easy_init ();
-  po->json_enc = json_dumps (req,
-                             JSON_COMPACT);
-  json_decref (req);
-  if (NULL == po->json_enc)
+  if (GNUNET_OK != TALER_curl_easy_post (&po->post_ctx,
+                                         eh,
+                                         req))
   {
     GNUNET_break (0);
+    json_decref (req);
     GNUNET_free (po);
     return NULL;
   }
+  json_decref (req);
+
   GNUNET_assert (CURLE_OK ==
                  curl_easy_setopt (eh,
                                    CURLOPT_URL,
                                    po->url));
-  GNUNET_assert (CURLE_OK ==
-                 curl_easy_setopt (eh,
-                                   CURLOPT_POSTFIELDS,
-                                   po->json_enc));
-  GNUNET_assert (CURLE_OK ==
-                 curl_easy_setopt (eh,
-                                   CURLOPT_POSTFIELDSIZE,
-                                   strlen (po->json_enc)));
-  po->job = GNUNET_CURL_job_add (ctx,
-                                 eh,
-                                 GNUNET_YES,
-                                 &handle_proposal_finished,
-                                 po);
+  po->job = GNUNET_CURL_job_add2 (ctx,
+                                  eh,
+                                  po->post_ctx.headers,
+                                  &handle_proposal_finished,
+                                  po);
   return po;
 }
 
@@ -264,19 +278,21 @@ TALER_MERCHANT_order_put (struct GNUNET_CURL_Context *ctx,
 static void
 handle_proposal_lookup_finished (void *cls,
                                  long response_code,
-                                 const json_t *json)
+                                 const void *response)
 {
   struct TALER_MERCHANT_ProposalLookupOperation *plo = cls;
   json_t *contract_terms;
   struct TALER_MerchantSignatureP sig;
   struct GNUNET_HashCode hash;
+  const json_t *json = response;
   struct GNUNET_JSON_Specification spec[] = {
     GNUNET_JSON_spec_json ("contract_terms",
                            &contract_terms),
     GNUNET_JSON_spec_fixed_auto ("sig",
                                  &sig),
-    GNUNET_JSON_spec_end()
+    GNUNET_JSON_spec_end ()
   };
+
   plo->job = NULL;
   if (MHD_HTTP_OK != response_code)
   {
@@ -284,7 +300,6 @@ handle_proposal_lookup_finished (void *cls,
                 "Proposal lookup failed with HTTP status code %u\n",
                 (unsigned int) response_code);
     GNUNET_break (0);
-
     plo->cb (plo->cb_cls,
              response_code,
              json,
@@ -309,7 +324,6 @@ handle_proposal_lookup_finished (void *cls,
              NULL,
              NULL,
              NULL);
-
     TALER_MERCHANT_proposal_lookup_cancel (plo);
     return;
   }
@@ -362,57 +376,56 @@ struct TALER_MERCHANT_ProposalLookupOperation *
 TALER_MERCHANT_proposal_lookup (struct GNUNET_CURL_Context *ctx,
                                 const char *backend_url,
                                 const char *order_id,
-                                const char *instance,
-                                const struct GNUNET_CRYPTO_EddsaPublicKey *nonce,
-                                TALER_MERCHANT_ProposalLookupOperationCallback plo_cb,
+                                const struct
+                                GNUNET_CRYPTO_EddsaPublicKey *nonce,
+                                TALER_MERCHANT_ProposalLookupOperationCallback
+                                plo_cb,
                                 void *plo_cb_cls)
 {
   struct TALER_MERCHANT_ProposalLookupOperation *plo;
   CURL *eh;
-  char *base;
+  char *nonce_str = NULL;
 
   plo = GNUNET_new (struct TALER_MERCHANT_ProposalLookupOperation);
   plo->ctx = ctx;
   plo->cb = plo_cb;
   plo->cb_cls = plo_cb_cls;
-  base = TALER_url_join (backend_url, "/public/proposal", NULL);
   if (NULL != nonce)
   {
-    char *nonce_str;
     plo->has_nonce = GNUNET_YES;
     plo->nonce = *nonce;
-    nonce_str = GNUNET_STRINGS_data_to_string_alloc (nonce, sizeof (struct GNUNET_CRYPTO_EddsaPublicKey));
-    GNUNET_assert (NULL != nonce_str);
-    GNUNET_asprintf (&plo->url,
-                     "%s?order_id=%s&instance=%s&nonce=%s",
-                     base,
-                     order_id,
-                     instance,
-                     nonce_str);
-    GNUNET_free (nonce_str);
+    nonce_str = GNUNET_STRINGS_data_to_string_alloc (nonce, sizeof (struct
+                                                                    GNUNET_CRYPTO_EddsaPublicKey));
   }
-  else
+  plo->url = TALER_url_join (backend_url, "proposal",
+                             "order_id",
+                             order_id,
+                             "nonce",
+                             nonce_str,
+                             NULL);
+  GNUNET_free_non_null (nonce_str);
+  if (NULL == plo->url)
   {
-    GNUNET_asprintf (&plo->url,
-                     "%s?order_id=%s&instance=%s",
-                     base,
-                     order_id,
-                     instance);
+    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                "Could not construct request URL.\n");
+    GNUNET_free (plo);
+    return NULL;
   }
-  GNUNET_free (base);
+  GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+              "looking up proposal from %s\n",
+              plo->url);
   eh = curl_easy_init ();
   if (CURLE_OK != curl_easy_setopt (eh,
                                     CURLOPT_URL,
                                     plo->url))
   {
     GNUNET_break (0);
+    curl_easy_cleanup (eh);
+    GNUNET_free (plo->url);
+    GNUNET_free (plo);
     return NULL;
   }
 
-  GNUNET_log (GNUNET_ERROR_TYPE_INFO,
-              "looking up proposal from %s\n",
-              plo->url);
-
   if (NULL == (plo->job = GNUNET_CURL_job_add (ctx,
                                                eh,
                                                GNUNET_YES,
@@ -420,6 +433,8 @@ TALER_MERCHANT_proposal_lookup (struct GNUNET_CURL_Context *ctx,
                                                plo)))
   {
     GNUNET_break (0);
+    GNUNET_free (plo->url);
+    GNUNET_free (plo);
     return NULL;
   }
   return plo;
@@ -433,7 +448,8 @@ TALER_MERCHANT_proposal_lookup (struct GNUNET_CURL_Context *ctx,
  * @param po the proposal operation request handle
  */
 void
-TALER_MERCHANT_proposal_cancel (struct TALER_MERCHANT_ProposalOperation *po)
+TALER_MERCHANT_proposal_cancel
+  (struct TALER_MERCHANT_ProposalOperation *po)
 {
   if (NULL != po->job)
   {
@@ -441,7 +457,7 @@ TALER_MERCHANT_proposal_cancel (struct TALER_MERCHANT_ProposalOperation *po)
     po->job = NULL;
   }
   GNUNET_free (po->url);
-  GNUNET_free (po->json_enc);
+  TALER_curl_easy_post_finished (&po->post_ctx);
   GNUNET_free (po);
 }
 
@@ -452,7 +468,8 @@ TALER_MERCHANT_proposal_cancel (struct TALER_MERCHANT_ProposalOperation *po)
  * @param plo handle to the request to be canceled
  */
 void
-TALER_MERCHANT_proposal_lookup_cancel (struct TALER_MERCHANT_ProposalLookupOperation *plo)
+TALER_MERCHANT_proposal_lookup_cancel
+  (struct TALER_MERCHANT_ProposalLookupOperation *plo)
 {
   if (NULL != plo->job)
   {
@@ -463,4 +480,5 @@ TALER_MERCHANT_proposal_lookup_cancel (struct TALER_MERCHANT_ProposalLookupOpera
   GNUNET_free (plo);
 }
 
+
 /* end of merchant_api_contract.c */
