@@ -76,9 +76,10 @@ struct TALER_MERCHANT_TrackTransactionHandle
 static void
 handle_track_transaction_finished (void *cls,
                                    long response_code,
-                                   const json_t *json)
+                                   const void *response)
 {
   struct TALER_MERCHANT_TrackTransactionHandle *tdo = cls;
+  const json_t *json = response;
 
   tdo->job = NULL;
   switch (response_code)
@@ -122,7 +123,7 @@ handle_track_transaction_finished (void *cls,
   }
   tdo->cb (tdo->cb_cls,
            response_code,
-	   TALER_JSON_get_error_code (json),
+           TALER_JSON_get_error_code (json),
            json);
   TALER_MERCHANT_track_transaction_cancel (tdo);
 }
@@ -133,7 +134,6 @@ handle_track_transaction_finished (void *cls,
  *
  * @param ctx execution context
  * @param backend_url base URL of the backend
- * @param instance which merchant instance is going to be tracked
  * @param order_id order id pointing to the transaction being tracked
  * @param track_transaction_cb the callback to call when a reply for this request is available
  * @param track_transaction_cb_cls closure for @a track_transaction_cb
@@ -142,26 +142,29 @@ handle_track_transaction_finished (void *cls,
 struct TALER_MERCHANT_TrackTransactionHandle *
 TALER_MERCHANT_track_transaction (struct GNUNET_CURL_Context *ctx,
                                   const char *backend_url,
-                                  const char *instance,
                                   const char *order_id,
-                                  TALER_MERCHANT_TrackTransactionCallback track_transaction_cb,
+                                  TALER_MERCHANT_TrackTransactionCallback
+                                  track_transaction_cb,
                                   void *track_transaction_cb_cls)
 {
   struct TALER_MERCHANT_TrackTransactionHandle *tdo;
   CURL *eh;
-  char *base;
 
   tdo = GNUNET_new (struct TALER_MERCHANT_TrackTransactionHandle);
   tdo->ctx = ctx;
   tdo->cb = track_transaction_cb;
   tdo->cb_cls = track_transaction_cb_cls;
-  base = TALER_url_join (backend_url, "/track/transaction", NULL);
-  GNUNET_asprintf (&tdo->url,
-                   "%s?order_id=%s&instance=%s",
-                   base,
-                   order_id,
-                   instance);
-  GNUNET_free (base);
+  tdo->url = TALER_url_join (backend_url,
+                             "track/transaction",
+                             "order_id", order_id,
+                             NULL);
+  if (NULL == tdo->url)
+  {
+    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                "Could not construct request URL.\n");
+    GNUNET_free (tdo);
+    return NULL;
+  }
   GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
               "Requesting URL '%s'\n",
               tdo->url);
@@ -186,7 +189,9 @@ TALER_MERCHANT_track_transaction (struct GNUNET_CURL_Context *ctx,
  * @param tdo handle to the tracking operation being cancelled
  */
 void
-TALER_MERCHANT_track_transaction_cancel (struct TALER_MERCHANT_TrackTransactionHandle *tdo)
+TALER_MERCHANT_track_transaction_cancel (struct
+                                         TALER_MERCHANT_TrackTransactionHandle *
+                                         tdo)
 {
   if (NULL != tdo->job)
   {
@@ -197,4 +202,5 @@ TALER_MERCHANT_track_transaction_cancel (struct TALER_MERCHANT_TrackTransactionH
   GNUNET_free (tdo);
 }
 
+
 /* end of merchant_api_track_transaction.c */
