@@ -71,7 +71,7 @@ struct TALER_MERCHANT_TrackTransferHandle
  * callback.  If not, return an error code.
  *
  * This code is very similar to
- * exchange_api_track_transfer.c::check_track_transfer_response_ok.
+ * exchange_api_transfers_get.c::check_transfers_get_response_ok.
  * (Except we do not check the signature, as that was done by the
  * backend which we trust already.)
  * Any changes should likely be reflected there as well.
@@ -82,8 +82,9 @@ struct TALER_MERCHANT_TrackTransferHandle
  *         #GNUNET_SYSERR if the response was bogus
  */
 static int
-check_track_transfer_response_ok (struct TALER_MERCHANT_TrackTransferHandle *wdh,
-                                  const json_t *json)
+check_transfers_get_response_ok (struct
+                                 TALER_MERCHANT_TrackTransferHandle *wdh,
+                                 const json_t *json)
 {
   json_t *deposits;
   struct GNUNET_HashCode h_wire;
@@ -94,10 +95,10 @@ check_track_transfer_response_ok (struct TALER_MERCHANT_TrackTransferHandle *wdh
   struct GNUNET_JSON_Specification inner_spec[] = {
     TALER_JSON_spec_amount ("total", &total_amount),
     GNUNET_JSON_spec_fixed_auto ("merchant_pub", &merchant_pub),
-    GNUNET_JSON_spec_fixed_auto ("H_wire", &h_wire),
+    GNUNET_JSON_spec_fixed_auto ("h_wire", &h_wire),
     GNUNET_JSON_spec_json ("deposits_sums", &deposits),
     GNUNET_JSON_spec_fixed_auto ("exchange_pub", &exchange_pub),
-    GNUNET_JSON_spec_end()
+    GNUNET_JSON_spec_end ()
   };
 
   if (GNUNET_OK !=
@@ -112,7 +113,7 @@ check_track_transfer_response_ok (struct TALER_MERCHANT_TrackTransferHandle *wdh
   {
     struct TALER_MERCHANT_TrackTransferDetails details[num_details];
 
-    for (unsigned int i=0;i<num_details;i++)
+    for (unsigned int i = 0; i<num_details; i++)
     {
       struct TALER_MERCHANT_TrackTransferDetails *detail = &details[i];
       json_t *deposit = json_array_get (deposits, i);
@@ -120,7 +121,7 @@ check_track_transfer_response_ok (struct TALER_MERCHANT_TrackTransferHandle *wdh
         GNUNET_JSON_spec_string ("order_id", &detail->order_id),
         TALER_JSON_spec_amount ("deposit_value", &detail->deposit_value),
         TALER_JSON_spec_amount ("deposit_fee", &detail->deposit_fee),
-        GNUNET_JSON_spec_end()
+        GNUNET_JSON_spec_end ()
       };
 
       if (GNUNET_OK !=
@@ -135,7 +136,7 @@ check_track_transfer_response_ok (struct TALER_MERCHANT_TrackTransferHandle *wdh
     }
     wdh->cb (wdh->cb_cls,
              MHD_HTTP_OK,
-	     TALER_EC_NONE,
+             TALER_EC_NONE,
              &exchange_pub,
              json,
              &h_wire,
@@ -157,11 +158,12 @@ check_track_transfer_response_ok (struct TALER_MERCHANT_TrackTransferHandle *wdh
  * @param json response body, NULL if not in JSON
  */
 static void
-handle_track_transfer_finished (void *cls,
-                                long response_code,
-                                const json_t *json)
+handle_transfers_get_finished (void *cls,
+                               long response_code,
+                               const void *response)
 {
   struct TALER_MERCHANT_TrackTransferHandle *tdo = cls;
+  const json_t *json = response;
 
   tdo->job = NULL;
   switch (response_code)
@@ -170,8 +172,8 @@ handle_track_transfer_finished (void *cls,
     break;
   case MHD_HTTP_OK:
     if (GNUNET_OK ==
-        check_track_transfer_response_ok (tdo,
-                                          json))
+        check_transfers_get_response_ok (tdo,
+                                         json))
     {
       TALER_MERCHANT_track_transfer_cancel (tdo);
       return;
@@ -179,6 +181,9 @@ handle_track_transfer_finished (void *cls,
     GNUNET_break_op (0);
     response_code = 0;
     break;
+  case MHD_HTTP_FAILED_DEPENDENCY:
+    /* Not a reason to break execution.  */
+    break;
   case MHD_HTTP_NOT_FOUND:
     /* Nothing really to verify, this should never
        happen, we should pass the JSON reply to the application */
@@ -198,7 +203,7 @@ handle_track_transfer_finished (void *cls,
   }
   tdo->cb (tdo->cb_cls,
            response_code,
-	   TALER_JSON_get_error_code (json),
+           TALER_JSON_get_error_code (json),
            NULL,
            json,
            NULL,
@@ -214,7 +219,6 @@ handle_track_transfer_finished (void *cls,
  *
  * @param ctx execution context
  * @param backend_url base URL of the backend
- * @param instance which merchant instance is going to be tracked
  * @param wire_method wire method used for the wire transfer
  * @param wtid base32 string indicating a wtid
  * @param exchange_url base URL of the exchange in charge of returning the wanted information
@@ -225,11 +229,12 @@ handle_track_transfer_finished (void *cls,
 struct TALER_MERCHANT_TrackTransferHandle *
 TALER_MERCHANT_track_transfer (struct GNUNET_CURL_Context *ctx,
                                const char *backend_url,
-                               const char *instance,
-			       const char *wire_method,
-                               const struct TALER_WireTransferIdentifierRawP *wtid,
+                               const char *wire_method,
+                               const struct
+                               TALER_WireTransferIdentifierRawP *wtid,
                                const char *exchange_url,
-                               TALER_MERCHANT_TrackTransferCallback track_transfer_cb,
+                               TALER_MERCHANT_TrackTransferCallback
+                               track_transfer_cb,
                                void *track_transfer_cb_cls)
 {
   struct TALER_MERCHANT_TrackTransferHandle *tdo;
@@ -237,18 +242,25 @@ TALER_MERCHANT_track_transfer (struct GNUNET_CURL_Context *ctx,
   char *wtid_str;
 
   wtid_str = GNUNET_STRINGS_data_to_string_alloc (wtid,
-                                                  sizeof (struct TALER_WireTransferIdentifierRawP));
+                                                  sizeof (struct
+                                                          TALER_WireTransferIdentifierRawP));
   tdo = GNUNET_new (struct TALER_MERCHANT_TrackTransferHandle);
   tdo->ctx = ctx;
   tdo->cb = track_transfer_cb; // very last to be called
   tdo->cb_cls = track_transfer_cb_cls;
-  tdo->url = TALER_url_join (backend_url, "/track/transfer",
+  tdo->url = TALER_url_join (backend_url, "track/transfer",
                              "wtid", wtid_str,
                              "exchange", exchange_url,
-                             "instance", instance,
                              "wire_method", wire_method,
                              NULL);
   GNUNET_free (wtid_str);
+  if (NULL == tdo->url)
+  {
+    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                "Could not construct request URL.\n");
+    GNUNET_free (tdo);
+    return NULL;
+  }
   eh = curl_easy_init ();
   GNUNET_assert (CURLE_OK ==
                  curl_easy_setopt (eh,
@@ -257,7 +269,7 @@ TALER_MERCHANT_track_transfer (struct GNUNET_CURL_Context *ctx,
   tdo->job = GNUNET_CURL_job_add (ctx,
                                   eh,
                                   GNUNET_YES,
-                                  &handle_track_transfer_finished,
+                                  &handle_transfers_get_finished,
                                   tdo);
   return tdo;
 }
@@ -270,7 +282,8 @@ TALER_MERCHANT_track_transfer (struct GNUNET_CURL_Context *ctx,
  * @param tdo handle to the tracking operation being cancelled
  */
 void
-TALER_MERCHANT_track_transfer_cancel (struct TALER_MERCHANT_TrackTransferHandle *tdo)
+TALER_MERCHANT_track_transfer_cancel (struct
+                                      TALER_MERCHANT_TrackTransferHandle *tdo)
 {
   if (NULL != tdo->job)
   {
@@ -281,4 +294,5 @@ TALER_MERCHANT_track_transfer_cancel (struct TALER_MERCHANT_TrackTransferHandle
   GNUNET_free (tdo);
 }
 
+
 /* end of merchant_api_track_transfer.c */
